#include "t3f/t3f.h"
#include "t3f/sound.h"
#include "t3f/music.h"
#include "defines.h"
#include "globals.h"
#include "init.h"

bool vgolf_supports_2x_window(void)
{
	ALLEGRO_MONITOR_INFO info;
	al_get_monitor_info(0, &info);
	if(info.x2 - info.x1 >= 1280 && info.y2 - info.y1 >= 960)
	{
		return true;
	}
	return false;
}

//save configuration to file
int save_config(char * filename)
{
	const ALLEGRO_FILE_INTERFACE * old_interface;
	ALLEGRO_FILE * out_file;
	int ret = 0;
	int i;

	old_interface = al_get_new_file_interface();
	al_set_standard_file_interface();

	out_file = t3f_open_file(t3f_data_path, filename, "wb");
	if(out_file)
	{
		//write header
		al_fputc(out_file, 'C');
		al_fputc(out_file, 'F');
		al_fputc(out_file, 'G');
		al_fputc(out_file, 14);

		for(i = 0; i < CONFIG_MAX; i++)
		{
			al_fwrite32le(out_file, vgolf_config[i]);
		}
		al_fwrite(out_file, vgolf_nick, 128);
		al_fwrite(out_file, vgolf_host_name, 128);

		al_fclose(out_file);
		ret = 1;
	}

	al_set_new_file_interface(old_interface);

	return ret;
}

void set_default_config_option(int option)
{
	switch(option)
	{
		case CONFIG_GFX_MODE:
		{
			if(vgolf_supports_2x_window())
			{
				vgolf_config[CONFIG_GFX_MODE] = 1;
			}
			else
			{
				vgolf_config[CONFIG_GFX_MODE] = 0;
			}
			if(vgolf_config[CONFIG_GFX_MODE] == 1)
			{
				set_gfx_mode();
			}
			break;
		}
/*		case CONFIG_COMMENTARY:
		{
			vgolf_config[CONFIG_COMMENTARY] = 0;
			break;
		} */
		case CONFIG_SFX_VOLUME:
		{
			vgolf_config[CONFIG_SFX_VOLUME] = 50;
			t3f_set_sound_volume(1.0);
			break;
		}
		case CONFIG_MUSIC_VOLUME:
		{
			vgolf_config[CONFIG_MUSIC_VOLUME] = 100;
			t3f_set_music_volume(1.0);
			break;
		}
		case CONFIG_COMMENTARY_VOLUME:
		{
			vgolf_config[CONFIG_COMMENTARY_VOLUME] = 100;
			break;
		}
		case CONFIG_UPLOAD:
		{
			vgolf_config[CONFIG_UPLOAD] = 1;
			break;
		}
		case CONFIG_INPUT_BUFFER:
		{
			vgolf_config[CONFIG_INPUT_BUFFER] = 4;
			break;
		}
		case CONFIG_DLC:
		{
			vgolf_config[CONFIG_DLC] = 0;
			break;
		}
		case CONFIG_LOW_BANDWIDTH:
		{
			vgolf_config[CONFIG_LOW_BANDWIDTH] = 0;
			break;
		}
		case CONFIG_HD_TEXTURES:
		{
			#ifdef T3F_ANDROID
				vgolf_config[CONFIG_HD_TEXTURES] = 0;
			#else
				vgolf_config[CONFIG_HD_TEXTURES] = 1;
			#endif
			break;
		}
		case CONFIG_DEAD_SPOT:
		{
			vgolf_config[CONFIG_DEAD_SPOT] = 10;
			break;
		}
	}
}

void check_config_option(int option)
{
	switch(option)
	{
		case CONFIG_GFX_MODE:
		{
			if(vgolf_config[CONFIG_GFX_MODE] < 0 || vgolf_config[CONFIG_GFX_MODE] > 2)
			{
				set_default_config_option(option);
			}
			break;
		}
		case CONFIG_COMMENTARY:
		{
/*			if(vgolf_config[CONFIG_COMMENTARY] < 0 || vgolf_config[CONFIG_COMMENTARY] > vgolf_commentaries.count)
			{
				set_default_config_option(option);
			} */
			break;
		}
		case CONFIG_SFX_VOLUME:
		{
			if(vgolf_config[CONFIG_SFX_VOLUME] < 0 || vgolf_config[CONFIG_SFX_VOLUME] > 100)
			{
				set_default_config_option(option);
			}
			else if(vgolf_config[CONFIG_SFX_VOLUME] % 5)
			{
				vgolf_config[CONFIG_SFX_VOLUME] -= vgolf_config[CONFIG_SFX_VOLUME] % 5;
			}
			break;
		}
		case CONFIG_MUSIC_VOLUME:
		{
			if(vgolf_config[CONFIG_MUSIC_VOLUME] < 0 || vgolf_config[CONFIG_MUSIC_VOLUME] > 100)
			{
				set_default_config_option(option);
			}
			else if(vgolf_config[CONFIG_MUSIC_VOLUME] % 5)
			{
				vgolf_config[CONFIG_MUSIC_VOLUME] -= vgolf_config[CONFIG_MUSIC_VOLUME] % 5;
			}
			break;
		}
		case CONFIG_COMMENTARY_VOLUME:
		{
			if(vgolf_config[CONFIG_COMMENTARY_VOLUME] < 0 || vgolf_config[CONFIG_COMMENTARY_VOLUME] > 100)
			{
				set_default_config_option(option);
			}
			else if(vgolf_config[CONFIG_COMMENTARY_VOLUME] % 5)
			{
				vgolf_config[CONFIG_COMMENTARY_VOLUME] -= vgolf_config[CONFIG_COMMENTARY_VOLUME] % 5;
			}
			break;
		}
		case CONFIG_UPLOAD:
		{
			if(vgolf_config[CONFIG_UPLOAD] < 0 || vgolf_config[CONFIG_UPLOAD] > 1)
			{
				set_default_config_option(option);
			}
			break;
		}
		case CONFIG_INPUT_BUFFER:
		{
			if(vgolf_config[CONFIG_INPUT_BUFFER] < 0 || vgolf_config[CONFIG_INPUT_BUFFER] > 16)
			{
				set_default_config_option(option);
			}
			break;
		}
		case CONFIG_DLC:
		{
			if(vgolf_config[CONFIG_DLC] < 0 || vgolf_config[CONFIG_DLC] > 1)
			{
				set_default_config_option(option);
			}
			break;
		}
		case CONFIG_LOW_BANDWIDTH:
		{
			if(vgolf_config[CONFIG_LOW_BANDWIDTH] < 0 || vgolf_config[CONFIG_LOW_BANDWIDTH] > 1)
			{
				set_default_config_option(option);
			}
			break;
		}
		case CONFIG_HD_TEXTURES:
		{
			if(vgolf_config[CONFIG_HD_TEXTURES] < 0 || vgolf_config[CONFIG_HD_TEXTURES] > 1)
			{
				set_default_config_option(option);
			}
			break;
		}
		case CONFIG_DEAD_SPOT:
		{
			if(vgolf_config[CONFIG_DEAD_SPOT] < 5 || vgolf_config[CONFIG_DEAD_SPOT] > 20)
			{
				set_default_config_option(option);
			}
			break;
		}
	}
}

//set a default configuration
void set_default_config(void)
{
	int i;

	for(i = 0; i < CONFIG_MAX; i++)
	{
		set_default_config_option(i);
	}
	strcpy(vgolf_nick, "Unnamed");
	strcpy(vgolf_host_name, "Unnamed Host");
}

//load saved configuration
int load_config(char * filename)
{
	const ALLEGRO_FILE_INTERFACE * old_interface;
	char temp_char[4];
	ALLEGRO_FILE * in_file;
	int ret = 0;
	int i;

	old_interface = al_get_new_file_interface();
	al_set_standard_file_interface();

	in_file = t3f_open_file(t3f_data_path, filename, "rb");
	if(in_file)
	{

		//read header
		temp_char[0] = al_fgetc(in_file);
		temp_char[1] = al_fgetc(in_file);
		temp_char[2] = al_fgetc(in_file);
		temp_char[3] = al_fgetc(in_file);
		if(temp_char[0] == 'C' && temp_char[1] == 'F' && temp_char[2] == 'G' && temp_char[3] == 14)
		{
			for(i = 0; i < CONFIG_MAX; i++)
			{
				vgolf_config[i] = al_fread32le(in_file);
				check_config_option(i);
			}
			al_fread(in_file, vgolf_nick, 128);
			for(i = 0; i < 128; i++)
			{
				if(vgolf_nick[i] == '\0')
				{
					break;
				}
			}
			if(i == 0 || i == 128)
			{
				strcpy(vgolf_nick, "Unnamed");
			}
			al_fread(in_file, vgolf_host_name, 128);
			for(i = 0; i < 128; i++)
			{
				if(vgolf_host_name[i] == '\0')
				{
					break;
				}
			}
			if(i == 0 || i == 128)
			{
				strcpy(vgolf_host_name, "Unnamed Host");
			}
			t3f_set_music_volume((float)vgolf_config[CONFIG_MUSIC_VOLUME] / 100.0);
			t3f_set_sound_volume((float)vgolf_config[CONFIG_SFX_VOLUME] / 100.0);
			ret = 1;
		}
		al_fclose(in_file);
	}

	al_set_new_file_interface(old_interface);

	return ret;
}
