#include "t3f/t3f.h"
#include "t3f/resource.h"
#include <string.h>
#include "globals.h"
#include "credits.h"
#include "message.h"
#include "vfont.h"

void credits_add_item(CREDITS * cp, char * text, T3F_FONT ** fp, int oy, ALLEGRO_COLOR color)
{
	strcpy(cp->item[cp->items].text, text);
	cp->item[cp->items].font = fp;
	cp->item[cp->items].oy = oy;
	cp->item[cp->items].color = color;
	cp->items++;
}

void credits_add_image(CREDITS * cp, const char * fn, int ox, int oy)
{
	cp->image[cp->images].fn = fn;
	cp->image[cp->images].ox = ox;
	cp->image[cp->images].oy = oy;
	cp->images++;
	if(cp->images >= MAX_CIMAGES)
	{
		cp->images = MAX_CIMAGES - 1;
	}
}

void credits_logic(CREDITS * cp)
{
	ALLEGRO_STATE old_state;
	ALLEGRO_TRANSFORM identity;
	ALLEGRO_BITMAP * bp;

	cp->y -= cp->vy;
	if(cp->y == cp->image[cp->current_image].oy)
	{
		cp->fade_dir = 16;
		cp->fade_level = 0;
		if(cp->image[cp->current_image].ox > 0)
		{
			cp->image[cp->current_image].bp = t3f_load_resource((void **)&cp->image[cp->current_image].bp, T3F_RESOURCE_TYPE_BITMAP, cp->image[cp->current_image].fn, 0, 0, 0);
			if(cp->image[cp->current_image].bp)
			{
				al_convert_mask_to_alpha(cp->image[cp->current_image].bp, al_get_pixel(cp->image[cp->current_image].bp, 0, 0));
			}
		}
		else
		{
			bp = t3f_load_resource((void **)&bp, T3F_RESOURCE_TYPE_BITMAP, cp->image[cp->current_image].fn, 0, 0, 0);
			if(bp)
			{
				al_convert_mask_to_alpha(bp, al_get_pixel(bp, 0, 0));
				al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM);
				cp->image[cp->current_image].bp = al_create_bitmap(640, 480);
				if(cp->image[cp->current_image].bp)
				{
					al_set_target_bitmap(cp->image[cp->current_image].bp);
					al_identity_transform(&identity);
					al_use_transform(&identity);
					al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
					al_draw_bitmap(bp, al_get_bitmap_width(cp->image[cp->current_image].bp) / 2 - al_get_bitmap_width(bp) / 2, al_get_bitmap_height(cp->image[cp->current_image].bp) / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2 - al_get_bitmap_height(bp) / 2, 0);
					int epx = 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Thank You For Playing") / 2;
					int epy = al_get_bitmap_height(cp->image[cp->current_image].bp) / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2 - al_get_bitmap_height(bp) / 2 + al_get_bitmap_height(bp);
					outline_text_shadow(vgolf_font[VGOLF_FONT_48], epx + 2, epy - 2, -4, 4, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "Thank You For Playing");
					al_restore_state(&old_state);
					t3f_destroy_resource(bp);
				}
			}
			cp->image[cp->current_image].ox = t3f_virtual_display_width / 2;
		}
	}
	cp->fade_level += cp->fade_dir;
	if(cp->fade_level < 0)
	{
		/* if resource destruction fails, destroy bitmap directly (some images are not resources) */
		if(!t3f_destroy_resource(cp->image[cp->current_image].bp))
		{
			al_destroy_bitmap(cp->image[cp->current_image].bp);
		}
		cp->image[cp->current_image].bp = NULL;
		cp->fade_dir = 0;
		cp->fade_level = 0;
		cp->current_image++;
		if(cp->current_image >= cp->images)
		{
			cp->current_image = cp->images - 1;
			cp->done = 1;
		}
	}
	if(cp->fade_level > 255)
	{
		cp->fade_dir = 0;
		cp->fade_level = 255;
		cp->image_ticks = 300;
	}
	if(cp->fade_level == 255)
	{
		cp->image_ticks--;
		if(cp->image_ticks <= 0)
		{
			cp->fade_dir = -16;
		}
	}
}

void credits_render(ALLEGRO_BITMAP * bp, CREDITS * cp)
{
	int i;

	if(cp->fade_level > 0)
	{
		al_draw_tinted_bitmap(cp->image[cp->current_image].bp, al_map_rgba(cp->fade_level, cp->fade_level, cp->fade_level, cp->fade_level), cp->image[cp->current_image].ox - al_get_bitmap_width(cp->image[cp->current_image].bp) / 2, t3f_virtual_display_height / 2 - al_get_bitmap_height(cp->image[cp->current_image].bp) / 2, 0);
	}
	for(i = 0; i < cp->items; i++)
	{
		outline_text_shadow(*cp->item[i].font, t3f_virtual_display_width / 2 + 2, cp->y + cp->item[i].oy - 2, -2, 2, cp->item[i].color, al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, cp->item[i].text);
//		textprintf_centre_ex(bp, cp->item[i].font, SCREEN_W / 2, cp->y + cp->item[i].oy, cp->item[i].color, -1, "%s", cp->item[i].text);
	}
}
