#include "t3f/t3f.h"
#include "fx.h"
#include "main.h"

ALLEGRO_COLOR fx_source_color;
ALLEGRO_COLOR fx_destination_color;
ALLEGRO_COLOR fx_temp_color;
int fx_fade_level;
int fx_fade_speed;
int fx_faded;
int fx_fade_finished;
void(*fx_fade_proc)();

int closest_match(int color, void * p1, void * p2)
{
	return 0;
}

void convert_palette(ALLEGRO_BITMAP * bp, void * p1, void * p2)
{
}

ALLEGRO_COLOR fx_fade_color(ALLEGRO_COLOR c1, ALLEGRO_COLOR c2, float amount)
{
	float r1, g1, b1, a1;
	float r2, g2, b2, a2;

	al_unmap_rgba_f(c1, &r1, &g1, &b1, &a1);
	al_unmap_rgba_f(c2, &r2, &g2, &b2, &a2);
	return al_map_rgba_f(r1 + (r2 - r1) * amount, g1 + (g2 - g1) * amount, b1 + (b2 - b1) * amount, a1 + (a2 - a1) * amount);
}

ALLEGRO_BITMAP * turn_bitmap(ALLEGRO_BITMAP * bp, int dir)
{
	ALLEGRO_BITMAP * newbp;

	newbp = al_create_bitmap(al_get_bitmap_height(bp), al_get_bitmap_width(bp));
	if(!newbp)
	{
		return bp;
	}
//	clear_bitmap(newbp);
	if(dir < 0)
	{
//		rotate_sprite(newbp, bp, 0, 0, itofix(-64));
	}
	else
	{
//		rotate_sprite(newbp, bp, 0, 0, itofix(64));
	}
	return newbp;
}

void fx_fade_start(ALLEGRO_COLOR c1, ALLEGRO_COLOR c2, int speed, void(*proc)())
{
	fx_fade_proc = proc;
	fx_fade_level = 0;
	fx_fade_speed = speed;
	fx_faded = 1;
	fx_fade_finished = 0;
	fx_source_color = c1;
	fx_temp_color = c1;
	fx_destination_color = c2;
}

void fx_fade_logic(void)
{
	bool finished = false;

	/* update if the fade level hasn't reached maximum */
	if(fx_fade_level < 64)
	{
		fx_fade_level += fx_fade_speed;
		fx_faded = 1;

		/* when we reach the maximum fade level, we want to ensure
		   the fx_fade_proc will be called next time we run the logic */
		if(fx_fade_level >= 64)
		{
			fx_fade_level = 63;
			finished = true;
		}
		fx_temp_color = fx_fade_color(fx_source_color, fx_destination_color, (float)fx_fade_level / 63.0);

		if(finished)
		{
			t3f_render(true);
			t3f_render(true);
			fx_fade_finished = 1;
			fx_fade_speed = 0;
			if(fx_fade_proc)
			{
				fx_fade_proc();
			}
		}
	}
}

void fx_fade_render(void)
{
	if(fx_faded)
	{
		fx_faded = 0;
	}
	if(!fx_fade_finished)
	{
		al_draw_filled_rectangle(0, 0, 640, 480, fx_temp_color);
	}
}

void fx_draw_3d_sprite(ALLEGRO_BITMAP * bp, ALLEGRO_BITMAP * sprite, int x, int y, int z, int ox, int oy)
{
	float space_x, space_y;

	space_x = al_get_bitmap_width(bp);
	space_y = al_get_bitmap_height(bp);

	/* upper left and bottom right points in 3d */
//	float obj_x[2], obj_y[2], obj_z[2];
	int obj_x[2], obj_y[2], obj_z[2];

	/* upper left and bottom right points in 2d */
//	float screen_x[2], screen_y[2];
	int screen_x[2], screen_y[2];
	int screen_w, screen_h;

	obj_x[0] = x - ox;
	obj_x[0] *= space_x;
	obj_y[0] = y - oy;
	obj_y[0] *= space_y;
	obj_x[1] = x + al_get_bitmap_width(sprite) - ox;
	obj_x[1] *= space_x;
	obj_y[1] = y + al_get_bitmap_height(sprite) - oy;
	obj_y[1] *= space_y;
	obj_z[0] = z + al_get_bitmap_width(bp);
	obj_z[1] = z + al_get_bitmap_height(bp);

	/* clip sprites at z = 0 */
	if(obj_z[0] > 0)
	{
		screen_x[0] = obj_x[0] / obj_z[0] + ox;
		screen_x[1] = obj_x[1] / obj_z[0] + ox;
		screen_y[0] = obj_y[0] / obj_z[1] + oy;
		screen_y[1] = obj_y[1] / obj_z[1] + oy;
		screen_w = screen_x[1] - screen_x[0];
		screen_h = screen_y[1] - screen_y[0];
		al_draw_scaled_bitmap(sprite, 0, 0, al_get_bitmap_width(sprite), al_get_bitmap_height(sprite), screen_x[0], screen_y[0], screen_w, screen_h, 0);
	}
}

FX_PARTICLE fx_particle[FX_MAX_PARTICLES];
void(*fx_particle_end_proc)();

int fx_particle_system_qsort_helper(const void * e1, const void * e2)
{
    FX_PARTICLE * thing1 = (FX_PARTICLE *)e1;
    FX_PARTICLE * thing2 = (FX_PARTICLE *)e2;

    if(thing1->z > thing2->z)
	{
        return -1;
    }
    if(thing1->z < thing2->z)
    {
        return 1;
    }

    // they are equal...
    return 0;
}

void fx_particle_system_init(void)
{
	int i;

	for(i = 0; i < FX_MAX_PARTICLES; i++)
	{
		fx_particle[i].active = 0;
	}
}

void fx_particle_system_logic(void)
{
	int i;
	int count = 0;
	int a = 0;

	for(i = 0; i < FX_MAX_PARTICLES; i++)
	{
		if(fx_particle[i].active)
		{
			if(fx_particle[i].logic_proc)
			{
				fx_particle[i].logic_proc(&fx_particle[i]);
			}
			count++;
			a = 1;
		}
	}
	if(a && count <= 0 && fx_particle_end_proc)
	{
		fx_particle_end_proc();
	}
}

void fx_particle_system_render(ALLEGRO_BITMAP * bp)
{
	int i;

    qsort(fx_particle, FX_MAX_PARTICLES, sizeof(FX_PARTICLE), fx_particle_system_qsort_helper);
	for(i = 0; i < FX_MAX_PARTICLES; i++)
	{
		if(fx_particle[i].active)
		{
			if(fx_particle[i].bp)
			{
				fx_draw_3d_sprite(bp, fx_particle[i].bp, fx_particle[i].x, fx_particle[i].y, fx_particle[i].z, al_get_bitmap_width(bp) / 2, al_get_bitmap_height(bp) / 2);
			}
		}
	}
}

int fx_particle_system_add_particle(ALLEGRO_BITMAP * bp, float x, float y, float z, float vx, float vy, float vz, void(*proc)(FX_PARTICLE * pp))
{
	int i;

	for(i = 0; i < FX_MAX_PARTICLES; i++)
	{
		if(!fx_particle[i].active)
		{
			fx_particle[i].x = x;
			fx_particle[i].y = y;
			fx_particle[i].z = z;
			fx_particle[i].vx = vx;
			fx_particle[i].vy = vy;
			fx_particle[i].vz = vz;
			fx_particle[i].bp = bp;
			fx_particle[i].logic_proc = proc;
			return 1;
			break;
		}
	}
	return 0;
}

void fx_particle_system_set_proc(void(*proc)())
{
	fx_particle_end_proc = proc;
}
