#include "structs.h"
#include "globals.h"

//hide the player huds to get a clear view at the playing area, or bring them back
void hide_player_huds(PLAYER pl[])
{
	switch(main_game_vars.player_huds_state)
	{
		//move player huds out of the way..
		case 1:
		{
			main_game_vars.player_huds_offset += 4;
			if(main_game_vars.player_huds_offset >= 68)
			{
				main_game_vars.player_huds_offset = 68;
				main_game_vars.player_huds_state = 2; // they're hidden now
			}
			break;
		}
		//bring player huds back into view..
		case 3:
		{
			main_game_vars.player_huds_offset -= 4;
			if(main_game_vars.player_huds_offset <= 0)
			{
				main_game_vars.player_huds_offset = 0;
				main_game_vars.player_huds_state = 0; // they're hidden now
			}
			break;
		}
	}
}

//draw this player's hud
void player_hud_display(int pl, HOLE * hole)
{
	char buf[32];
	ALLEGRO_COLOR tcol, wcol, tint;
	int dx, dy;

	/* find hud display position, taking offset into account */
	dx = game_player[pl].hud.x;
	dy = game_player[pl].hud.y;
	switch(game_player[pl].id)
	{
		case 0:
		{
			dx -= main_game_vars.player_huds_offset;
			dy -= main_game_vars.player_huds_offset;
			break;
		}
		case 1:
		{
			dx += main_game_vars.player_huds_offset;
			dy -= main_game_vars.player_huds_offset;
			break;
		}
		case 2:
		{
			dx -= main_game_vars.player_huds_offset;
			dy += main_game_vars.player_huds_offset;
			break;
		}
		case 3:
		{
			dx += main_game_vars.player_huds_offset;
			dy += main_game_vars.player_huds_offset;
			break;
		}
		case 4:
		{
			dx -= main_game_vars.player_huds_offset * 3;
			break;
		}
		case 5:
		{
			dx += main_game_vars.player_huds_offset * 3;
			break;
		}
		case 6:
		{
			dx -= main_game_vars.player_huds_offset * 3;
			break;
		}
		case 7:
		{
			dx += main_game_vars.player_huds_offset * 3;
			break;
		}
	}

	/* highlight current player */
	if(pl == main_game_vars.cur_player)
	{
		tcol = al_map_rgb(0, 255, 0);
		wcol = al_map_rgb(255, 255, 255);
		tint = al_map_rgba_f(1.0, 1.0, 1.0, 1.0);
	}
	else
	{
		tcol = al_map_rgb(0, 128, 0);
		wcol = al_map_rgb(128, 128, 128);
		tint = al_map_rgba_f(0.5, 0.5, 0.5, 1.0);
	}
	//print the current stroke tally

	al_hold_bitmap_drawing(true);
	outline_text_shadow(vgolf_font[VGOLF_FONT_32], dx + 58, dy, -2, 2, wcol, al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, vgolf_client_game->player[pl]->name);
	sprintf(buf, "Strokes: %i (%s)", game_player[pl].cur_strokes, game_player[pl].scorecard.score_text);
	outline_text_shadow(vgolf_font[VGOLF_FONT_32], dx + 58, dy + 24, -2, 2, tcol, al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, buf);
	al_hold_bitmap_drawing(false);

	//check to see if the char pic overlays any important stuff (hole, ball, etc)
	t3f_draw_animation(*game_player[pl].pic[game_player[pl].pic_mode], tint, 0, dx - 5, dy - 6, 0, 0);
}

//go through and display all player huds..
void player_hud_display_all(PLAYER pl[], int num_players, HOLE * hole)
{
	int p;

	for(p = 0; p < VGOLF_MAX_PLAYERS; p++)
	{
		if(vgolf_player_list[p])
		{
			player_hud_display(p, hole);
		}
	}
}

//display misc. hole info
void hole_hud_display(HOLE * hole, int mode)
{
	char buf[32];
	//draw par score
	sprintf(buf, "Par %i", hole->par_score);
	outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], t3f_virtual_display_width / 2, t3f_virtual_display_height - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) - 4, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
}
