#include "structs.h"
#include "globals.h"
#include "menu_proc.h"
#include "core/legacy_course.h"

//start the seg-out sequence
void vgolf_start_scoreboard_segout(void)
{
	main_game_vars.game_state = main_game_vars.game_state_old = GAMESTATE_SCOREBOARD_SEGOUT;
}

//get key input on the scoreboard screen
void get_scoreboard_keys(VGOLF_COURSE * cr, PLAYER pl[], int num_players, int cur_hole)
{
	if(cr->scoreboard.scoreboard_seg_count >= cr->scoreboard.num_scoreboard_segs + 1)
	{
		hyperlink_click_logic(scoreboard_page);
		hyperlink_page_logic(scoreboard_page);
	}
}

//seg-out is done, goto next game state
void finish_scoreboard_segout(void)
{
	//since we're doing the seg-out sequence on the victory screen also, do a check first
	//before going to the "start hole" state
	if(!(main_game_vars.game_state > GAMESTATE_COURSE_COMPLETED_1P && main_game_vars.game_state < GAMESTATE_COIN_COLLECT_FINISHED))
	{
		main_game_vars.game_state = GAMESTATE_START_HOLE;
	}
}

//handles logic related to the scoreboard screen seg'ing out
void course_scoreboard_segout_logic(VGOLF_COURSE * cr)
{
	//increment segment counter until hits the end, then decrease the # of segments to show
	cr->scoreboard.scoreboard_seg_counter -= 16;
	if(cr->scoreboard.scoreboard_seg_counter <= 0)
	{
		cr->scoreboard.scoreboard_seg_counter = cr->scoreboard.scoreboard_seg_counter_end;
		cr->scoreboard.scoreboard_seg_count--;
		if(cr->scoreboard.scoreboard_seg_count <= 0)
		{
			//all segments are removed now, finish seg-out sequence
			finish_scoreboard_segout();
		}
	}
}

//seg-in is done, goto next game state
void finish_scoreboard_segin(void)
{
	int chosen;

	/* change states and render to disguise loading time */
	main_game_vars.game_state = GAMESTATE_SCOREBOARD_SEGMID;
	t3f_render(true);

	chosen = vgolf_find_course(vgolf_selected_course);
	if(main_game_vars.cur_hole >= 0)
	{
		vgolf_destroy_hole_graphics(&game_course->hole[main_game_vars.cur_hole - 1]);
	}
	if(main_game_vars.cur_hole + 1 < game_course->num_holes)
	{
		if(game_course->legacy)
		{
			vgolf_load_legacy_hole_graphics(&game_course->hole[main_game_vars.cur_hole + 1], vgolf_courses.item[chosen].filename, vgolf_config[CONFIG_HD_TEXTURES], vgolf_courses.item[chosen].fi);
		}
		else
		{
			vgolf_load_hole_graphics(&game_course->hole[main_game_vars.cur_hole + 1], vgolf_courses.item[chosen].filename, vgolf_config[CONFIG_HD_TEXTURES], vgolf_courses.item[chosen].fi);
		}
	}
	fake_page_hidden = false;
}

//handles logic related to the scoreboard screen seg'ing in
void course_scoreboard_segin_logic(VGOLF_COURSE * cr)
{
	//increment segment counter until hits the end, then increase the # of segments to show
	cr->scoreboard.scoreboard_seg_counter += 16;
	if(cr->scoreboard.scoreboard_seg_counter >= cr->scoreboard.scoreboard_seg_counter_end)
	{
		cr->scoreboard.scoreboard_seg_counter = 0;
		cr->scoreboard.scoreboard_seg_count++;
		if(cr->scoreboard.scoreboard_seg_count >= cr->scoreboard.num_scoreboard_segs + 1)
		{
			//all segments are visible now, finish the seg-in sequence
			finish_scoreboard_segin();
		}
	}
}


//handles main logic related to the scoreboard screen
void course_scoreboard_main_logic(VGOLF_COURSE * cr, PLAYER pl[], int num_players, int cur_hole)
{
	get_scoreboard_keys(cr, pl, num_players, cur_hole);
}

void course_scoreboard_render_card(float x, float y, int max_players, bool pars)
{
	int i, h, o;

	h = al_get_bitmap_height(scorecard_pic) - 1;

	al_draw_tinted_bitmap(scorecard_pic, al_map_rgb(0, 0, 0), x - 2, y + 2, 0);
	al_draw_tinted_bitmap(scorecard_pic, al_map_rgb(0, 192, 0), x, y, 0);
	if(pars)
	{
		al_draw_tinted_bitmap(scorecard_pic, al_map_rgb(0, 0, 0), x - 2, y + h * 1 + 2, 0);
		al_draw_tinted_bitmap(scorecard_pic, al_map_rgb(192, 0, 0), x, y + h * 1, 0);
		o = 0;
	}
	else
	{
		o = 1;
	}
	for(i = 0; i < max_players; i++)
	{
		al_draw_tinted_bitmap(scorecard_pic, al_map_rgb(0, 0, 0), x - 2, y + h * (2 + i - o) + 2, 0);
		al_draw_tinted_bitmap(scorecard_pic, al_map_rgb(255, 255, 255), x, y + h * (2 + i - o), 0);
	}
}

void course_scoreboard_render_header(float x, float y, int num_holes)
{
	int i;
	char buf[256];

	outline_text_shadow(vgolf_font[VGOLF_FONT_32], x + 6, y + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "Hole");
	for(i = 0; i < num_holes; i++)
	{
		sprintf(buf, "%d", i + 1);
		outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + i * 25, y + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
	}
	outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + 18 * 25 + 22, y + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), "Total");
}

void course_scoreboard_render_pars(float x, float y, int h)
{
	int i;
	char buf[256];

	outline_text_shadow(vgolf_font[VGOLF_FONT_32], x + 6, y + 3 + h * 1, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "Par");
	for(i = 0; i < game_course->num_holes; i++)
	{
		sprintf(buf, "%d", game_course->hole[i].par_score);
		outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + i * 25, y + h * 1 + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
	}
	sprintf(buf, "%d", game_course->course_par);
	outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + 18 * 25 + 22, y + h * 1 + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
}

void course_scoreboard_render_score(int player, int spot, float x, float y, int h)
{
	int j;
	char buf[256];

	for(j = 0; j < main_game_vars.cur_hole + 1; j++)
	{
		sprintf(buf, "%d", game_player[player].scorecard.score[j]);
		outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + j * 25, y + h * (2 + spot) + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
	}
	outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + 18 * 25 + 22, y + h * (2 + spot) + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), game_player[player].scorecard.score_text);
}

void course_scoreboard_render_scores(float x, float y, int h)
{
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			course_scoreboard_render_score(i, i, x, y, h);
		}
	}
}

void course_scoreboard_render_record(int profile, int c, int spot, bool local, float x, float y, int h, int num_holes, int o)
{
	int p_total, c_total;
	int j;
	char buf[256];

	if(o)
	{
		outline_text_shadow(vgolf_font[VGOLF_FONT_32], x + 6, y + 3 + (h), -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "Strokes");
	}
	if(vgolf_profiles.item[profile].course_info[c].completed > 0 && local)
	{
		for(j = 0; j < num_holes; j++)
		{
			if(vgolf_profiles.item[profile].course_info[c].score[j])
			{
				sprintf(buf, "%d", vgolf_profiles.item[profile].course_info[c].score[j]);
				outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + j * 25, y + h * (2 + spot - o) + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
			}
		}
		p_total = vgolf_profiles.item[profile].course_info[c].total_score;
		c_total = vgolf_courses.item[c].par;
		if(p_total - c_total == 0)
		{
			sprintf(buf, "E");
		}
		else if(p_total - c_total > 0)
		{
			sprintf(buf, "+%d", p_total - c_total);
		}
		else
		{
			sprintf(buf, "%d", p_total - c_total);
		}
	}
	else
	{
		sprintf(buf, "N/A");
	}
	outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x + 109 + 6 + 18 * 25 + 22, y + h * (2 + spot - o) + 3, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
}

void course_scoreboard_render_records(int c, float x, float y, int h, bool pars)
{
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			course_scoreboard_render_record(game_player[i].profile, c, i, vgolf_client_game->player[game_player[i].id]->local, x, y, h, game_course->num_holes, pars ? 0 : 1);
		}
	}
}

//draws the course scoreboard
void course_scoreboard_display(ALLEGRO_BITMAP * bp, void * a, VGOLF_COURSE * cr, PLAYER pl[], int num_players, int cur_hole)
{
	int s, i, seg_h = t3f_virtual_display_height/cr->scoreboard.num_scoreboard_segs;
	char buf[256];
	int c, h, score_width, ox, oy;

	al_hold_bitmap_drawing(true);
	t3f_draw_animation(cr->scoreboard.bgpic, t3f_color_white, 0, 0, (cr->scoreboard.scoreboard_seg_count - 1) * seg_h + cr->scoreboard.scoreboard_seg_counter - 48, 0, 0);
	for(s = 0; s < cr->scoreboard.scoreboard_seg_count - 1; s++)
	{
		t3f_draw_animation(cr->scoreboard.bgpic, t3f_color_white, 0, 0, s * seg_h, 0, 0);
	}
	c = vgolf_find_course(vgolf_selected_course);
	if(c < 0)
	{
		return;
	}
//	if(cr->scoreboard.scoreboard_seg_count >= 10)
	if(main_game_vars.game_state == GAMESTATE_SCOREBOARD_SEGMID)
	{
		h = al_get_bitmap_height(game_course->image_bank.scorecard) - 1;
		score_width = 25;
		ox = 8;
		oy = 240 - (h * (VGOLF_MAX_PLAYERS + 2)) / 2;

		/* render scorecard */
		course_scoreboard_render_card(ox, oy, VGOLF_MAX_PLAYERS, true);

		/* render scorecard text */
		course_scoreboard_render_header(ox, oy, game_course->num_holes);
		course_scoreboard_render_pars(ox, oy, h);
		al_hold_bitmap_drawing(false);
		al_hold_bitmap_drawing(true);
		t3f_set_clipping_rectangle(0, 0, ox + 6 + 88, 480);
		for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
		{
			if(vgolf_player_list[i])
			{
				sprintf(buf, "%s", vgolf_client_game->player[i]->name);
				outline_text_shadow(vgolf_font[VGOLF_FONT_32], ox + 6, oy + 3 + h * (2 + i), -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, buf);
			}
		}
		al_hold_bitmap_drawing(false);
		t3f_set_clipping_rectangle(0, 0, 0, 0);
		al_hold_bitmap_drawing(true);
		if(show_records)
		{
			course_scoreboard_render_records(c, ox, oy, h, true);
		}
		else
		{
			course_scoreboard_render_scores(ox, oy, h);
		}

		//identify course name (only display if it lies within the visible scoreboard area)
		if(cr->scoreboard.y + cr->scoreboard.cname_y <= cr->scoreboard.scoreboard_seg_count * seg_h)
		{
			sprintf(buf, "Course: %s", cr->name);
			outline_text_shadow(vgolf_font[VGOLF_FONT_48], 18, cr->scoreboard.y+cr->scoreboard.cname_y - 1 - 4, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, buf);
			if(next_hole < 18)
			{
		        sprintf(buf, "Next hole: %i", next_hole + 1);
				outline_text_shadow(vgolf_font[VGOLF_FONT_48], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], buf) - 18, cr->scoreboard.y+cr->scoreboard.cname_y - 1 - 4, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, buf);
			}
			else
			{
		        sprintf(buf, "Next: Awards");
				outline_text_shadow(vgolf_font[VGOLF_FONT_48], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], buf) - 18, cr->scoreboard.y+cr->scoreboard.cname_y - 1 - 4, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, buf);
			}
		}
		if(cr->scoreboard.scoreboard_seg_count >= cr->scoreboard.num_scoreboard_segs + 1)
		{
			if(vgolf_client_game->player[main_game_vars.cur_player]->local)
			{
				if(!fake_page_hidden)
				{
					hyperlink_page_render(fake_scoreboard_page, bp);
				}
			}
			else if(!vgolf_client)
			{
				hyperlink_page_render(scoreboard_page, bp);
			}
		}
	}
	al_hold_bitmap_drawing(false);
}
