#include "t3f/t3f.h"
#include "main.h"
#include "hyperlink.h"
#include "message.h"
#include "core/utility.h"

static int * hyperlink_mouse_x;
static int * hyperlink_mouse_y;
static int * hyperlink_mouse_z;
static int * hyperlink_mouse_b;

void hyperlink_link_mouse(int * x, int * y, int * z, int * b)
{
	hyperlink_mouse_x = x;
	hyperlink_mouse_y = y;
	hyperlink_mouse_z = z;
	hyperlink_mouse_b = b;
}

HYPERLINK_PAGE * hyperlink_page_create(int ox, int oy)
{
	HYPERLINK_PAGE * lp;
	lp = malloc(sizeof(HYPERLINK_PAGE));
	if(!lp)
	{
		return NULL;
	}
	lp->elements = 0;
	lp->ox = ox;
	lp->oy = oy;
	lp->hover_element = -1;
	lp->tick = 0;
	lp->state.left_click = 2;
	lp->state.right_click = 2;
	return lp;
}

void hyperlink_page_destroy(HYPERLINK_PAGE * pp)
{
	if(pp)
	{
		free(pp);
	}
}

int hyperlink_page_add_element_image(HYPERLINK_PAGE * pp, int (*proc)(int, void *), ALLEGRO_BITMAP ** bp, int ox, int oy, int flags)
{
	pp->element[pp->elements].type = HYPERLINK_ELEMENT_TYPE_IMAGE;
	pp->element[pp->elements].proc = proc;
	pp->element[pp->elements].data2 = (void **)bp;
	pp->element[pp->elements].ox = ox;
	pp->element[pp->elements].oy = oy;
	pp->element[pp->elements].flags = flags;
	pp->elements++;
	return 1;
}

int hyperlink_page_add_element_animation(HYPERLINK_PAGE * pp, int (*proc)(int, void *), T3F_ANIMATION ** ap, int ox, int oy, int flags)
{
	pp->element[pp->elements].type = HYPERLINK_ELEMENT_TYPE_ANIMATION;
	pp->element[pp->elements].proc = proc;
	pp->element[pp->elements].data2 = (void **)ap;
	pp->element[pp->elements].ox = ox;
	pp->element[pp->elements].oy = oy;
	pp->element[pp->elements].flags = flags;
	pp->elements++;
	return 1;
}

int hyperlink_page_add_element_text(HYPERLINK_PAGE * pp, int (*proc)(int, void *), char * text, T3F_FONT ** fp, int ox, int oy, ALLEGRO_COLOR color, ALLEGRO_COLOR hcolor, ALLEGRO_COLOR outline, int flags)
{
	pp->element[pp->elements].type = HYPERLINK_ELEMENT_TYPE_TEXT;
	pp->element[pp->elements].proc = proc;
	pp->element[pp->elements].data = text;
	pp->element[pp->elements].aux_data = (void **)fp;
	pp->element[pp->elements].ox = ox;
	pp->element[pp->elements].oy = oy;
	pp->element[pp->elements].d1 = color;
	pp->element[pp->elements].d2 = hcolor;
	pp->element[pp->elements].d3 = outline;
	pp->element[pp->elements].flags = flags;
	pp->elements++;
	return 1;
}

void hyperlink_click_logic(HYPERLINK_PAGE * pp)
{
	if(*hyperlink_mouse_b & 1)
	{
		pp->state.left_click++;
	}
	else
	{
		pp->state.left_click = 0;
	}
	if(*hyperlink_mouse_b & 2)
	{
		pp->state.right_click++;
	}
	else
	{
		pp->state.right_click = 0;
	}
}

void hyperlink_page_logic(HYPERLINK_PAGE * pp)
{
	int i, m = 1;

	if(pp)
	{
		pp->hover_element = -1;
		for(i = 0; i < pp->elements; i++)
		{
			if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_CLICKABLE)
			{
				switch(pp->element[i].type)
				{
					case HYPERLINK_ELEMENT_TYPE_TEXT:
					{
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
						{
							if(*hyperlink_mouse_x >= pp->ox + pp->element[i].ox - t3f_get_text_width((T3F_FONT *)(*pp->element[i].aux_data), (char *)pp->element[i].data) / 2 && *hyperlink_mouse_x < pp->ox + pp->element[i].ox + t3f_get_text_width((T3F_FONT *)(*pp->element[i].aux_data), (char *)pp->element[i].data) / 2 && *hyperlink_mouse_y >= pp->oy + pp->element[i].oy && *hyperlink_mouse_y < pp->oy + pp->element[i].oy + t3f_get_font_line_height((T3F_FONT *)(*pp->element[i].aux_data)))
							{
								pp->hover_element = i;
							}
						}
						else
						{
							if(*hyperlink_mouse_x >= pp->ox + pp->element[i].ox && *hyperlink_mouse_x < pp->ox + pp->element[i].ox + t3f_get_text_width((T3F_FONT *)(*pp->element[i].aux_data), (char *)pp->element[i].data) && *hyperlink_mouse_y >= pp->oy + pp->element[i].oy && *hyperlink_mouse_y < pp->oy + pp->element[i].oy + t3f_get_font_line_height((T3F_FONT *)(*pp->element[i].aux_data)))
							{
								pp->hover_element = i;
							}
						}
						break;
					}
					case HYPERLINK_ELEMENT_TYPE_IMAGE:
					{
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
						{
							if(*hyperlink_mouse_x >= pp->ox + pp->element[i].ox - (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2 && *hyperlink_mouse_x < pp->ox + pp->element[i].ox + (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2 && *hyperlink_mouse_y >= pp->oy + pp->element[i].oy - (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2 && *hyperlink_mouse_y < pp->oy + pp->element[i].oy + (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2)
							{
								pp->hover_element = i;
							}
						}
						else
						{
							if(*hyperlink_mouse_x >= pp->ox + pp->element[i].ox && *hyperlink_mouse_x < pp->ox + pp->element[i].ox + (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[i].data2))) && *hyperlink_mouse_y >= pp->oy + pp->element[i].oy && *hyperlink_mouse_y < pp->oy + pp->element[i].oy + (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[i].data2))))
							{
								pp->hover_element = i;
							}
						}
						break;
					}
					case HYPERLINK_ELEMENT_TYPE_ANIMATION:
					{
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_2X)
						{
							m = 2;
						}
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
						{
							if(*hyperlink_mouse_x >= pp->ox + pp->element[i].ox - ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m && *hyperlink_mouse_x < pp->ox + pp->element[i].ox + ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m && *hyperlink_mouse_y >= pp->oy + pp->element[i].oy - ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m && *hyperlink_mouse_y < pp->oy + pp->element[i].oy + ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m)
							{
								pp->hover_element = i;
							}
						}
						else
						{
							if(*hyperlink_mouse_x >= pp->ox + pp->element[i].ox && *hyperlink_mouse_x < pp->ox + pp->element[i].ox + (vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[i].data2))) * m && *hyperlink_mouse_y >= pp->oy + pp->element[i].oy && *hyperlink_mouse_y < pp->oy + pp->element[i].oy + (vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[i].data2))) * m)
							{
								pp->hover_element = i;
							}
						}
						break;
					}
				}
			}
		}
		if(pp->state.left_click == 1 && pp->hover_element >= 0)
		{
			if(pp->element[pp->hover_element].proc)
			{
				pp->element[pp->hover_element].proc(0, pp);
			}
		}
	}
	pp->tick++;
}

void hyperlink_page_render(HYPERLINK_PAGE * pp, ALLEGRO_BITMAP * bp)
{
	int i;
	float m = 1.0;

	if(pp)
	{
		for(i = 0; i < pp->elements; i++)
		{
			if(i != pp->hover_element)
			{
				switch(pp->element[i].type)
				{
					case HYPERLINK_ELEMENT_TYPE_TEXT:
					{
						int sx, sy;
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_SHADOW)
						{
							sx = -2;
							sy = 2;
							if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
							{
								outline_text_shadow_center((T3F_FONT *)(*pp->element[i].aux_data), pp->ox + pp->element[i].ox - sx, pp->oy + pp->element[i].oy - sy, sx, sy, pp->element[i].d1, pp->element[i].d3, al_map_rgb(0, 0, 0), (char *)pp->element[i].data);
							}
							else
							{
								outline_text_shadow((T3F_FONT *)(*pp->element[i].aux_data), pp->ox + pp->element[i].ox - sx, pp->oy + pp->element[i].oy - sy, sx, sy, pp->element[i].d1, pp->element[i].d3, al_map_rgb(0, 0, 0), 0.0, (char *)pp->element[i].data);
							}
						}
						else
						{
						}
						break;
					}
					case HYPERLINK_ELEMENT_TYPE_IMAGE:
					{
						int sx, sy;
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_SHADOW)
						{
							sx = -2;
							sy = 2;
							if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
							{
								if(*pp->element[i].data2)
								{
									al_draw_tinted_bitmap((ALLEGRO_BITMAP *)(*pp->element[i].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->ox + pp->element[i].ox - (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2, pp->oy + pp->element[i].oy - (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2, 0);
									if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
									{
										al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[i].data2), pp->ox + pp->element[i].ox - (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2 - sx, pp->oy + pp->element[i].oy - (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2 - sy, 0);
									}
									else
									{
										al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[i].data2), pp->ox + pp->element[i].ox - (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2 - sx, pp->oy + pp->element[i].oy - (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[i].data2))) / 2 - sy, 0);
									}
								}
							}
							else
							{
								if(pp->element[i].data)
								{
									al_draw_tinted_bitmap((ALLEGRO_BITMAP *)(*pp->element[i].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->ox + pp->element[i].ox, pp->oy + pp->element[i].oy, 0);
									if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
									{
										al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[i].data2), pp->ox + pp->element[i].ox - sx, pp->oy + pp->element[i].oy - sy, 0);
									}
									else
									{
										al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[i].data2), pp->ox + pp->element[i].ox - sx, pp->oy + pp->element[i].oy - sy, 0);
									}
								}
							}
						}
						break;
					}
					case HYPERLINK_ELEMENT_TYPE_ANIMATION:
					{
						int sx, sy;
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_2X)
						{
							m = 2.0;
						}
						if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_SHADOW)
						{
							sx = -2;
							sy = 2;
							if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
							{
								if(pp->element[i].data2)
								{
									t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[i].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->tick, pp->ox + pp->element[i].ox - ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m, pp->oy + pp->element[i].oy - ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m, 0.0, m, 0);
									if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
									{
										t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[i].data2), t3f_color_white, pp->tick, pp->ox + pp->element[i].ox - ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m - sx, pp->oy + pp->element[i].oy - ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m - sy, 0.0, m, 0);
									}
									else
									{
										t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[i].data2), t3f_color_white, pp->tick, pp->ox + pp->element[i].ox - ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m - sx, pp->oy + pp->element[i].oy - ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[i].data2))) / 2) * m - sy, 0.0, m, 0);
									}
								}
							}
							else
							{
								if(pp->element[i].data2)
								{
									t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[i].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->tick, pp->ox + pp->element[i].ox, pp->oy + pp->element[i].oy, 0.0, m, 0);
									if(pp->element[i].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
									{
										t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[i].data2), t3f_color_white, pp->tick, pp->ox + pp->element[i].ox - sx, pp->oy + pp->element[i].oy - sy, 0.0, m, 0);
									}
									else
									{
										t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[i].data2), t3f_color_white, pp->tick, pp->ox + pp->element[i].ox - sx, pp->oy + pp->element[i].oy - sy, 0.0, m, 0);
									}
								}
							}
						}
						break;
					}
					default:
					{
						break;
					}
				}
			}
		}

		/* render the hover element last so it appears on top */
		if(pp->hover_element >= 0)
		{
			switch(pp->element[pp->hover_element].type)
			{
				case HYPERLINK_ELEMENT_TYPE_TEXT:
				{
					int sx, sy;
					if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_SHADOW)
					{
						sx = -4;
						sy = 4;
						if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
						{
							outline_text_shadow_center((T3F_FONT *)(*pp->element[pp->hover_element].aux_data), pp->ox + pp->element[pp->hover_element].ox - sx, pp->oy + pp->element[pp->hover_element].oy - sy, sx, sy, pp->element[pp->hover_element].d2, pp->element[pp->hover_element].d3, al_map_rgb(0, 0, 0), (char *)pp->element[pp->hover_element].data);
						}
						else
						{
							outline_text_shadow((T3F_FONT *)(*pp->element[pp->hover_element].aux_data), pp->ox + pp->element[pp->hover_element].ox - sx, pp->oy + pp->element[pp->hover_element].oy - sy, sx, sy, pp->element[pp->hover_element].d2, pp->element[pp->hover_element].d3, al_map_rgb(0, 0, 0), 0.0, (char *)pp->element[pp->hover_element].data);
						}
					}
					else
					{
					}
					break;
				}
				case HYPERLINK_ELEMENT_TYPE_IMAGE:
				{
					int sx, sy;
					if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_SHADOW)
					{
						sx = -4;
						sy = 4;
						if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
						{
							if(*pp->element[pp->hover_element].data2)
							{
								al_draw_tinted_bitmap((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->ox + pp->element[pp->hover_element].ox - (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2))) / 2, pp->oy + pp->element[pp->hover_element].oy - (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2))) / 2, 0);
								if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
								{
									al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2), pp->ox + pp->element[pp->hover_element].ox - (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2))) / 2 - sx, pp->oy + pp->element[pp->hover_element].oy - (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2))) / 2 - sy, 0);
								}
								else
								{
									al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2), pp->ox + pp->element[pp->hover_element].ox - (al_get_bitmap_width((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2))) / 2 - sx, pp->oy + pp->element[pp->hover_element].oy - (al_get_bitmap_height((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2))) / 2 - sy, 0);
								}
							}
						}
						else
						{
							if(*pp->element[pp->hover_element].data2)
							{
								al_draw_tinted_bitmap((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->ox + pp->element[pp->hover_element].ox, pp->oy + pp->element[pp->hover_element].oy, 0);
								if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
								{
									al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2), pp->ox + pp->element[pp->hover_element].ox - sx, pp->oy + pp->element[pp->hover_element].oy - sy, 0);
								}
								else
								{
									al_draw_bitmap((ALLEGRO_BITMAP *)(*pp->element[pp->hover_element].data2), pp->ox + pp->element[pp->hover_element].ox - sx, pp->oy + pp->element[pp->hover_element].oy - sy, 0);
								}
							}
						}
					}
					break;
				}
				case HYPERLINK_ELEMENT_TYPE_ANIMATION:
				{
					int sx, sy;
					if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_2X)
					{
						m = 2.0;
					}
					if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_SHADOW)
					{
						sx = -4;
						sy = 4;
						if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_CENTER)
						{
							if(pp->element[pp->hover_element].data2)
							{
								t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->tick, pp->ox + pp->element[pp->hover_element].ox - ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2))) / 2) * m, pp->oy + pp->element[pp->hover_element].oy - ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2))) / 2) * m, 0.0, m, 0);
								if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
								{
									t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2), t3f_color_white, pp->tick, pp->ox + pp->element[pp->hover_element].ox - ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2))) / 2) * m - sx, pp->oy + pp->element[pp->hover_element].oy - ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2))) / 2) * m - sy, 0.0, m, 0);
								}
								else
								{
									t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2), t3f_color_white, pp->tick, pp->ox + pp->element[pp->hover_element].ox - ((vgolf_get_bitmap_width((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2))) / 2) * m - sx, pp->oy + pp->element[pp->hover_element].oy - ((vgolf_get_bitmap_height((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2))) / 2) * m - sy, 0.0, m, 0);
								}
							}
						}
						else
						{
							if(pp->element[pp->hover_element].data2)
							{
								t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), pp->tick, pp->ox + pp->element[pp->hover_element].ox, pp->oy + pp->element[pp->hover_element].oy, 0.0, m, 0);
								if(pp->element[pp->hover_element].flags & HYPERLINK_ELEMENT_FLAG_BLIT)
								{
									t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2), t3f_color_white, pp->tick, pp->ox + pp->element[pp->hover_element].ox - sx, pp->oy + pp->element[pp->hover_element].oy - sy, 0.0, m, 0);
								}
								else
								{
									t3f_draw_scaled_animation((T3F_ANIMATION *)(*pp->element[pp->hover_element].data2), t3f_color_white, pp->tick, pp->ox + pp->element[pp->hover_element].ox - sx, pp->oy + pp->element[pp->hover_element].oy - sy, 0.0, m, 0);
								}
							}
						}
					}
					break;
				}
				default:
				{
					break;
				}
			}
		}
	}
}
