#ifndef HYPERLINK_H
#define HYPERLINK_H

#include "t3f/t3f.h"
#include "t3f/animation.h"
#include "vfont.h"

#define HYPERLINK_PAGE_MAX_ELEMENTS 128

#define HYPERLINK_ELEMENT_TYPE_TEXT      0
#define HYPERLINK_ELEMENT_TYPE_IMAGE     1
#define HYPERLINK_ELEMENT_TYPE_ANIMATION 2

#define HYPERLINK_ELEMENT_FLAG_CLICKABLE 1
#define HYPERLINK_ELEMENT_FLAG_CENTER    2
#define HYPERLINK_ELEMENT_FLAG_SHADOW    4
#define HYPERLINK_ELEMENT_FLAG_BLIT      8
#define HYPERLINK_ELEMENT_FLAG_2X       16

#define HYPERLINK_PAGE_FLAG_DISABLED     1

typedef struct
{

	int type;
	void * data;
	void ** data2;
	void ** aux_data;
	int flags;
	int (*proc)(int, void *);

	int ox, oy;
	ALLEGRO_COLOR d1, d2, d3;

} HYPERLINK_ELEMENT;

/* store the state of the GUI */
typedef struct
{

	int left_click;
	int right_click;

} HYPERLINK_STATE;

typedef struct
{

	HYPERLINK_ELEMENT element[HYPERLINK_PAGE_MAX_ELEMENTS];
	int elements;
	int flags;

	int ox, oy;

	HYPERLINK_STATE state;

	int hover_element;
	int tick;

} HYPERLINK_PAGE;

void hyperlink_link_mouse(int * x, int * y, int * z, int * b);
void hyperlink_clear_state(HYPERLINK_STATE * sp);
void hyperlink_select_state(HYPERLINK_STATE * sp);

HYPERLINK_PAGE * hyperlink_page_create(int ox, int oy);
void hyperlink_page_destroy(HYPERLINK_PAGE * pp);

int hyperlink_page_add_element_image(HYPERLINK_PAGE * pp, int (*proc)(int, void *), ALLEGRO_BITMAP ** bp, int ox, int oy, int flags);
int hyperlink_page_add_element_animation(HYPERLINK_PAGE * pp, int (*proc)(int, void *), T3F_ANIMATION ** ap, int ox, int oy, int flags);
int hyperlink_page_add_element_text(HYPERLINK_PAGE * pp, int (*proc)(int, void *), char * text, T3F_FONT ** fp, int ox, int oy, ALLEGRO_COLOR color, ALLEGRO_COLOR hcolor, ALLEGRO_COLOR outline, int flags);

void hyperlink_click_logic(HYPERLINK_PAGE * pp);
void hyperlink_page_logic(HYPERLINK_PAGE * pp);
void hyperlink_page_render(HYPERLINK_PAGE * pp, ALLEGRO_BITMAP * bp);

#endif
