#include "t3f/t3f.h"
#include "t3f/font.h"
#include "t3f/sound.h"
#include "t3f/music.h"
#include "t3f/resource.h"
#include "t3f/debug.h"
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <limits.h>
#include "t3net/t3net.h"
#include "t3net/dlc.h"
#include "init.h"
#include "fx.h"
#include "hyperlink.h"
#include "structs.h"
#include "defines.h"
#include "core/body.h"
#include "core/poly.h"
#include "vfont.h"
#include "newmenu.h"
#include "main.h"
#include "profile.h"
#include "message.h"
#include "credits.h"
#include "core/avatar.h"
#include "core/course.h"
#include "core/legacy_course.h"
#include "t3_logo.h"

#include "menu_proc.h"
#include "gameplay.h"
#include "replay.h"
#include "gameplay_scoreboard.h"

#include "network/server.h"
#include "network/callbacks.h"

#include "core/utility.h"

#include "state_title.h"

#include "globals.h"

int vgolf_find_avatar(unsigned long hash)
{
	int i;

	for(i = 0; i < vgolf_avatars.avatars; i++)
	{
		if(vgolf_avatars.sum[i] == hash)
		{
			return i;
		}
	}
	return -1;
}

int vgolf_find_course(unsigned long hash)
{
	int i;

	for(i = 0; i < vgolf_courses.items; i++)
	{
		if(vgolf_courses.item[i].sum == hash)
		{
			return i;
		}
	}
	return -1;
}

void text_enter_logic(void)
{
	char k;

	if(entering_text)
	{
		k = t3f_read_key(0);

		/* if BACK is pressed and no other keys are in the buffer, simulate ESC */
		if(k < 0)
		{
			if(t3f_key[ALLEGRO_KEY_BACK])
			{
				k = 0x1B;
			}
		}
		if(k > 0)
		{
			switch(k)
			{
				/* enter */
				case 0xD:
				{
	            	entered_text[entered_text_pos] = '\0';
					t3f_key[ALLEGRO_KEY_ENTER] = 0;
					if(entering_text == 66)
					{
						if(strlen(entered_text) > 0)
						{
							joynet_send_client_chat(vgolf_client, entered_text, 0);
						}
						entering_text = 0;
					}
					else if(entering_text == 2)
					{
						strcpy(vgolf_nick, entered_text);
						newmenu_enable_controls();
						entering_text = 0;
					}
					else if(entering_text == 3)
					{
						if(strlen(entered_text) > 0)
						{
							hl_player_add_account(0, NULL);
							entering_text = 0;
						}
					}
					else if(entering_text == 4)
					{
						vgolf_menu_proc_begin_hosting();
					}
					else if(entering_text == 5)
					{
						vgolf_menu_proc_accept_name();
					}
					break;
				}

				/* escape */
				case 0x1B:
				{
					if(entering_text == 1 || entering_text == 4)
					{
						strcpy(entered_text, old_entered_text);
						t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
						t3f_key[ALLEGRO_KEY_BACK] = 0;
						entering_text = 0;
					}
					else if(entering_text == 2)
					{
						newmenu_enable_controls();
						t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
						t3f_key[ALLEGRO_KEY_BACK] = 0;
						entering_text = 0;
					}
					else if(entering_text == 3)
					{
						entering_text = 0;
					}
					break;
				}

				/* backspace */
				#ifndef ALLEGRO_MACOSX
				case 0x8:
				#else
				case 127:
				#endif
				{
	       			entered_text[entered_text_pos] = '\0';
					if(entered_text_pos > 0)
					{
	           			entered_text_pos--;
   						entered_text[entered_text_pos] = '\0';
					}
					break;
				}

				/* ignore these keys */
				case '\t':
				{
					break;
				}

				/* normal key */
				default:
				{
					if(entered_text_pos < entered_text_max - 1)
					{
	       				entered_text[entered_text_pos] = k;
						entered_text_pos++;
						entered_text[entered_text_pos] = '\0';
					}
					break;
				}
			}
		}
	}
}

//try and free a bitmap from memory..
int trash_bitmap(ALLEGRO_BITMAP * bp)
{
	if(bp != NULL)
	{
		al_destroy_bitmap(bp);
	}
	return -1;
}

void game_settings_update(void)
{
}

void config_set_nick(void)
{
	strcpy(old_vgolf_nick, vgolf_nick);
	strcpy(entered_text, "");
	entered_text_pos = strlen("");
	entering_text = 2;
	newmenu_disable_controls();
	t3f_clear_keys();
}

int get_course(unsigned long sum)
{
	int i;

	for(i = 0; i < vgolf_courses.items; i++)
	{
		if(vgolf_courses.item[i].sum == sum)
		{
			return i;
		}
	}
	return -1;
}

/* draw shadow boxes */
void shadow_box(ALLEGRO_BITMAP * bp, int x, int y, int dx, int dy, int lw, ALLEGRO_COLOR oc, ALLEGRO_COLOR bc)
{
	al_draw_filled_rectangle(x, y, dx, dy, bc);
	al_draw_rectangle(x + 0.5, y + 0.5, dx - 1.0 + 0.5, dy - 1.0 + 0.5, oc, lw);
}

void double_shadow_box(ALLEGRO_BITMAP * bp, int x, int y, int dx, int dy, int lw, ALLEGRO_COLOR oc, ALLEGRO_COLOR bc)
{
	shadow_box(bp, x, y, dx, dy, lw, oc, bc);
	shadow_box(bp, x - 8, y - 8, dx + 8, dy + 8, lw - 1, oc, bc);
}

void intro_helper(void)
{
	t3_logo_finish();
}

void credits_escape_helper(void)
{
	int i;

	for(i = 0; i < vgolf_credits.images; i++)
	{
		if(vgolf_credits.image[i].bp)
		{
			t3f_destroy_resource(vgolf_credits.image[i].bp);
			vgolf_credits.image[i].bp = NULL;
		}
	}
	fx_fade_start(menu_bg_color, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 2, NULL);
	vgolf_state = VGOLF_STATE_MENU;
}

void vgolf_hide_chat(void)
{
	switch(vgolf_chat_state)
	{
		case 1:
		{
			if(vgolf_chat_offset_x > -vgolf_chat_width)
			{
				vgolf_chat_offset_x -= 32;
			}
			if(vgolf_chat_offset_x <= -vgolf_chat_width)
			{
				vgolf_chat_offset_x = -vgolf_chat_width;
				vgolf_chat_state = 2;
			}
			break;
		}
		case 3:
		{
			if(vgolf_chat_offset_x < 0)
			{
				vgolf_chat_offset_x += 32;
			}
			if(vgolf_chat_offset_x >= 0)
			{
				vgolf_chat_offset_x = 0;
				vgolf_chat_state = 0;
				vgolf_chat_new_message = false;
			}
			break;
		}
	}
	chat_page->ox = vgolf_chat_offset_x;
}

static bool vgolf_touched = false;

void vgolf_logic(void * data)
{
	int i;

	if(vgolf_state == VGOLF_STATE_NONE)
	{
		return;
	}
	if(vgolf_client)
	{
		joynet_poll_client(vgolf_client);
		if(vgolf_client_disconnected)
		{
			joynet_destroy_client(vgolf_client);
			vgolf_client = NULL;
			vgolf_client_disconnected = false;
		}
	}

	if(t3f_key[ALLEGRO_KEY_T] && vgolf_client && entering_text == 0)
	{
		strcpy(entered_text, "");
		entered_text_pos = 0;
		entering_text = 66;
		t3f_key[ALLEGRO_KEY_T] = 0;
		if(vgolf_chat_state != 0)
		{
			vgolf_chat_state = 3;
		}
		t3f_clear_keys();
	}
	if(entering_text == 0 && t3f_key[ALLEGRO_KEY_TAB])
	{
		vgolf_menu_proc_toggle_chat(0, NULL);
		t3f_key[ALLEGRO_KEY_TAB] = 0;
	}
	vgolf_hide_chat();
	if(entering_text)
	{
		text_enter_logic();
	}

	vgolf_old_mouse_x = vgolf_mouse_x;
	vgolf_old_mouse_y = vgolf_mouse_y;
	vgolf_mouse_x = t3f_mouse_x;
	vgolf_mouse_y = t3f_mouse_y;

	/* limit wheel to range (0-20) */
	if(t3f_mouse_z < 0)
	{
		al_set_mouse_z(0);
		t3f_mouse_z = 0;
	}
	else if(t3f_mouse_z > 20)
	{
		al_set_mouse_z(20);
		t3f_mouse_z = 20;
	}
	vgolf_mouse_z = t3f_mouse_z * 5;

	/* update mouse */
	vgolf_mouse_b = t3f_mouse_button[0] ? 1 : 0;
	vgolf_mouse_b |=  t3f_mouse_button[1] ? 2 : 0;

	/* emulate mouse buttons with game buttons for pandora */
	if(t3f_key[ALLEGRO_KEY_HOME])
	{
		vgolf_mouse_b |= 1;
	}
	if(t3f_key[ALLEGRO_KEY_PGUP])
	{
		vgolf_mouse_b |= 2;
	}

	/* emulate mouse with touch controls on Android */
	#ifdef T3F_ANDROID
		vgolf_touched = false;
		vgolf_mouse_b = 0; // assume mouse button is up unless touch is detected below
		for(i = 0; i < T3F_MAX_TOUCHES; i++)
		{
			if(t3f_touch[i].active)
			{
				vgolf_mouse_x = t3f_touch[i].x;
				vgolf_mouse_y = t3f_touch[i].y;
				vgolf_touched = true;
			}
			if(t3f_touch[i].released)
			{
				vgolf_mouse_x = t3f_touch[i].x;
				vgolf_mouse_y = t3f_touch[i].y;
				vgolf_mouse_b = 1;
				t3f_touch[i].released = false;
			}
		}
	#endif
	if(vgolf_mouse_b & 1)
	{
		left_click++;
	}
	else
	{
		left_click = 0;
	}
	if(vgolf_mouse_b & 2)
	{
		right_click++;
	}
	else
	{
		right_click = 0;
	}

	/* game start timer countdown */
	if(vgolf_start_game_timer > 0)
	{
		vgolf_start_game_timer--;

		if(vgolf_client && vgolf_client->master)
		{
			/* master will start the game when timer reaches 0 */
			if(vgolf_start_game_timer <= 0)
			{
				t3f_play_sample(main_game_vars.sound_bank.sfx_title_swing, 1.0, 0.0, 1.0);
				play_proc();
			}

			/* also start if all players have finished selecting */
			else
			{
				for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
				{
					if(vgolf_player_list[i] && game_player[i].setup_step != VGOLF_PLAYER_SETUP_SUMMARY)
					{
						break;
					}
				}
				if(i == VGOLF_MAX_PLAYERS)
				{
					t3f_play_sample(main_game_vars.sound_bank.sfx_title_swing, 1.0, 0.0, 1.0);
					play_proc();
				}
			}
		}
	}

	if(vgolf_client)
	{
		hyperlink_click_logic(chat_page);
		hyperlink_page_logic(chat_page);
	}
	if(!(vgolf_mouse_b & 1) && !vgolf_touched)
	{
		locally_block_click = false;
	}

	switch(vgolf_state)
	{
		case VGOLF_STATE_INTRO:
		{
			t3_logo_logic();
			if(t3_logo_done)
			{
				t3f_render(true);
				t3f_render(true);
				intro_helper();
			}
			break;
		}
		case VGOLF_STATE_PLAY:
		{
			vgolf_game_logic();
			if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
			{
				if(vgolf_replay_mode == VGOLF_REPLAY_MODE_PLAY)
				{
					vgolf_end_game();
				}
				else
				{
					joynet_pause_game(vgolf_client_game);
				}
				t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
				t3f_key[ALLEGRO_KEY_BACK] = 0;
			}
			break;
		}
		case VGOLF_STATE_PAUSE:
		{
			if(!vgolf_client || vgolf_client->master)
			{
				newmenu_logic();
				if(current_newmenu != nmenu[VGOLF_MENU_IN_GAME])
				{
					hyperlink_click_logic(sub_menu_page);
					hyperlink_page_logic(sub_menu_page);
				}
			}
			break;
		}
		case VGOLF_STATE_PROFILE:
		{
			if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
			{
				vgolf_state = VGOLF_STATE_MENU;
				t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
				t3f_key[ALLEGRO_KEY_BACK] = 0;
			}
			vgolf_title_bg_logic();
			hyperlink_click_logic(profile_page);
			hyperlink_page_logic(profile_page);
			hyperlink_click_logic(profile_global_page);
			hyperlink_page_logic(profile_global_page);
			break;
		}
		case VGOLF_STATE_PROFILE_DELETE:
		{
			vgolf_title_bg_logic();
			newmenu_logic();
			break;
		}
		case VGOLF_STATE_PLAYER_SETUP:
		{
			if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
			{
				vgolf_state = VGOLF_STATE_MENU;
				t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
				t3f_key[ALLEGRO_KEY_BACK] = 0;
			}
			vgolf_title_bg_logic();
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				hyperlink_click_logic(player_page_pointer[i]);
				hyperlink_page_logic(player_page_pointer[i]);
			}
			if(vgolf_client_game->player_count < VGOLF_MAX_PLAYERS)
			{
				hyperlink_click_logic(player_add_page);
				hyperlink_page_logic(player_add_page);
			}
//			if(!vgolf_client || vgolf_client->master)
//			{
				hyperlink_click_logic(player_global_page);
				hyperlink_page_logic(player_global_page);
//			}
//			else
//			{
//				hyperlink_click_logic(player_global_slave_page);
//				hyperlink_page_logic(player_global_slave_page);
//			}
			break;
		}
		case VGOLF_STATE_PLAYER_AVATAR:
		{
			vgolf_title_bg_logic();
			hyperlink_click_logic(player_avatar_page);
			hyperlink_page_logic(player_avatar_page);
			break;
		}
		case VGOLF_STATE_DLC_AVATAR:
		{
			vgolf_title_bg_logic();
			hyperlink_click_logic(player_dlc_avatar_page);
			hyperlink_page_logic(player_dlc_avatar_page);
			break;
		}
		case VGOLF_STATE_PLAYER_ACCOUNT:
		{
			vgolf_title_bg_logic();
			hyperlink_click_logic(player_account_page);
			hyperlink_page_logic(player_account_page);
			break;
		}
		case VGOLF_STATE_PLAYER_NEW:
		{
			vgolf_title_bg_logic();
			hyperlink_click_logic(player_new_account_page);
			hyperlink_page_logic(player_new_account_page);
			break;
		}
		case VGOLF_STATE_COURSE:
		{
			vgolf_title_bg_logic();
			hyperlink_click_logic(course_page);
			hyperlink_page_logic(course_page);
			break;
		}
		case VGOLF_STATE_SUMMARY:
		{
			vgolf_title_bg_logic();
			if(!vgolf_client || vgolf_client->master)
			{
				hyperlink_click_logic(course_global_page);
				hyperlink_page_logic(course_global_page);
			}
			else
			{
				hyperlink_click_logic(course_global_slave_page);
				hyperlink_page_logic(course_global_slave_page);
			}
			break;
		}
		case VGOLF_STATE_CREDITS:
		{
			credits_logic(&vgolf_credits);
			if(vgolf_credits.done)
			{
				credits_escape_helper();
			}
			else if((t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK] || left_click == 1) && fx_fade_speed == 0)
			{
				fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), menu_bg_color, 2, credits_escape_helper);
				t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
				t3f_key[ALLEGRO_KEY_BACK] = 0;
			}
			break;
		}
		case VGOLF_STATE_LEADERBOARDS:
		{
			if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
			{
				play_esc_sample();
				vgolf_state = VGOLF_STATE_MENU;
				if(leaderboard_mode == 1)
				{
					current_newmenu = nmenu[VGOLF_MENU_MAIN];
					t3f_play_music("data/music/menu.xm");
				}
				t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
				t3f_key[ALLEGRO_KEY_BACK] = 0;
			}
			vgolf_title_bg_logic();
			hyperlink_click_logic(leaderboard_page);
			hyperlink_page_logic(leaderboard_page);
			hyperlink_click_logic(leaderboard_global_page);
			hyperlink_page_logic(leaderboard_global_page);
			break;
		}
		case VGOLF_STATE_DLC:
		{
			if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
			{
				play_esc_sample();
				vgolf_state = VGOLF_STATE_MENU;
				t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
				t3f_key[ALLEGRO_KEY_BACK] = 0;
			}
			vgolf_title_bg_logic();
			hyperlink_click_logic(dlc_page);
			hyperlink_page_logic(dlc_page);
			break;
		}
		case VGOLF_STATE_LOAD:
		case VGOLF_STATE_NONE:
		{
			break;
		}
		case VGOLF_STATE_PRIVACY:
		{
			vgolf_title_bg_logic();
			hyperlink_click_logic(sub_menu_page);
			hyperlink_page_logic(sub_menu_page);
			break;
		}
		default:
		{
			vgolf_state_title_logic();
			break;
		}
	}

	ncdmsg_logic(&vgolf_messages);
	fx_fade_logic();
	vgolf_ticks++;
}

void vgolf_render(void * data)
{
	int i, a, y, s;
	int px[VGOLF_MAX_PLAYERS] = {0, 213, 426, 0, 213, 426, 0, 426};
	int py[VGOLF_MAX_PLAYERS] = {0, 0, 0, 160, 160, 160, 320, 320};
	T3F_TEXT_LINE_DATA line_data;
	float ox, oy;
	char text[64] = {0};

	switch(vgolf_state)
	{
		case VGOLF_STATE_INTRO:
		{
			t3_logo_render();
			break;
		}
		case VGOLF_STATE_PLAY:
		{
			vgolf_game_render();
			break;
		}
		case VGOLF_STATE_PAUSE:
		{
			vgolf_game_render();
			al_draw_filled_rectangle(0, 0, t3f_virtual_display_width - 1, t3f_virtual_display_height - 1, al_map_rgba_f(0.0, 0.0, 0.0, 0.5));
			if(!vgolf_client || vgolf_client->master)
			{
				newmenu_render(NULL);
				if(current_newmenu != nmenu[VGOLF_MENU_IN_GAME])
				{
					hyperlink_page_render(sub_menu_page, NULL);
				}
			}
			break;
		}
		case VGOLF_STATE_PROFILE:
		{
			vgolf_render_title_bg();
			al_draw_filled_rectangle(0, 0, 640, 480, al_map_rgba(0, 0, 0, 64));
//			double_shadow_box(NULL, 132, 64, 640 - 132 - 1, 64 + 352, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			al_hold_bitmap_drawing(true);
			hyperlink_page_render(profile_page, NULL);
			hyperlink_page_render(profile_global_page, NULL);

			/* draw detailed scores */
			if(profile_show_details && profile_course >= 0)
			{
				course_scoreboard_render_card(8, 80 + 160, 1, false);
				course_scoreboard_render_header(8, 80 + 160, 18);
				course_scoreboard_render_record(selected_profile, profile_course, 0, true, 8, 80 + 160, al_get_bitmap_height(scorecard_pic) - 1, 18, 1);
			}
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_PLAYER_SETUP:
		{
			vgolf_render_title_bg();
//			shadow_box(NULL, 32, 32, 640 - 32, 480 - 32, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
//			shadow_box(NULL, 32 + 8, 32 + 8, 640 - 32 - 8, 480 - 32 - 8, 1, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				double_shadow_box(NULL, px[i] + 8 + 8, py[i] + 8 + 8, px[i] + 213 - 8 - 8, py[i] + 160 - 8 - 8, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
				hyperlink_page_render(player_page_pointer[i], NULL);
			}
			al_hold_bitmap_drawing(true);
			if(vgolf_client_game->player_count < VGOLF_MAX_PLAYERS)
			{
				hyperlink_page_render(player_add_page, NULL);
			}
//			if(!vgolf_client || vgolf_client->master)
//			{
				hyperlink_page_render(player_global_page, NULL);
//			}
//			else
//			{
//				hyperlink_page_render(player_global_slave_page, NULL);
//			}
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_PLAYER_AVATAR:
		{
			vgolf_render_title_bg();
			double_shadow_box(NULL, 320 - 196, 240 - 152, 320 + 196 - 1, 240 + 152 - 1, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			al_hold_bitmap_drawing(true);
			hyperlink_page_render(player_avatar_page, NULL);
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_DLC_AVATAR:
		{
			double_shadow_box(NULL, 144, 80, 144 + 320 + 24 - 1, 80 + 256 + 24 + 24 + 8 - 1, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			al_hold_bitmap_drawing(true);
			hyperlink_page_render(player_dlc_avatar_page, NULL);
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_PLAYER_ACCOUNT:
		{
			vgolf_render_title_bg();
			double_shadow_box(NULL, 112, 32, 532 - 1, 448 - 1, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			al_hold_bitmap_drawing(true);
			hyperlink_page_render(player_account_page, NULL);
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_PLAYER_NEW:
		{
			vgolf_render_title_bg();
			double_shadow_box(NULL, 144, 80, 144 + 320 + 24 - 1, 80 + 256 + 24 + 24 + 8 - 1, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			al_hold_bitmap_drawing(true);
			hyperlink_page_render(player_new_account_page, NULL);
			if(vgolf_ticks % 30 < 15)
			{
				outline_text_shadow(vgolf_font[VGOLF_FONT_96], 320 + 2 + t3f_get_text_width(vgolf_font[VGOLF_FONT_96], entered_text) / 2 + 4, player_new_account_page->element[1].oy, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "_");
			}
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_COURSE:
		{
			vgolf_render_title_bg();
			if(newmenu_controls_active)
			{
				double_shadow_box(NULL, t3f_virtual_display_width / 2 - vgolf_get_bitmap_width(vgolf_title_course_preview) / 2 - 32 - 64, 32, t3f_virtual_display_width / 2 + vgolf_get_bitmap_width(vgolf_title_course_preview) / 2 + 32 + 64 - 1, t3f_virtual_display_height - 32 - 1, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
				al_hold_bitmap_drawing(true);
				hyperlink_page_render(course_page, NULL);
				al_hold_bitmap_drawing(false);
        	}
			break;
		}
		case VGOLF_STATE_SUMMARY:
		{
			vgolf_render_title_bg();
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				double_shadow_box(NULL, px[i] + 8 + 8, py[i] + 8 + 8, px[i] + 213 - 8 - 8, py[i] + 160 - 8 - 8, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
				if(vgolf_player_list[i])
				{
					y = -1;
					outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], px[i] + 213 / 2, py[i] + 160 / 2 - 40 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]), -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), vgolf_client_game->player[i]->name);
					a = vgolf_find_avatar(vgolf_client_game->player[i]->selected_content[VGOLF_CONTENT_AVATARS]);
					if(a < 0)
					{
						a = 0;
					}
					if(a >= 0)
					{
						y = py[i] + 160 / 2 - vgolf_get_bitmap_height(game_player[i].avatar->image[0]) / 2;
						t3f_draw_animation(game_player[i].avatar->image[0], t3f_color_black, 0, px[i] + 213 / 2 - vgolf_get_bitmap_width(game_player[i].avatar->image[0]) - 4 - 2, y + 2, 0.0, 0);
						t3f_draw_animation(game_player[i].avatar->image[0], t3f_color_white, 0, px[i] + 213 / 2 - vgolf_get_bitmap_width(game_player[i].avatar->image[0]) - 4, y, 0.0, 0);
					}
					a = vgolf_find_course(vgolf_client_game->player[i]->selected_content[VGOLF_CONTENT_COURSES]);
					if(a >= 0)
					{
						if(y < 0)
						{
							y = py[i] + 160 / 2 - vgolf_get_bitmap_height(game_player[i].course_preview) / 2;
						}
						t3f_draw_scaled_animation(game_player[i].course_preview, t3f_color_black, 0, px[i] + 213 / 2 + 4 - 2, y + 2, 0.0, 0.25, 0);
						t3f_draw_scaled_animation(game_player[i].course_preview, t3f_color_white, 0, px[i] + 213 / 2 + 4, y, 0.0, 0.25, 0);
					}
				}
			}
			al_hold_bitmap_drawing(true);
			if(!vgolf_client || vgolf_client->master)
			{
				hyperlink_page_render(course_global_page, NULL);
			}
			else
			{
				hyperlink_page_render(course_global_slave_page, NULL);
			}
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_CREDITS:
		{
			al_clear_to_color(menu_bg_color);
			al_hold_bitmap_drawing(true);
			credits_render(NULL, &vgolf_credits);
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_LEADERBOARDS:
		{
			vgolf_render_title_bg();
			double_shadow_box(NULL, 132, 32, 640 - 132 - 1, 32 + 384 + 32, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			al_hold_bitmap_drawing(true);
			hyperlink_page_render(leaderboard_page, NULL);
			hyperlink_page_render(leaderboard_global_page, NULL);
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_DLC:
		{
			vgolf_render_title_bg();
//			shadow_box(NULL, 132, 32, 640 - 132 - 1, 32 + 384 + 32, 2, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
//			shadow_box(NULL, 132 + 8, 32 + 8, 640 - 132 - 8 - 1, 32 + 384 + 32 - 8, 1, al_map_rgba(0, 0, 0, 64), al_map_rgba(0, 0, 0, 64));
			al_hold_bitmap_drawing(true);
			hyperlink_page_render(dlc_page, NULL);
			al_hold_bitmap_drawing(false);
			break;
		}
		case VGOLF_STATE_LOAD:
		{
			al_clear_to_color(t3f_color_white);
			break;
		}
		case VGOLF_STATE_NONE:
		{
			al_clear_to_color(t3f_color_white);
			break;
		}
		case VGOLF_STATE_PRIVACY:
		{
			const char * privacy_text = "If you submit scores to the leaderboards, naturally your profile name, score, and course selection can be seen by others who access the leaderboards.\n\nIf you host an onlime game, your server name as well as your configured nick name and profile names will be visible to others who connect to your server.\n\nIf you Join an online game server, your nick name and profile name will be visible to the host as well as others connected to the server.\n\nOnline games may produce chat logs which are potentially stored on the host's as well as clients' devices. These chat logs contain the nick names and text content of those participating in the chat system.\n\nReplay data may be produced and stored during gameplay. This replay data contains the profile names and controller input of all participating players. When playing online games, this means replay data may potentially be stored on each client's device, as well as the host's.";

			al_hold_bitmap_drawing(true);
			vgolf_render_title_bg();
			al_hold_bitmap_drawing(false);
			al_draw_filled_rectangle(0.0, 0.0, 640.0, 480.0, al_map_rgba_f(0.0, 0.0, 0.0, 0.5));
			al_hold_bitmap_drawing(true);
			t3f_draw_text(vgolf_font[VGOLF_FONT_32], al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 16.0 - 1, 16.0 + 1, 0.0, 640.0 - 32.0, 0.0, 0, privacy_text);
			t3f_draw_text(vgolf_font[VGOLF_FONT_32], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 16.0, 16.0, 0.0, 640.0 - 32.0, 0.0, 0, privacy_text);
			hyperlink_page_render(sub_menu_page, NULL);
			al_hold_bitmap_drawing(false);
			break;
		}
		default:
		{
			vgolf_state_title_render();
			break;
		}
	}
	fx_fade_render();

	al_hold_bitmap_drawing(true);

	/* render start game timer */
	if(vgolf_start_game_timer > 0)
	{
		s = (vgolf_start_game_timer + 59) / 60;
		sprintf(text, "Game will start in %d second%s", s, s > 1 ? "s" : "");
		outline_text_shadow_center(vgolf_font[VGOLF_FONT_96], 320, 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), text);
	}

	/* render messages */
	oy = 1.0;
	for(i = 0; i < vgolf_messages.messages; i++)
	{
		if(vgolf_messages.message[i].time == 0)
		{
			ox = 1 + vgolf_chat_offset_x;
		}
		else
		{
			ox = 1;
		}
		t3f_create_text_line_data(&line_data, vgolf_font[VGOLF_FONT_32], vgolf_chat_width, vgolf_messages.message[i].tab, vgolf_messages.message[i].text);
		t3f_draw_text_lines(&line_data, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), ox - 1, oy + 1, 0.0);
		t3f_draw_text_lines(&line_data, vgolf_messages.message[i].color, ox, oy, 0.0);
		oy += t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) * line_data.lines;
	}

	/* render typed text */
	if(entering_text == 1)
	{
		outline_text_shadow(vgolf_font[VGOLF_FONT_32], 1, t3f_virtual_display_height - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) - 1, -1, 1, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, entered_text);
	}

	/* show chat message that user is typing */
	if(entering_text == 66)
	{
		t3f_create_text_line_data(&line_data, vgolf_font[VGOLF_FONT_32], 640.0, 0.0, entered_text);
		oy = t3f_virtual_display_height - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) * line_data.lines - 1;
		t3f_draw_text_lines(&line_data, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 1 - 1, oy + 1, 0.0);
		t3f_draw_text_lines(&line_data, al_map_rgb(255, 255, 255), 1, oy, 0.0);
		if(vgolf_ticks % 30 < 15)
		{
			if(line_data.lines > 0)
			{
				outline_text_shadow(vgolf_font[VGOLF_FONT_32], 1 + t3f_get_text_width(vgolf_font[VGOLF_FONT_32], line_data.line[line_data.lines - 1].text), 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) - 1, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "_");
			}
			else
			{
				outline_text_shadow(vgolf_font[VGOLF_FONT_32], 1 + t3f_get_text_width(vgolf_font[VGOLF_FONT_32], line_data.line[0].text), 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) - 1, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "_");
			}
		}
	}

	/* render chat interface */
	if(vgolf_client)
	{
		if(vgolf_chat_new_message)
		{
			sprintf(vgolf_chat_page_text, "< New Message");
		}
		else
		{
			sprintf(vgolf_chat_page_text, "<");
		}
		hyperlink_page_render(chat_page, NULL);
	}

	al_hold_bitmap_drawing(false);
}

int main(int argc, char * argv[])
{
	if(!vgolf_init(argc, argv))
	{
		printf("Failed to initialize game!\n");
		return 1;
	}

	nmenu[VGOLF_MENU_MAIN]->current_item = 0;
	current_newmenu = nmenu[VGOLF_MENU_TITLE];

	fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 2, NULL);
	t3f_run();
	vgolf_exit();
	t3f_show_resources();

	return 0;
}
