#include "t3f/sound.h"
#include "t3f/music.h"
#include "t3f/android.h"
#include "t3net/t3net.h"
#include "joynet/joynet.h"
#include "menu_proc.h"
#include "globals.h"
#include "init.h"
#include "version.h"
#include "message.h"
#include "hyperlink.h"
#include "newmenu.h"
#include "gameplay.h"
#include "replay.h"
#include "vfont.h"
#include "fx.h"
#include "config.h"
#include "network/server.h"
#include "network/client.h"
#include "network/callbacks.h"
#include "game_camera.h"

#include "core/utility.h"
#include "dlc.h"

/* hyperlink pages to supplement menus */
HYPERLINK_PAGE * player_page[VGOLF_MAX_PLAYERS] = {NULL};
HYPERLINK_PAGE * player_network_page[VGOLF_MAX_PLAYERS] = {NULL};
HYPERLINK_PAGE * player_start_page[VGOLF_MAX_PLAYERS] = {NULL};
HYPERLINK_PAGE * level_select_page = NULL;
HYPERLINK_PAGE * player_page_pointer[VGOLF_MAX_PLAYERS] = {NULL};
HYPERLINK_PAGE * player_add_page = NULL;
HYPERLINK_PAGE * player_global_page = NULL;
HYPERLINK_PAGE * player_global_slave_page = NULL;
HYPERLINK_PAGE * course_global_page = NULL;
HYPERLINK_PAGE * course_global_slave_page = NULL;
HYPERLINK_PAGE * player_avatar_page = NULL;
HYPERLINK_PAGE * player_dlc_avatar_page = NULL;
HYPERLINK_PAGE * player_account_page = NULL;
HYPERLINK_PAGE * player_new_account_page = NULL;
HYPERLINK_PAGE * profile_page = NULL;
HYPERLINK_PAGE * leaderboard_page = NULL;
HYPERLINK_PAGE * profile_global_page = NULL;
HYPERLINK_PAGE * leaderboard_global_page = NULL;
HYPERLINK_PAGE * sub_menu_page = NULL;
HYPERLINK_PAGE * course_page = NULL;
HYPERLINK_PAGE * course_select_page = NULL;
HYPERLINK_PAGE * chat_page = NULL;
HYPERLINK_PAGE * dlc_page = NULL;

/* in-game pages (pages used within the game logic) */
HYPERLINK_PAGE * ball_drop_page = NULL;
HYPERLINK_PAGE * take_gimme_page = NULL;
HYPERLINK_PAGE * scoreboard_page = NULL;
HYPERLINK_PAGE * game_complete_page = NULL;

/* fake in-game pages (used for latency hiding) */
HYPERLINK_PAGE * fake_ball_drop_page = NULL;
HYPERLINK_PAGE * fake_take_gimme_page = NULL;
HYPERLINK_PAGE * fake_scoreboard_page = NULL;
HYPERLINK_PAGE * fake_game_complete_page = NULL;

bool fake_page_hidden = false; // don't draw fake pages when this is set to 'true'

char player_page_text_buffer[VGOLF_MAX_PLAYERS][256];
char course_page_name[256] = {0};
char course_page_par[256] = {0};
char course_page_holes[256] = {0};

NEWMENU * nmenu[256] = {NULL};

int vgolf_player_choosing = 0;
int player_account_selected_page[VGOLF_MAX_PLAYERS] = {0};
int vgolf_avatar_selected = 0;

T3NET_LEADERBOARD * leaderboard = NULL;
int leaderboard_mode = 0;
char leaderboard_menu_text[32] = {0};

int profile_course = 0;
int selected_profile = 0;
int selected_course = 0;
int show_records = 0;
int profile_show_details = 0;
char profile_text[16][256] = {{0}};
char             temp_text[256] = {0};
int              last_commentary = -1;
bool vgolf_show_course_dlc = false;
int vgolf_selected_course_dlc = 0;
VGOLF_COURSE * vgolf_course_dlc_preview = NULL;
bool vgolf_show_avatar_dlc = false;
int vgolf_selected_avatar_dlc = 0;
T3F_ANIMATION * vgolf_avatar_dlc_preview = NULL;
int vgolf_selected_dlc_type = VGOLF_CONTENT_AVATARS;
int vgolf_selected_dlc_item = 0;

void vgolf_menu_proc_dlc_courses(void);
void vgolf_menu_proc_dlc_avatars(void);
int hl_dlc_download(int p, void * pp);
int vgolf_menu_proc_hide_fake_page(int i, void * p);

void gameplay_options_update(void)
{
	sprintf(current_newmenu->vtext[0], "%d", vgolf_config[CONFIG_DEAD_SPOT]);
}

void online_options_update(void)
{
	int i, blank = 1;

	if(vgolf_config[CONFIG_UPLOAD])
	{
		sprintf(current_newmenu->vtext[0], "Yes");
	}
	else
	{
		sprintf(current_newmenu->vtext[0], "No");
	}
	if(vgolf_config[CONFIG_DLC])
	{
		sprintf(current_newmenu->vtext[1], "Auto");
	}
	else
	{
		sprintf(current_newmenu->vtext[1], "Manual");
	}
	if(vgolf_config[CONFIG_LOW_BANDWIDTH])
	{
		sprintf(current_newmenu->vtext[2], "On");
	}
	else
	{
		sprintf(current_newmenu->vtext[2], "Off");
	}
	sprintf(current_newmenu->vtext[3], "%d %s%s", vgolf_config[CONFIG_INPUT_BUFFER], "Frame", vgolf_config[CONFIG_INPUT_BUFFER] > 1 ? "s" : "");
	if(entering_text)
	{
		sprintf(current_newmenu->vtext[4], "%s", entered_text);
	}
	else
	{
		for(i = 0; i < strlen(vgolf_nick); i++)
		{
			if(vgolf_nick[i] != ' ')
			{
				blank = 0;
			}
		}
		if(!blank)
		{
			sprintf(current_newmenu->vtext[4], "%s", vgolf_nick);
		}
		else
		{
			sprintf(current_newmenu->vtext[4], "Unnamed");
		}
	}
}

void network_options_update(void)
{
	if(entering_text == 2)
	{
		sprintf(current_newmenu->vtext[0], "%s", entered_text);
	}
	else
	{
		sprintf(current_newmenu->vtext[0], "%s", vgolf_nick);
	}
}

void video_options_update(void)
{
	char * gfx_text[3] = {"Window", "2x Window", "Fullscreen"};
	sprintf(current_newmenu->vtext[0], "%s", gfx_text[vgolf_config[CONFIG_GFX_MODE]]);
	sprintf(current_newmenu->vtext[1], "%s", vgolf_config[CONFIG_HD_TEXTURES] ? "High" : "Standard");
}

void audio_options_update(void)
{
	sprintf(current_newmenu->vtext[0], "%d%%", vgolf_config[CONFIG_MUSIC_VOLUME]);
	sprintf(current_newmenu->vtext[1], "%d%%", vgolf_config[CONFIG_SFX_VOLUME]);
	sprintf(current_newmenu->vtext[2], "%d%%", vgolf_config[CONFIG_COMMENTARY_VOLUME]);
	sprintf(current_newmenu->vtext[3], "%s", vgolf_commentaries.name[vgolf_config[CONFIG_COMMENTARY]]);
}

void commentary_options_update(void)
{
	sprintf(current_newmenu->vtext[0], "%d%%", vgolf_config[CONFIG_COMMENTARY_VOLUME]);
	sprintf(current_newmenu->vtext[1], "%s", vgolf_commentaries.name[vgolf_config[CONFIG_COMMENTARY]]);
}

void profile_name_update(void)
{
	sprintf(current_newmenu->vtext[0], "%s", entered_text);
}

void host_name_update(void)
{
	if(strlen(entered_text) < 127)
	{
		strcpy(current_newmenu->item[1].vtext, entered_text);
	}
}

int hl_player_back(int p, void * pp)
{
	int i;

	play_esc_sample();

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
		{
			joynet_disconnect_from_game(vgolf_client_game, 0, i);
		}
	}
	vgolf_state = VGOLF_STATE_MENU;
	return 1;
}

int hl_player_next(int p, void * pp)
{
	int i;

	play_select_sample();

	/* need at least 1 local player to be able to go to the course select screen */
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
		{
			break;
		}
	}
	if(i == VGOLF_MAX_PLAYERS)
	{
		ncdmsg_add(&vgolf_messages, "No Players!", al_map_rgb(255, 255, 255), 0.0, 300);
		return 0;
	}
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
		{
			joynet_select_game_content(vgolf_client_game, i, VGOLF_CONTENT_COURSES, vgolf_client_game->content_list[VGOLF_CONTENT_COURSES]->hash[selected_course]);
			game_player[i].setup_step = VGOLF_PLAYER_SETUP_COURSE;
			joynet_update_player_options(vgolf_client_game, i);
		}
	}
	vgolf_selected_course = vgolf_client_game->content_list[VGOLF_CONTENT_COURSES]->hash[selected_course];
	joynet_update_game_options(vgolf_client_game);
	vgolf_show_course_dlc = false; // start on installed courses page
	create_course_page();
	vgolf_state = VGOLF_STATE_COURSE;
	course_page->state.left_click = 2;
	return 1;
}

int hl_player_cancel(int p, void * pp)
{
	play_esc_sample();
	joynet_disconnect_from_game(vgolf_client_game, 0, vgolf_player_choosing);
	vgolf_state = VGOLF_STATE_PLAYER_SETUP;
	return 1;
}

int hl_player_name(int p, void * pp)
{
	int i = -1, j;
//	play_select_sample();
	for(j = 0; j < VGOLF_MAX_PLAYERS; j++)
	{
		if(pp == player_page[j])
		{
			i = j;
			break;
		}
	}
	if(i >= 0)
	{
		vgolf_player_choosing = i;
		vgolf_state = VGOLF_STATE_PLAYER_ACCOUNT;
	}
	player_account_selected_page[i] = 0;
	create_account_page(i);
	return 1;
}

int hl_add_player(int p, void * pp)
{
	int i;

	play_select_sample();
	joynet_connect_to_game(vgolf_client_game, 0, -1);
	for(i = 0; i < vgolf_client_game->players; i++)
	{
		player_page_pointer[i]->state.left_click = 2;
	}
	return 1;
}

int hl_player_avatar(int p, void * pp)
{
	int i = -1;
	play_select_sample();
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(pp == player_page[i])
		{
			break;
		}
	}
	if(i < VGOLF_MAX_PLAYERS)
	{
		vgolf_player_choosing = i;
		create_avatar_page();
		vgolf_state = VGOLF_STATE_PLAYER_AVATAR;
	}
	return 1;
}

int hl_player_avatar_previous(int p, void * pp)
{
	play_select_sample();
	vgolf_avatar_selected--;
	if(vgolf_avatar_selected < 0)
	{
		vgolf_avatar_selected = vgolf_client_game->content_list[VGOLF_CONTENT_AVATARS]->count - 1;
	}
	create_avatar_page();
	return 1;
}

int hl_player_avatar_next(int p, void * pp)
{
	play_select_sample();
	vgolf_avatar_selected++;
	if(vgolf_avatar_selected >= vgolf_client_game->content_list[VGOLF_CONTENT_AVATARS]->count)
	{
		vgolf_avatar_selected = 0;
	}
	create_avatar_page();
	return 1;
}

int hl_player_avatar_okay(int p, void * pp)
{
	play_select_sample();
	joynet_select_game_content(vgolf_client_game, vgolf_player_choosing, VGOLF_CONTENT_AVATARS, vgolf_client_game->content_list[VGOLF_CONTENT_AVATARS]->hash[vgolf_avatar_selected]);
	return 1;
}

int hl_player_dlc_avatar_cancel(int p, void * pp)
{
	play_esc_sample();
	vgolf_state = VGOLF_STATE_PLAYER_AVATAR;
	return 1;
}

static char avatar_menu_text[10][256] = {{0}};

static void choose_profile(int player, int profile)
{
	int i;

	vgolf_avatar_selected = 0;
	game_player[player].profile = profile;
	strcpy(player_page_pointer[player]->element[0].data, vgolf_profiles.item[game_player[profile].profile].name);

	/* update name through JoyNet */
	strcpy(vgolf_client_game->player[player]->name, vgolf_profiles.item[game_player[player].profile].name);
	joynet_update_player_options(vgolf_client_game, player);
	if(game_player[player].profile > 0)
	{
		game_player[player].savatar = vgolf_profiles.item[game_player[player].profile].avatar;
	}
	else
	{
		game_player[player].savatar = vgolf_avatars.sum[0];
	}
	for(i = 0; i < vgolf_client_game->content_list[VGOLF_CONTENT_AVATARS]->count; i++)
	{
		if(vgolf_client_game->content_list[VGOLF_CONTENT_AVATARS]->hash[i] == game_player[player].savatar)
		{
			vgolf_avatar_selected = i;
			break;
		}
	}
}

int hl_player_account_name_proc(int p, void * pp)
{
	return 1;
}

#ifdef T3F_ANDROID
	static void vgolf_edit_box_new_account_callback(void * data)
	{
		hl_player_add_account(0, NULL);
	}
#endif

int hl_player_account_pick(int p, void * pp)
{
	if(((HYPERLINK_PAGE *)(pp))->hover_element - 1 == 0)
	{
		#ifndef T3F_ANDROID
			strcpy(entered_text, "");
			entered_text_pos = 0;
			entering_text = 3;
			newmenu_disable_controls();
			t3f_clear_keys();
			vgolf_state = VGOLF_STATE_PLAYER_NEW;
		#else
			t3f_open_edit_box("Enter Profile Name", entered_text, 64, "CapWords", vgolf_edit_box_new_account_callback, NULL);
		#endif
	}
	else
	{
		choose_profile(vgolf_player_choosing, ((HYPERLINK_PAGE *)(pp))->hover_element - 2 + player_account_selected_page[vgolf_player_choosing] * 5);
		vgolf_state = VGOLF_STATE_PLAYER_SETUP;
		hl_player_avatar(vgolf_player_choosing, player_page_pointer[vgolf_player_choosing]);
	}
	return 1;
}

int hl_player_account_next_page(int p, void * pp)
{
	play_select_sample();
	player_account_selected_page[vgolf_player_choosing]++;
	create_account_page(vgolf_player_choosing);
	return 1;
}
int hl_player_account_previous_page(int p, void * pp)
{
	play_select_sample();
	player_account_selected_page[vgolf_player_choosing]--;
	create_account_page(vgolf_player_choosing);
	return 1;
}

int hl_profile_back(int p, void * pp)
{
	play_esc_sample();
	if(vgolf_state == VGOLF_STATE_LEADERBOARDS && leaderboard_mode == 1)
	{
		t3f_play_music("data/music/menu.xm");
	}
	if(vgolf_hosting)
	{
		current_newmenu = nmenu[VGOLF_MENU_MAIN_HOST];
	}
	else if(vgolf_client)
	{
		current_newmenu = nmenu[VGOLF_MENU_MAIN_JOIN];
	}
	else
	{
		current_newmenu = nmenu[VGOLF_MENU_MAIN];
	}
	vgolf_state = VGOLF_STATE_MENU;
	return 1;
}

int hl_profile_delete(int p, void * pp)
{
	if(selected_profile > 0)
	{
		play_select_sample();
		current_newmenu = nmenu[VGOLF_MENU_PROFILE_DELETE];
		vgolf_state = VGOLF_STATE_PROFILE_DELETE;
	}
	else
	{
		play_select_sample();
		ncdmsg_add(&vgolf_messages, "Cannot delete guest account!", al_map_rgb(255, 255, 255), 0.0, 300);
	}
	return 1;
}

int hl_sub_menu_back(int p, void * pp)
{
	if(vgolf_state == VGOLF_STATE_PRIVACY)
	{
		vgolf_state = VGOLF_STATE_MENU;
		play_esc_sample();
	}
	else if(current_newmenu->parent)
	{
		current_newmenu = (NEWMENU *)current_newmenu->parent;
		current_newmenu->current_item = -1;
		play_esc_sample();
	}
	else if(current_newmenu == nmenu[VGOLF_MENU_MAIN])
	{
		current_newmenu = nmenu[VGOLF_MENU_TITLE];
		play_esc_sample();
	}
	else if(current_newmenu == nmenu[VGOLF_MENU_HOST_NAME])
	{
		current_newmenu = nmenu[VGOLF_MENU_PLAY_ONLINE];
		play_esc_sample();
	}
	else if(current_newmenu == nmenu[VGOLF_MENU_NICK])
	{
		current_newmenu = nmenu[VGOLF_MENU_OPTIONS_ONLINE];
		play_esc_sample();
	}
	else if(current_newmenu == nmenu[VGOLF_MENU_HOST_LIST])
	{
		current_newmenu = nmenu[VGOLF_MENU_PLAY_ONLINE];
		play_esc_sample();
	}
	return 1;
}

int hl_profile_previous(int p, void * pp)
{
	play_select_sample();
	selected_profile--;
	create_profile_page(selected_profile);
	return 1;
}

int hl_profile_next(int p, void * pp)
{
	play_select_sample();
	selected_profile++;
	create_profile_page(selected_profile);
	return 1;
}

int hl_leaderboard_previous(int p, void * pp)
{
	play_select_sample();
	profile_course--;
	create_leaderboard_page();
	return 1;
}

int hl_leaderboard_next(int p, void * pp)
{
	play_select_sample();
	profile_course++;
	create_leaderboard_page();
	return 1;
}

int hl_profile_course_previous(int p, void * pp)
{
	play_select_sample();
	profile_course--;
	create_profile_page(selected_profile);
	return 1;
}

int hl_profile_course_next(int p, void * pp)
{
	play_select_sample();
	profile_course++;
	create_profile_page(selected_profile);
	return 1;
}

int hl_course_select_previous(int p, void * pp)
{
	int i;

	play_select_sample();
	if(vgolf_show_course_dlc)
	{
		vgolf_selected_course_dlc--;
	}
	else
	{
		selected_course--;
		for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
		{
			if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
			{
				joynet_select_game_content(vgolf_client_game, i, VGOLF_CONTENT_COURSES, vgolf_client_game->content_list[VGOLF_CONTENT_COURSES]->hash[selected_course]);
			}
		}
		vgolf_selected_course = vgolf_client_game->content_list[VGOLF_CONTENT_COURSES]->hash[selected_course];
//		joynet_update_game_options(vgolf_client_game);
	}
	create_course_page();
	return 1;
}

int hl_course_select_next(int p, void * pp)
{
	int i;

	play_select_sample();
	if(vgolf_show_course_dlc)
	{
		vgolf_selected_course_dlc++;
	}
	else
	{
		selected_course++;
		for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
		{
			if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
			{
				joynet_select_game_content(vgolf_client_game, i, VGOLF_CONTENT_COURSES, vgolf_client_game->content_list[VGOLF_CONTENT_COURSES]->hash[selected_course]);
			}
		}
		vgolf_selected_course = vgolf_client_game->content_list[VGOLF_CONTENT_COURSES]->hash[selected_course];
		joynet_update_game_options(vgolf_client_game);
	}
	create_course_page();
	return 1;
}

int hl_course_dlc_toggle(int p, void * pp)
{
	play_select_sample();
	vgolf_show_course_dlc = !vgolf_show_course_dlc;
	vgolf_selected_course_dlc = 0;
	create_course_page();
	return 1;
}

int hl_avatar_dlc_toggle(int p, void * pp)
{
	play_select_sample();
	vgolf_show_avatar_dlc = !vgolf_show_avatar_dlc;
	vgolf_selected_avatar_dlc = 0;
	vgolf_avatar_selected = 0;
	create_avatar_page();
	return 1;
}

int hl_dlc_download(int p, void * pp)
{
	char dlc_path[1024] = {0};
	char buffer[1024];

	play_select_sample();
	switch(vgolf_selected_dlc_type)
	{
		case VGOLF_CONTENT_AVATARS:
		{
			strcpy(dlc_path, t3f_get_filename(t3f_data_path, "avatars/", buffer, 1024));
			al_stop_timer(t3f_timer);
			if(vgolf_download_dlc(vgolf_dlc_avatars, vgolf_selected_dlc_item, dlc_path))
			{
				vgolf_free_avatar_database();
				vgolf_build_avatar_database();
				vgolf_reset_content_lists();
				if(!vgolf_create_dlc_page())
				{
					vgolf_state = VGOLF_STATE_MENU;
				}
			}
			else
			{
				ncdmsg_add(&vgolf_messages, "Error downloading avatar!", al_map_rgb(255, 255, 255), 0.0, 300);
			}
			al_start_timer(t3f_timer);
			break;
		}
		case VGOLF_CONTENT_COURSES:
		{
			strcpy(dlc_path, t3f_get_filename(t3f_data_path, "courses/", buffer, 1024));
			al_stop_timer(t3f_timer);
			if(vgolf_download_dlc(vgolf_dlc_courses, vgolf_selected_dlc_item, dlc_path))
			{
				vgolf_free_course_database();
				vgolf_build_course_database();
				vgolf_reset_content_lists();
				if(!vgolf_create_dlc_page())
				{
					vgolf_state = VGOLF_STATE_MENU;
				}
			}
			else
			{
				ncdmsg_add(&vgolf_messages, "Error downloading course!", al_map_rgb(255, 255, 255), 0.0, 300);
			}
			al_start_timer(t3f_timer);
			break;
		}
		case VGOLF_CONTENT_COMMENTARIES:
		{
			strcpy(dlc_path, t3f_get_filename(t3f_data_path, "commentaries/", buffer, 1024));
			al_stop_timer(t3f_timer);
			if(vgolf_download_dlc(vgolf_dlc_commentaries, vgolf_selected_dlc_item, dlc_path))
			{
				vgolf_build_commentary_database();
				if(!vgolf_create_dlc_page())
				{
					vgolf_state = VGOLF_STATE_MENU;
				}
			}
			else
			{
				ncdmsg_add(&vgolf_messages, "Error downloading commentary!", al_map_rgb(255, 255, 255), 0.0, 300);
			}
			al_start_timer(t3f_timer);
			break;
		}
	}
	return 1;
}

void vgolf_destroy_dlc_preview_callback(void * data)
{
	t3f_destroy_animation((T3F_ANIMATION *)data);
}

void vgolf_destroy_course_dlc_preview_callback(void * data)
{
	vgolf_destroy_course((VGOLF_COURSE *)data);
}

void vgolf_destroy_dlc_lists(void)
{
	if(vgolf_dlc_avatars)
	{
		t3net_destroy_dlc_list(vgolf_dlc_avatars, vgolf_destroy_dlc_preview_callback);
		vgolf_dlc_avatars = NULL;
	}
	if(vgolf_dlc_courses)
	{
		t3net_destroy_dlc_list(vgolf_dlc_courses, vgolf_destroy_course_dlc_preview_callback);
		vgolf_dlc_courses = NULL;
	}
}

int hl_dlc_cancel(int p, void * pp)
{
	vgolf_destroy_dlc_lists();
	play_esc_sample();
	vgolf_state = VGOLF_STATE_MENU;
	return 1;
}

int hl_game_complete_finished(int p, void * pp)
{
	play_select_sample();
	main_game_vars.quit_game = 1;
	main_game_vars.quit_title = 0;
	vgolf_skip_leaderboard = false;
	joynet_end_game(vgolf_client_game);
	return 1;
}

int hl_game_next_hole(int p, void * pp)
{
	play_select_sample();
	if(main_game_vars.cur_hole < game_course->num_holes - 1)
	{
		//increment to next hole
		main_game_vars.cur_hole++;

		//load hole data
		memcpy(&game_course->current_hole, &game_course->hole[main_game_vars.cur_hole], sizeof(HOLE));

		vgolf_initialize_hole(&game_course->current_hole);       //initialize the next hole before we start it
		vgolf_hole_initialize_players();   //initialize the players before starting the next hole
		vgolf_start_scoreboard_segout();                //start the seg-out sequence
		locally_block_click = 1;
		vgolf_local_club_blocked = true;
		vgolf_reset_camera();
	}
	else
	{
		//last hole has been passed, go to victory screen
		if(vgolf_player_count == 1)
		{
			single_player_course_completed_logic(&main_game_vars, game_player, game_course);
		}
		else
		{
			multi_player_course_completed_logic(&main_game_vars, game_player, vgolf_player_count, game_course);
		}
		if(vgolf_config[CONFIG_UPLOAD] && vgolf_replay_mode != VGOLF_REPLAY_MODE_PLAY)
		{
			vgolf_handle_score_upload();
		}
		else
		{
			start_victory_screen_segin(&main_game_vars, game_course, game_player, vgolf_player_count);
		}
	}
	return 1;
}

int hl_game_toggle_records(int p, void * pp)
{
	play_select_sample();
	if(show_records)
	{
		show_records = 0;
	}
	else
	{
		show_records = 1;
	}
	create_scoreboard_page();
	return 1;
}

int hl_game_quit(int p, void * pp)
{
	play_esc_sample();
	joynet_pause_game(vgolf_client_game);
	return 1;
}

int hl_course_back(int p, void * pp)
{
	int i;

	play_esc_sample();
	if(vgolf_title_course_preview)
	{
		t3f_destroy_animation(vgolf_title_course_preview);
		vgolf_title_course_preview = NULL;
	}
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			player_page_pointer[i] = player_page[i];
		}
		else
		{
			player_page_pointer[i] = player_start_page[i];
		}
	}
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
		{
			game_player[i].setup_step = VGOLF_PLAYER_SETUP_START;
			joynet_update_player_options(vgolf_client_game, i);
		}
	}
	vgolf_state = VGOLF_STATE_PLAYER_SETUP;

	return 1;
}

int hl_avatar_back(int p, void * pp)
{
	play_esc_sample();
	vgolf_state = VGOLF_STATE_PLAYER_ACCOUNT;

	return 1;
}

int hl_course_next(int p, void * pp)
{
	int i;

	play_select_sample();
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
		{
			if(vgolf_title_course_preview)
			{
				game_player[i].course_preview = t3f_clone_animation(vgolf_title_course_preview);
			}
			game_player[i].setup_step = VGOLF_PLAYER_SETUP_SUMMARY;
			joynet_update_player_options(vgolf_client_game, i);
		}
	}
	if(vgolf_title_course_preview)
	{
		t3f_destroy_animation(vgolf_title_course_preview);
		vgolf_title_course_preview = NULL;
	}
	course_global_page->state.left_click = 2;
	vgolf_state = VGOLF_STATE_SUMMARY;
	return 1;
}

int hl_summary_back(int p, void * pp)
{
	ENetPacket * packet;
	play_esc_sample();

	/* cancel countdown */
	if(vgolf_start_game_timer > 0 && vgolf_client && vgolf_client->master)
	{
		vgolf_start_game_timer = 0; // cancel it here before message propogates to prevent possibility of game starting anyway
		packet = joynet_create_packet(VGOLF_GAME_MESSAGE_CANCEL_START, NULL);
		enet_peer_send(vgolf_client->peer, JOYNET_CHANNEL_GAME, packet);
	}
	create_course_page();
	vgolf_state = VGOLF_STATE_COURSE;
	return 1;
}

int hl_summary_play(int p, void * pp)
{
	ENetPacket * packet;
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && game_player[i].setup_step != VGOLF_PLAYER_SETUP_SUMMARY)
		{
			break;
		}
	}
	if(i == VGOLF_MAX_PLAYERS)
	{
		t3f_play_sample(main_game_vars.sound_bank.sfx_title_swing, 1.0, 0.0, 1.0);
		play_proc();
	}

	/* some players still setting up (should only get here when playing online as master) */
	else if(vgolf_client && vgolf_client->master)
	{
		/* send start request to begin timer countdown */
		packet = joynet_create_packet(VGOLF_GAME_MESSAGE_WANT_START, NULL);
		enet_peer_send(vgolf_client->peer, JOYNET_CHANNEL_GAME, packet);
//		ncdmsg_add(&vgolf_messages, "Player(s) still selecting!", al_map_rgb(255, 255, 255), 0.0, 300);
	}
	return 1;
}

int hl_profile_toggle_details(int p, void * pp)
{
	play_select_sample();
	if(profile_show_details)
	{
		profile_show_details = 0;
	}
	else
	{
		profile_show_details = 1;
	}
	create_profile_page(selected_profile);
	return 1;
}

void create_account_page(int player)
{
	int i, line_pos = 1;
	int oy = t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2 - 4;

	if(player_account_page)
	{
		hyperlink_page_destroy(player_account_page);
	}
	player_account_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(player_account_page, NULL, "Select Profile", &vgolf_font[VGOLF_FONT_96], 320 + 2, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * line_pos + 4, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	line_pos += 2;
	if(vgolf_profiles.items < 6)
	{
		hyperlink_page_add_element_text(player_account_page, hl_player_account_pick, "New Player", &vgolf_font[VGOLF_FONT_96], 320 + 2, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * line_pos - oy, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		line_pos++;
	}
	for(i = 0; i < 6 && i < vgolf_profiles.items; i++)
	{
		hyperlink_page_add_element_text(player_account_page, hl_player_account_pick, vgolf_profiles.item[i].name, &vgolf_font[VGOLF_FONT_96], 320 + 2, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * line_pos - oy, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		line_pos++;
	}
	hyperlink_page_add_element_text(player_account_page, hl_player_cancel, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
}

void create_avatar_page(void)
{
	int a;

	a = vgolf_find_avatar(vgolf_client_game->content_list[VGOLF_CONTENT_AVATARS]->hash[vgolf_avatar_selected]);
	if(player_avatar_page)
	{
		hyperlink_page_destroy(player_avatar_page);
	}
	player_avatar_page = hyperlink_page_create(0, 0);
	if(game_player[vgolf_player_choosing].avatar)
	{
		vgolf_destroy_avatar(game_player[vgolf_player_choosing].avatar);
		game_player[vgolf_player_choosing].avatar = NULL;
	}
	game_player[vgolf_player_choosing].avatar = vgolf_load_avatar(vgolf_avatars.fn[a], NULL, vgolf_avatars.fi[a]);
	hyperlink_page_add_element_text(player_avatar_page, NULL, "Select Avatar", &vgolf_font[VGOLF_FONT_96], 320 + 2, 92, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	if(game_player[vgolf_player_choosing].avatar)
	{
		hyperlink_page_add_element_animation(player_avatar_page, NULL, &(game_player[vgolf_player_choosing].avatar->image[0]), 320, 240, HYPERLINK_ELEMENT_FLAG_SHADOW | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_2X);
	}
	hyperlink_page_add_element_text(player_avatar_page, hl_player_avatar_previous, "<", &vgolf_font[VGOLF_FONT_96], 320 - 64 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "<"), 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(player_avatar_page, hl_player_avatar_next, ">", &vgolf_font[VGOLF_FONT_96], 320 + 96, 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(player_avatar_page, hl_player_avatar_okay, "Okay", &vgolf_font[VGOLF_FONT_96], 320 + 2, 480 - 92 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(player_avatar_page, hl_avatar_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
}

void create_profile_page(int profile)
{
	if(profile_page)
	{
		hyperlink_page_destroy(profile_page);
	}
	profile_page = hyperlink_page_create(0, 0);

	/* name of profile */
	hyperlink_page_add_element_text(profile_page, NULL, vgolf_profiles.item[profile].name, &vgolf_font[VGOLF_FONT_48], 320 + 2, 80, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	if(selected_profile > 0)
	{
		hyperlink_page_add_element_text(profile_page, hl_profile_previous, "<", &vgolf_font[VGOLF_FONT_48], 320 + 2 - 120, 80, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
	if(selected_profile < vgolf_profiles.items - 1)
	{
		hyperlink_page_add_element_text(profile_page, hl_profile_next, ">", &vgolf_font[VGOLF_FONT_48], 320 + 2 + 120, 80, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}

	/* name of selected course */
	if(profile_course < 0)
	{
		/* total stats */
		hyperlink_page_add_element_text(profile_page, hl_profile_course_next, ">", &vgolf_font[VGOLF_FONT_48], 320 + 2 + 152, 80 + 64, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(profile_page, NULL, "Statistics", &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 64, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		sprintf(profile_text[5], "Strokes: %d", vgolf_profiles.item[profile].total.data[VGOLF_PROFILE_DATA_STROKES]);
		hyperlink_page_add_element_text(profile_page, NULL, profile_text[5], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 128, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		sprintf(profile_text[6], "Pars: %d", vgolf_profiles.item[profile].total.data[VGOLF_PROFILE_DATA_PARS]);
		hyperlink_page_add_element_text(profile_page, NULL, profile_text[6], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 160, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		sprintf(profile_text[7], "Birdies: %d", vgolf_profiles.item[profile].total.data[VGOLF_PROFILE_DATA_BIRDIES]);
		hyperlink_page_add_element_text(profile_page, NULL, profile_text[7], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 192, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		sprintf(profile_text[8], "Eagles: %d", vgolf_profiles.item[profile].total.data[VGOLF_PROFILE_DATA_EAGLES]);
		hyperlink_page_add_element_text(profile_page, NULL, profile_text[8], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 224, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		sprintf(profile_text[9], "Albatrosses: %d", vgolf_profiles.item[profile].total.data[VGOLF_PROFILE_DATA_ALBATROSSES]);
		hyperlink_page_add_element_text(profile_page, NULL, profile_text[9], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 256, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		sprintf(profile_text[10], "Holes In One: %d", vgolf_profiles.item[profile].total.data[VGOLF_PROFILE_DATA_HOLES_IN_ONE]);
		hyperlink_page_add_element_text(profile_page, NULL, profile_text[10], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 288, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
	else
	{
		sprintf(profile_text[0], "%s", vgolf_courses.item[profile_course].name);
		hyperlink_page_add_element_text(profile_page, NULL, profile_text[0], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 64, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		if(profile_course >= 0)
		{
			hyperlink_page_add_element_text(profile_page, hl_profile_course_previous, "<", &vgolf_font[VGOLF_FONT_48], 320 + 2 - 152 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "<"), 80 + 64, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
		if(profile_course < vgolf_courses.items - 1)
		{
			hyperlink_page_add_element_text(profile_page, hl_profile_course_next, ">", &vgolf_font[VGOLF_FONT_48], 320 + 2 + 152, 80 + 64, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}

		if(profile_show_details)
		{
			sprintf(profile_text[4], "View Overall");
			hyperlink_page_add_element_text(profile_page, hl_profile_toggle_details, profile_text[4], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 288, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
		else
		{
			/* course stats */
			sprintf(profile_text[1], "Best Score: %d", vgolf_profiles.item[profile].course_info[profile_course].total_score);
			hyperlink_page_add_element_text(profile_page, NULL, profile_text[1], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 128, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
			sprintf(profile_text[2], "Wins: %d", vgolf_profiles.item[profile].course_info[profile_course].won);
			hyperlink_page_add_element_text(profile_page, NULL, profile_text[2], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 160, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
			sprintf(profile_text[3], "Losses: %d", vgolf_profiles.item[profile].course_info[profile_course].lost);
			hyperlink_page_add_element_text(profile_page, NULL, profile_text[3], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 192, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
			sprintf(profile_text[4], "Total Played: %d", vgolf_profiles.item[profile].course_info[profile_course].completed);
			hyperlink_page_add_element_text(profile_page, NULL, profile_text[4], &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 224, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
			hyperlink_page_add_element_text(profile_page, hl_profile_toggle_details, "View Details", &vgolf_font[VGOLF_FONT_48], 320 + 2, 80 + 288, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
	}
}

void create_leaderboard_page(void)
{
	char text[64] = {0};
	int i, j, score;
	int top[VGOLF_MAX_PLAYERS] = {-1, -1, -1, -1, -1, -1, -1, -1};
	ALLEGRO_COLOR color;

	sprintf(text, "%lu", vgolf_courses.item[profile_course].sum);
	if(leaderboard)
	{
		t3net_destroy_leaderboard(leaderboard);
	}
	al_stop_timer(t3f_timer);
	vgolf_spawn_client_keep_alive_thread();
	leaderboard = t3net_get_leaderboard("http://www.t3-i.com/t3net2/leaderboards/query.php", "vgolf_deluxe", "1.0", "normal", text, 10, 1);
	vgolf_kill_client_keep_alive_thread();
	al_start_timer(t3f_timer);
	if(!leaderboard)
	{
		vgolf_state = VGOLF_STATE_MENU;
		return;
	}
	if(leaderboard_page)
	{
		hyperlink_page_destroy(leaderboard_page);
	}
	leaderboard_page = hyperlink_page_create(0, 0);

	if(leaderboard_mode == 1)
	{
		for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
		{
			if(vgolf_player_list[i])
			{
				for(j = 0; j < leaderboard->entries; j++)
				{
					if(!strcasecmp(vgolf_client_game->player[i]->name, leaderboard->entry[j]->name) && (vgolf_player_course_stroke_total(&game_player[i], vgolf_leaderboard_num_holes) * 2 + 'v' + 'g' + 'o' + 'l' + 'f') == leaderboard->entry[j]->score)
					{
						top[i] = j;
						break;
					}
				}
			}
		}
	}

	/* course */
	hyperlink_page_add_element_text(leaderboard_page, NULL, vgolf_courses.item[profile_course].name, &vgolf_font[VGOLF_FONT_48], 320 + 2, 60, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	if(leaderboard_mode == 0)
	{
		if(profile_course > 0)
		{
			hyperlink_page_add_element_text(leaderboard_page, hl_leaderboard_previous, "<", &vgolf_font[VGOLF_FONT_48], 320 + 2 - 152 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "<"), 60, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
		if(profile_course < vgolf_courses.items - 1)
		{
			hyperlink_page_add_element_text(leaderboard_page, hl_leaderboard_next, ">", &vgolf_font[VGOLF_FONT_48], 320 + 2 + 152, 60, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
	}

	/* name of selected course */
	for(i = 0; i < 10; i++)
	{
		color = VGOLF_COLOR_MENU_OPTION;

		/* highlight top scores in mode 1 */
		if(leaderboard_mode == 1)
		{
			for(j = 0; j < VGOLF_MAX_PLAYERS; j++)
			{
				if(i == top[j])
				{
					color = al_map_rgb(255, 255, 0);
					break;
				}
			}
		}
		if(i < leaderboard->entries && strlen(leaderboard->entry[i]->name) > 0)
		{
			hyperlink_page_add_element_text(leaderboard_page, NULL, leaderboard->entry[i]->name, &vgolf_font[VGOLF_FONT_32], 160 + 2, 60 + 64 + 32 * i, color, color, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_SHADOW);
			score = (leaderboard->entry[i]->score - 'f' - 'l' - 'o' - 'g' - 'v') / 2;
			if(score == vgolf_courses.item[profile_course].par)
			{
				sprintf(profile_text[i], "E");
			}
			else if(score > vgolf_courses.item[profile_course].par)
			{
				sprintf(profile_text[i], "+%d", score - vgolf_courses.item[profile_course].par);
			}
			else
			{
				sprintf(profile_text[i], "%d", score - vgolf_courses.item[profile_course].par);
			}
			hyperlink_page_add_element_text(leaderboard_page, NULL, profile_text[i], &vgolf_font[VGOLF_FONT_32], 480 - t3f_get_text_width(vgolf_font[VGOLF_FONT_32], profile_text[i]) + 2, 60 + 64 + 32 * i, color, color, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
		else
		{
			hyperlink_page_add_element_text(leaderboard_page, NULL, "---", &vgolf_font[VGOLF_FONT_32], 320 + 2, 60 + 64 + 32 * i, color, color, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_SHADOW | HYPERLINK_ELEMENT_FLAG_CENTER);
		}
	}
}

void create_fake_scoreboard_page(void)
{
	float bar_height = 480.0 / 10.0;
	float bar_mid = bar_height / 2.0;

	if(fake_scoreboard_page)
	{
		hyperlink_page_destroy(fake_scoreboard_page);
	}
	fake_scoreboard_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(fake_scoreboard_page, vgolf_menu_proc_hide_fake_page, "Next >", &vgolf_font[VGOLF_FONT_48], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Next >") - 4 - 8, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(fake_scoreboard_page, NULL, "Menu", &vgolf_font[VGOLF_FONT_48], 2 + 8, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	if(show_records)
	{
//		hyperlink_page_add_element_text(fake_scoreboard_page, NULL, "View Scores", &vgolf_font[VGOLF_FONT_32], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_32], "View Scores") / 2 - 4, 480 - bar_height * 2.0 + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(fake_scoreboard_page, NULL, "View Scores", &vgolf_font[VGOLF_FONT_48], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "View Scores") / 2 - 4, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
	else
	{
//		hyperlink_page_add_element_text(fake_scoreboard_page, NULL, "View Records", &vgolf_font[VGOLF_FONT_32], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_32], "View Records") / 2 - 4, 480 - bar_height * 2.0 + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(fake_scoreboard_page, NULL, "View Records", &vgolf_font[VGOLF_FONT_48], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "View Records") / 2 - 4, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
}

void create_scoreboard_page(void)
{
	float bar_height = 480.0 / 10.0;
	float bar_mid = bar_height / 2.0;

	if(scoreboard_page)
	{
		hyperlink_page_destroy(scoreboard_page);
	}
	scoreboard_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(scoreboard_page, hl_game_next_hole, "Next >", &vgolf_font[VGOLF_FONT_48], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Next >") - 4 - 8, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(scoreboard_page, hl_game_quit, "Menu", &vgolf_font[VGOLF_FONT_48], 2 + 8, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	if(show_records)
	{
//		hyperlink_page_add_element_text(scoreboard_page, hl_game_toggle_records, "View Scores", &vgolf_font[VGOLF_FONT_32], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_32], "View Scores") / 2 - 4, 480 - bar_height * 2.0 + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(scoreboard_page, hl_game_toggle_records, "View Scores", &vgolf_font[VGOLF_FONT_48], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "View Scores") / 2 - 4, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
	else
	{
//		hyperlink_page_add_element_text(scoreboard_page, hl_game_toggle_records, "View Records", &vgolf_font[VGOLF_FONT_32], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_32], "View Records") / 2 - 4, 480 - bar_height * 2.0 + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(scoreboard_page, hl_game_toggle_records, "View Records", &vgolf_font[VGOLF_FONT_48], 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "View Records") / 2 - 4, 480 - bar_height + bar_mid - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2.0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
	create_fake_scoreboard_page();
}

void create_course_page(void)
{
	T3F_ANIMATION * ap;
	int c;

	c = vgolf_find_course(vgolf_selected_course);
	if(c >= 0)
	{
		ap = vgolf_load_course_preview(vgolf_courses.item[c].filename, vgolf_courses.item[c].fi);
		if(ap)
		{
			if(vgolf_title_course_preview)
			{
				t3f_destroy_animation(vgolf_title_course_preview);
			}
			vgolf_title_course_preview = ap;
		}
		else
		{
			printf("failed to load course preview\n");
			return;
		}
		if(course_page)
		{
			hyperlink_page_destroy(course_page);
		}
		course_page = hyperlink_page_create(0, 0);
		sprintf(course_page_name, "%s", vgolf_courses.item[c].name);
		sprintf(course_page_par, "Par: %d", vgolf_courses.item[c].par);
		sprintf(course_page_holes, "Holes: %d", vgolf_courses.item[c].holes);
		hyperlink_page_add_element_text(course_page, NULL, "Select Course", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - 2 - vgolf_get_bitmap_height(vgolf_title_course_preview) / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - 32, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		if(c > 0)
		{
			hyperlink_page_add_element_text(course_page, hl_course_select_previous, "<", &vgolf_font[VGOLF_FONT_96], 320 + 2 - 132 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "<"), 240 - 2 -  t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2 - 8, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
		if(c < vgolf_client_game->content_list[VGOLF_CONTENT_COURSES]->count - 1)
		{
			hyperlink_page_add_element_text(course_page, hl_course_select_next, ">", &vgolf_font[VGOLF_FONT_96], 320 + 2 + 132, 240 - 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2 - 8, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		}
		hyperlink_page_add_element_animation(course_page, NULL, &vgolf_title_course_preview, 320, 240 - 16, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW | HYPERLINK_ELEMENT_FLAG_BLIT);
		hyperlink_page_add_element_text(course_page, NULL, course_page_par, &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 + vgolf_get_bitmap_height(vgolf_title_course_preview) / 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(course_page, NULL, course_page_holes, &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 + vgolf_get_bitmap_height(vgolf_title_course_preview) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(course_page, hl_course_next, "Next >", &vgolf_font[VGOLF_FONT_96], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "Next >") - 4, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(course_page, hl_course_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
}

bool vgolf_dlc_installed(int type, unsigned long hash)
{
	int i;

	switch(vgolf_selected_dlc_type)
	{
		case VGOLF_CONTENT_AVATARS:
		{
			for(i = 0; i < vgolf_avatars.avatars; i++)
			{
				if(vgolf_avatars.sum[i] == hash)
				{
					return true;
				}
			}
			break;
		}
		case VGOLF_CONTENT_COURSES:
		{
			for(i = 0; i < vgolf_courses.items; i++)
			{
				if(vgolf_courses.item[i].sum == hash)
				{
					return true;
				}
			}
			break;
		}
		case VGOLF_CONTENT_COMMENTARIES:
		{
			for(i = 0; i < vgolf_commentaries.count; i++)
			{
				if(vgolf_commentaries.sum[i] == hash)
				{
					return true;
				}
			}
			break;
		}
	}
	return false;
}

int hl_dlc_select_previous(int i, void * p)
{
	play_select_sample();
	vgolf_selected_dlc_item--;
	al_stop_timer(t3f_timer);
	if(!vgolf_create_dlc_page())
	{
		vgolf_state = VGOLF_STATE_MENU;
	}
	al_start_timer(t3f_timer);
	return 1;
}

int hl_dlc_select_next(int i, void * p)
{
	play_select_sample();
	vgolf_selected_dlc_item++;
	al_stop_timer(t3f_timer);
	if(!vgolf_create_dlc_page())
	{
		vgolf_state = VGOLF_STATE_MENU;
	}
	al_start_timer(t3f_timer);
	return 1;
}

bool vgolf_create_dlc_page(void)
{
	const ALLEGRO_FILE_INTERFACE * old_interface;
	VGOLF_COURSE * cp = NULL;
	char buffer[1024];
	int i;

	switch(vgolf_selected_dlc_type)
	{
		case VGOLF_CONTENT_AVATARS:
		{
			if(vgolf_dlc_avatars)
			{

				for(i = 0; i < vgolf_dlc_avatars->items; i++)
				{
					if(vgolf_dlc_avatars->item[i]->preview)
					{
						t3f_destroy_animation(vgolf_dlc_avatars->item[i]->preview);
						vgolf_dlc_avatars->item[i]->preview = NULL;
					}
				}

				/* download preview */
				if(t3net_download_file(vgolf_dlc_avatars->item[vgolf_selected_dlc_item]->preview_url, t3f_get_filename(t3f_temp_path, "dlc_avatar.t3a", buffer, 1024)))
				{
					/* use stdio for reading downloaded content and revert back to whatever was in use */
					old_interface = al_get_new_file_interface();
					al_set_standard_file_interface();
					vgolf_dlc_avatars->item[vgolf_selected_dlc_item]->preview = t3f_load_animation(t3f_get_filename(t3f_temp_path, "dlc_avatar.t3a", buffer, 1024));
					al_set_new_file_interface(old_interface);
				}
				else
				{
					return false;
				}
				if(dlc_page)
				{
					hyperlink_page_destroy(dlc_page);
				}
				dlc_page = hyperlink_page_create(0, 0);
				if(dlc_page)
				{
					hyperlink_page_add_element_animation(dlc_page, NULL, (T3F_ANIMATION **)(&vgolf_dlc_avatars->item[vgolf_selected_dlc_item]->preview), 320, 240, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					if(vgolf_selected_dlc_item > 0)
					{
						hyperlink_page_add_element_text(dlc_page, hl_dlc_select_previous, "<", &vgolf_font[VGOLF_FONT_96], 320 + 2 - 152 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "<"), 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
					}
					if(vgolf_selected_dlc_item < vgolf_dlc_avatars->items - 1)
					{
						hyperlink_page_add_element_text(dlc_page, hl_dlc_select_next, ">", &vgolf_font[VGOLF_FONT_96], 320 + 2 + 152, 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
					}
					sprintf(avatar_menu_text[0], "Name: %s", vgolf_dlc_avatars->item[vgolf_selected_dlc_item]->name);
					sprintf(avatar_menu_text[1], "Author: %s", vgolf_dlc_avatars->item[vgolf_selected_dlc_item]->author);
					hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[0], &vgolf_font[VGOLF_FONT_48], 320, 240 - 32 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[1], &vgolf_font[VGOLF_FONT_48], 320, 240 + 32 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 1, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					if(vgolf_dlc_installed(VGOLF_CONTENT_AVATARS, vgolf_dlc_avatars->item[vgolf_selected_dlc_item]->hash))
					{
						hyperlink_page_add_element_text(dlc_page, NULL, "Already Installed", &vgolf_font[VGOLF_FONT_48], 320, 240 + 32 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					}
					else
					{
						hyperlink_page_add_element_text(dlc_page, hl_dlc_download, "Download", &vgolf_font[VGOLF_FONT_96], 320, 240 + 32 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW | HYPERLINK_ELEMENT_FLAG_CLICKABLE);
					}
					hyperlink_page_add_element_text(dlc_page, hl_dlc_cancel, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
				}
			}
			break;
		}
		case VGOLF_CONTENT_COURSES:
		{
			if(vgolf_dlc_courses)
			{

				for(i = 0; i < vgolf_dlc_courses->items; i++)
				{
					if(vgolf_dlc_courses->item[i]->preview)
					{
						vgolf_destroy_course(vgolf_dlc_courses->item[i]->preview);
						vgolf_dlc_courses->item[i]->preview = NULL;
					}
				}
				/* download preview */
				if(t3net_download_file(vgolf_dlc_courses->item[vgolf_selected_dlc_item]->preview_url, t3f_get_filename(t3f_temp_path, "dlc_course.vdci", buffer, 1024)))
				{
					/* use stdio for reading downloaded content and revert back to whatever was in use */
					old_interface = al_get_new_file_interface();
					al_set_standard_file_interface();
					vgolf_dlc_courses->item[vgolf_selected_dlc_item]->preview = vgolf_load_course_info(t3f_get_filename(t3f_temp_path, "dlc_course.vdci", buffer, 1024), 0, al_get_new_file_interface());
					al_set_new_file_interface(old_interface);
				}
				else
				{
					return false;
				}
				if(dlc_page)
				{
					hyperlink_page_destroy(dlc_page);
				}
				dlc_page = hyperlink_page_create(0, 0);
				if(dlc_page)
				{
					cp = vgolf_dlc_courses->item[vgolf_selected_dlc_item]->preview;
					hyperlink_page_add_element_animation(dlc_page, NULL, &cp->image_bank.iconpic, 320, 240, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					if(vgolf_selected_dlc_item > 0)
					{
						hyperlink_page_add_element_text(dlc_page, hl_dlc_select_previous, "<", &vgolf_font[VGOLF_FONT_96], 320 + 2 - 152 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "<"), 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
					}
					if(vgolf_selected_dlc_item < vgolf_dlc_courses->items - 1)
					{
						hyperlink_page_add_element_text(dlc_page, hl_dlc_select_next, ">", &vgolf_font[VGOLF_FONT_96], 320 + 2 + 152, 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
					}
					sprintf(avatar_menu_text[0], "Name: %s", vgolf_dlc_courses->item[vgolf_selected_dlc_item]->name);
					sprintf(avatar_menu_text[1], "Author: %s", vgolf_dlc_courses->item[vgolf_selected_dlc_item]->author);
					sprintf(avatar_menu_text[2], "Holes: %d", cp->num_holes);
					sprintf(avatar_menu_text[3], "Par: %d", cp->course_par);
					hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[0], &vgolf_font[VGOLF_FONT_48], 320, 240 - 96 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 4, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[1], &vgolf_font[VGOLF_FONT_48], 320, 240 + 96 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 1, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[2], &vgolf_font[VGOLF_FONT_48], 320, 240 - 96 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[3], &vgolf_font[VGOLF_FONT_48], 320, 240 - 96 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					if(vgolf_dlc_installed(VGOLF_CONTENT_COURSES, vgolf_dlc_courses->item[vgolf_selected_dlc_item]->hash))
					{
						hyperlink_page_add_element_text(dlc_page, NULL, "Already Installed", &vgolf_font[VGOLF_FONT_48], 320, 240 + 96 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
					}
					else
					{
						hyperlink_page_add_element_text(dlc_page, hl_dlc_download, "Download", &vgolf_font[VGOLF_FONT_96], 320, 240 + 96 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW | HYPERLINK_ELEMENT_FLAG_CLICKABLE);
					}
					hyperlink_page_add_element_text(dlc_page, hl_dlc_cancel, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
				}
			}
			break;
		}
		case VGOLF_CONTENT_COMMENTARIES:
		{
			T3F_TEXT_LINE_DATA text_lines;

			if(dlc_page)
			{
				hyperlink_page_destroy(dlc_page);
			}
			dlc_page = hyperlink_page_create(0, 0);
			if(dlc_page)
			{
				if(vgolf_selected_dlc_item > 0)
				{
					hyperlink_page_add_element_text(dlc_page, hl_dlc_select_previous, "<", &vgolf_font[VGOLF_FONT_96], 320 - 176 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "<"), 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
				}
				if(vgolf_selected_dlc_item < vgolf_dlc_commentaries->items - 1)
				{
					hyperlink_page_add_element_text(dlc_page, hl_dlc_select_next, ">", &vgolf_font[VGOLF_FONT_96], 320 + 176, 240 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
				}
				sprintf(avatar_menu_text[0], "%s", vgolf_dlc_commentaries->item[vgolf_selected_dlc_item]->name);
				sprintf(avatar_menu_text[1], "Author: %s", vgolf_dlc_commentaries->item[vgolf_selected_dlc_item]->author);
				t3f_create_text_line_data(&text_lines, vgolf_font[VGOLF_FONT_48], 320.0, 0.0, vgolf_dlc_commentaries->item[vgolf_selected_dlc_item]->description);
				for(i = 0; i < text_lines.lines; i++)
				{
					sprintf(avatar_menu_text[2 + i], "%s", text_lines.line[i].text);
				}
				hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[0], &vgolf_font[VGOLF_FONT_48], 320, 240 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * -4, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
				hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[1], &vgolf_font[VGOLF_FONT_48], 320, 240 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
				for(i = 0; i < text_lines.lines; i++)
				{
					hyperlink_page_add_element_text(dlc_page, NULL, avatar_menu_text[2 + i], &vgolf_font[VGOLF_FONT_48], 160, 240 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * (-2 + i), VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_SHADOW);
				}
				if(vgolf_dlc_installed(VGOLF_CONTENT_COMMENTARIES, vgolf_dlc_commentaries->item[vgolf_selected_dlc_item]->hash))
				{
					hyperlink_page_add_element_text(dlc_page, NULL, "Already Installed", &vgolf_font[VGOLF_FONT_48], 320, 240 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 5, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
				}
				else
				{
					hyperlink_page_add_element_text(dlc_page, hl_dlc_download, "Download", &vgolf_font[VGOLF_FONT_96], 320, 240 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 5, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, t3f_color_black, HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW | HYPERLINK_ELEMENT_FLAG_CLICKABLE);
				}
				hyperlink_page_add_element_text(dlc_page, hl_dlc_cancel, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
			}
			break;
		}
	}
	return true;
}

int hl_player_add_account(int p, void * pp)
{
	int i;

	if(strlen(entered_text) > 0 && strlen(entered_text) < 127 && vgolf_profiles.items < VGOLF_MAX_PROFILES)
	{
		strcpy(vgolf_profiles.item[vgolf_profiles.items].name, entered_text);
		vgolf_profiles.item[vgolf_profiles.items].avatar = vgolf_avatars.sum[0];
		for(i = 0; i < vgolf_courses.items; i++)
		{
			vgolf_profiles.item[vgolf_profiles.items].course_info[i].id = vgolf_courses.item[i].sum;
		}
		vgolf_profiles.items++;
		create_account_page(vgolf_player_choosing);
		vgolf_state = VGOLF_STATE_PLAYER_ACCOUNT;
		newmenu_enable_controls();
		choose_profile(vgolf_player_choosing, vgolf_profiles.items - 1);
		vgolf_state = VGOLF_STATE_PLAYER_SETUP;
		hl_player_avatar(vgolf_player_choosing, player_page_pointer[vgolf_player_choosing]);
	}
	return 1;
}

int hl_player_cancel_account(int p, void * pp)
{
	play_esc_sample();
	vgolf_state = VGOLF_STATE_PLAYER_ACCOUNT;
	newmenu_enable_controls();
	return 1;
}

int hl_remove_player(int p, void * pp)
{
	int i, pl = -1;
	play_esc_sample();
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(pp == player_page[i])
		{
			pl = i;
			break;
		}
	}
	if(pl >= 0)
	{
		joynet_disconnect_from_game(vgolf_client_game, 0, pl);
	}
	return 1;
}

void config_gfx_mode_right(void)
{
	vgolf_config[CONFIG_GFX_MODE]++;
	if(vgolf_config[CONFIG_GFX_MODE] == 1 && !vgolf_supports_2x_window())
	{
		vgolf_config[CONFIG_GFX_MODE] = 2;
	}
	if(vgolf_config[CONFIG_GFX_MODE] > 2)
	{
		vgolf_config[CONFIG_GFX_MODE] = 0;
	}
	set_gfx_mode();
}

void config_gfx_mode_left(void)
{
	vgolf_config[CONFIG_GFX_MODE]--;
	if(vgolf_config[CONFIG_GFX_MODE] == 1 && !vgolf_supports_2x_window())
	{
		vgolf_config[CONFIG_GFX_MODE] = 0;
	}
	if(vgolf_config[CONFIG_GFX_MODE] < 0)
	{
		vgolf_config[CONFIG_GFX_MODE] = 2;
	}
	set_gfx_mode();
}

void config_gfx_quality_right(void)
{
	vgolf_config[CONFIG_HD_TEXTURES]++;
	if(vgolf_config[CONFIG_HD_TEXTURES] > 1)
	{
		vgolf_config[CONFIG_HD_TEXTURES] = 0;
	}
}

void config_gfx_quality_left(void)
{
	vgolf_config[CONFIG_HD_TEXTURES]--;
	if(vgolf_config[CONFIG_HD_TEXTURES] < 0)
	{
		vgolf_config[CONFIG_HD_TEXTURES] = 1;
	}
}

void config_toggle_depth(void)
{
}

void config_toggle_upload(void)
{
	if(vgolf_config[CONFIG_UPLOAD])
	{
		vgolf_config[CONFIG_UPLOAD] = 0;
	}
	else
	{
		vgolf_config[CONFIG_UPLOAD] = 1;
	}
}

void config_toggle_dlc(void)
{
	if(vgolf_config[CONFIG_DLC])
	{
		vgolf_config[CONFIG_DLC] = 0;
	}
	else
	{
		vgolf_config[CONFIG_DLC] = 1;
	}
}

void config_toggle_low_bandwidth(void)
{
	if(vgolf_config[CONFIG_LOW_BANDWIDTH])
	{
		vgolf_config[CONFIG_LOW_BANDWIDTH] = 0;
	}
	else
	{
		vgolf_config[CONFIG_LOW_BANDWIDTH] = 1;
	}
}

void config_set_sfx_volume_left(void)
{
	if(vgolf_config[CONFIG_SFX_VOLUME] > 0)
	{
		vgolf_config[CONFIG_SFX_VOLUME] -= 5;
		if(vgolf_config[CONFIG_SFX_VOLUME] < 0)
		{
			vgolf_config[CONFIG_SFX_VOLUME] = 0;
		}
		t3f_set_sound_volume((float)vgolf_config[CONFIG_SFX_VOLUME] / 100.0);
	}
}

void config_set_sfx_volume_right(void)
{
	if(vgolf_config[CONFIG_SFX_VOLUME] < 100)
	{
		vgolf_config[CONFIG_SFX_VOLUME] += 5;
		if(vgolf_config[CONFIG_SFX_VOLUME] > 100)
		{
			vgolf_config[CONFIG_SFX_VOLUME] = 100;
		}
		t3f_set_sound_volume((float)vgolf_config[CONFIG_SFX_VOLUME] / 100.0);
	}
}

void config_set_music_volume_left(void)
{
	if(vgolf_config[CONFIG_MUSIC_VOLUME] > 0)
	{
		vgolf_config[CONFIG_MUSIC_VOLUME] -= 5;
		if(vgolf_config[CONFIG_MUSIC_VOLUME] < 0)
		{
			vgolf_config[CONFIG_MUSIC_VOLUME] = 0;
		}
		t3f_set_music_volume((float)vgolf_config[CONFIG_MUSIC_VOLUME] / 100.0);
	}
}

void config_set_music_volume_right(void)
{
	if(vgolf_config[CONFIG_MUSIC_VOLUME] < 100)
	{
		vgolf_config[CONFIG_MUSIC_VOLUME] += 5;
		if(vgolf_config[CONFIG_MUSIC_VOLUME] > 100)
		{
			vgolf_config[CONFIG_MUSIC_VOLUME] = 100;
		}
		t3f_set_music_volume((float)vgolf_config[CONFIG_MUSIC_VOLUME] / 100.0);
	}
}

void config_set_commentary_volume_left(void)
{
	if(vgolf_config[CONFIG_COMMENTARY_VOLUME] > 0)
	{
		vgolf_config[CONFIG_COMMENTARY_VOLUME] -= 5;
		if(vgolf_config[CONFIG_COMMENTARY_VOLUME] < 0)
		{
			vgolf_config[CONFIG_COMMENTARY_VOLUME] = 0;
		}
	}
}

void config_set_commentary_volume_right(void)
{
	if(vgolf_config[CONFIG_COMMENTARY_VOLUME] < 100)
	{
		vgolf_config[CONFIG_COMMENTARY_VOLUME] += 5;
		if(vgolf_config[CONFIG_COMMENTARY_VOLUME] > 100)
		{
			vgolf_config[CONFIG_COMMENTARY_VOLUME] = 100;
		}
	}
}

void config_set_commentary_announcer_left(void)
{
	vgolf_config[CONFIG_COMMENTARY]--;
	if(vgolf_config[CONFIG_COMMENTARY] < 0)
	{
		vgolf_config[CONFIG_COMMENTARY] = vgolf_commentaries.count - 1;
	}
}

void config_set_commentary_announcer_right(void)
{
	vgolf_config[CONFIG_COMMENTARY]++;
	if(vgolf_config[CONFIG_COMMENTARY] >= vgolf_commentaries.count)
	{
		vgolf_config[CONFIG_COMMENTARY] = 0;
	}
}

void config_dead_spot_left(void)
{
	vgolf_config[CONFIG_DEAD_SPOT] -= 5;
	if(vgolf_config[CONFIG_DEAD_SPOT] < 5)
	{
		vgolf_config[CONFIG_DEAD_SPOT] = 5;
	}
}

void config_dead_spot_right(void)
{
	vgolf_config[CONFIG_DEAD_SPOT] += 5;
	if(vgolf_config[CONFIG_DEAD_SPOT] > 20)
	{
		vgolf_config[CONFIG_DEAD_SPOT] = 20;
	}
}

void exit_game_yes(void)
{
	joynet_end_game(vgolf_client_game);
}

void exit_game_no(void)
{
	hl_sub_menu_back(0, NULL);
}

int vgolf_gimme_take(int i, void * p)
{
	int j;
	int hole_vert = -1;

	play_select_sample();
	for(j = 0; j < game_course->current_hole.num_verts; j++)
	{
		if(game_course->current_hole.vert[j].flag == VERT_FLAG_HOLE)
		{
			hole_vert = j;
			break;
		}
	}
	if(hole_vert >= 0)
	{
		main_game_vars.game_state = GAMESTATE_READY_TOPLAY;
		do_ball_hole_sink(&game_player[main_game_vars.cur_player].ball, &game_player[main_game_vars.cur_player], &game_course->current_hole.vert[hole_vert], &game_course->current_hole);
		vgolf_local_club_blocked = true;
		vgolf_game_block_click = true;
	}
	return 1;
}

int vgolf_gimme_keep_playing(int i, void * p)
{
	play_select_sample();
	if(game_player[main_game_vars.cur_player].gimme_offered == 2)
	{
		main_game_vars.game_state = GAMESTATE_BALL_LOST;
		fake_page_hidden = false;
	}
	else
	{
		main_game_vars.game_state = GAMESTATE_READY_TOPLAY;
		locally_block_click = true;
		vgolf_local_club_blocked = true;
		vgolf_game_block_click = true;
		if(vgolf_player_count > 1)
		{
			int p_no = vgolf_get_next_player();

			if(p_no != -1)
			{
				vgolf_select_player(p_no);
				build_turn_text(p_no);
				add_vfont_queue_message(game_vmessage, GAME_VMESSAGE_QUEUE_SIZE, vgolf_font[VGOLF_FONT_96], &main_game_vars.cur_vmessage_slot, temp_text, t3f_virtual_display_width / 2 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], temp_text) / 2, t3f_virtual_display_height / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, 0, VMSG_TEXT_ENTER_FROMRIGHT, -10, 4, 0, 0, -1);
			}
		}
	}
	return 1;
}

void in_game_resume(void)
{
	joynet_resume_game(vgolf_client_game);
	locally_block_click = true;
}

void delete_profile_yes(void)
{
	int i;

	for(i = selected_profile; i < vgolf_profiles.items - 1; i++)
	{
		memcpy(&vgolf_profiles.item[i], &vgolf_profiles.item[i + 1], sizeof(PROFILE));
	}
	vgolf_profiles.items--;
	if(selected_profile >= vgolf_profiles.items)
	{
		selected_profile = vgolf_profiles.items - 1;
	}
	create_profile_page(selected_profile);
	vgolf_state = VGOLF_STATE_PROFILE;
}

void delete_profile_no(void)
{
	vgolf_state = VGOLF_STATE_PROFILE;
}

void fade_quit(void)
{
	t3f_exit();
}

void menu_quit(void)
{
	fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 2, fade_quit);
}

void menu_purchase(void)
{
	#ifdef T3F_ANDROID_GP
		t3f_open_url("market://details?id=com.t3i.vgolfdeluxe");
	#else
		t3f_open_url("http://www.t3-i.com/pages/link.php?id=27");
	#endif
}

void menu_privacy(void)
{
	vgolf_state = VGOLF_STATE_PRIVACY;
}

static void vgolf_tally_votes(void)
{
	int pick[VGOLF_MAX_PLAYERS];
	int picks = 0;
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			if(game_player[i].scourse != 0)
			{
				pick[picks] = game_player[i].scourse;
				picks++;
			}
		}
	}
	vgolf_selected_course = pick[rand() % picks]; // pick course from votes
}

static void play_fade_done(void)
{
	vgolf_seed = time(0);
	vgolf_low_bandwidth_mode = vgolf_config[CONFIG_LOW_BANDWIDTH];
	vgolf_tally_votes();
	fx_fade_finished = 0;
	vgolf_state = VGOLF_STATE_LOAD;
	joynet_start_game(vgolf_client_game);
	newmenu_enable_controls();
}

void play_proc(void)
{
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			break;
		}
	}
	if(i == VGOLF_MAX_PLAYERS)
	{
		ncdmsg_add(&vgolf_messages, "No Players!", al_map_rgb(255, 255, 255), 0.0, 300);
		return;
	}
/*	for(i = 0; i < 5; i++)
	{
		letter[i].vz = -(float)(rand() % 27) / 3.0;
		letter[i].state = 1;
	}
	letter[2].vz = -5.7;
	letter[2].vy = -4.0;
	letter[2].active = 1; */
	fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 6, play_fade_done);
	newmenu_disable_controls();
}

void player_setup_proc(void)
{
	int i;

	/* enable players that are connected according to JoyNet */
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_client_game->player[i]->playing)
		{
			vgolf_player_list[i] = 1;
		}
		else
		{
			vgolf_player_list[i] = 0;
		}
	}

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			if(vgolf_client_game->player[i]->local)
			{
				player_page_pointer[i] = player_page[i];
			}
			else
			{
				player_page_pointer[i] = player_network_page[i];
			}
//			player_page[i]->element[1].data = vgolf_avatars.avatar[game_player[i].avatar]->image[0];
		}
		else
		{
			player_page_pointer[i] = player_start_page[i];
		}
	}
	vgolf_state = VGOLF_STATE_PLAYER_SETUP;
}

void credits_proc_helper(void)
{
	vgolf_state = VGOLF_STATE_CREDITS;
	vgolf_credits.y = 480;
	vgolf_credits.vy = 1;
	vgolf_credits.fade_level = 0;
	vgolf_credits.fade_dir = 0;
	vgolf_credits.current_image = 0;
	vgolf_credits.done = 0;
//	clear_to_color(screen, getpixel(letter[0].bp, 0, 0));
	newmenu_enable_controls();
}

void credits_proc(void)
{
	newmenu_disable_controls();
	fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), menu_bg_color, 2, credits_proc_helper);
}

void profile_view_proc(void)
{
	vgolf_state = VGOLF_STATE_PROFILE;
	selected_profile = 0;
	profile_course = -1;
	profile_show_details = 0;
	create_profile_page(selected_profile);
}

void leaderboards_view_proc(void)
{
	vgolf_state = VGOLF_STATE_LEADERBOARDS;
	strcpy(leaderboard_menu_text, "< Back");
	profile_course = 0;
	leaderboard_mode = 0;
	create_leaderboard_page();
}

void menu_proc_upload_retry(void)
{
	vgolf_handle_score_upload();
}

void menu_proc_upload_cancel(void)
{
	start_victory_screen_segin(&main_game_vars, game_course, game_player, vgolf_player_count);
}

#ifdef T3F_ANDROID
	static void vgolf_host_game_name_callback(void * data)
	{
		if(strlen(entered_text) > 0)
		{
			strcpy(vgolf_host_name, entered_text);
		}
	}
#endif

void vgolf_menu_proc_host_game_name(void)
{
	#ifndef T3F_ANDROID
		strcpy(old_entered_text, entered_text);
		entered_text_pos = strlen(entered_text);
		t3f_clear_keys();
		entering_text = 4;
	#else
		t3f_open_edit_box("Enter Host Name", entered_text, 64, "CapWords", vgolf_host_game_name_callback, NULL);
	#endif
}

void vgolf_menu_proc_host_game(void)
{
	if(strlen(vgolf_host_name) > 0)
	{
		strcpy(entered_text, vgolf_host_name);
	}
	else
	{
		strcpy(entered_text, "Unnamed Host");
	}
	current_newmenu = nmenu[VGOLF_MENU_HOST_NAME];
}

void vgolf_menu_proc_close_host(void)
{
	al_stop_timer(t3f_timer);
	joynet_disconnect_from_game_server(vgolf_client_game, vgolf_client);
	joynet_destroy_client(vgolf_client);
	vgolf_client = NULL;
	vgolf_close_server();
	vgolf_hosting = false;
	current_newmenu = nmenu[VGOLF_MENU_MAIN];
	al_start_timer(t3f_timer);
	ncdmsg_clear(&vgolf_messages);
	vgolf_close_chat_log();
}

void vgolf_open_chat_log(void)
{
	char tempfn[1024] = {0};
	char buffer[1024];

	sprintf(tempfn, "%s%s.txt", t3f_get_filename(t3f_data_path, "chatlogs/", buffer, 1024), vgolf_get_date_string());
	if(vgolf_chat_log_file)
	{
		al_fclose(vgolf_chat_log_file);
	}
	vgolf_chat_log_file = al_fopen(tempfn, "wb");
}

void vgolf_close_chat_log(void)
{
	if(vgolf_chat_log_file)
	{
		al_fclose(vgolf_chat_log_file);
		vgolf_chat_log_file = NULL;
	}
}

/* need to fix this for when user enters gibberish with no spaces */
void vgolf_log_chat(const char * user, const char * text)
{
	char line[128] = {0};
	int i, l, lp, p = 0, ls = 0, s = 0;

	if(vgolf_chat_log_file)
	{
		l = strlen(user);

		/* copy user name into first line */
		for(i = 0; i < l; i++)
		{
			line[i] = user[i];
		}
		lp = l; // move line position to end of username
		while(1)
		{

			/* remember last space */
			if(text[p] == ' ')
			{
				s = p;
				ls = lp;
			}

			line[lp] = text[p];
			lp++;
			line[lp] = '\0';

			/* once we fill up the line, go back to last space and make newline */
			if(lp >= 80)
			{
				line[ls] = '\n';
				line[ls + 1] = '\0';
				al_fputs(vgolf_chat_log_file, line);
				lp = l;
				for(i = 0; i < l; i++)
				{
					line[i] = ' ';
				}
				line[lp] = '\0';
				p = s;
			}
			p++;
			if(p >= strlen(text))
			{
				line[lp] = '\n';
				line[lp + 1] = '\0';
				al_fputs(vgolf_chat_log_file, line);
				break;
			}
		}
	}
}

void vgolf_select_host(void)
{
	al_stop_timer(t3f_timer);
	vgolf_client = joynet_create_client();
	if(vgolf_client)
	{
		if(joynet_connect_to_game_server(vgolf_client_game, vgolf_client, vgolf_server_list->entry[current_newmenu->current_item - 1]->address, vgolf_server_list->entry[current_newmenu->current_item - 1]->port))
		{
			joynet_set_client_screen_name(vgolf_client, vgolf_nick);
			joynet_watch_game(vgolf_client_game);
			current_newmenu = nmenu[VGOLF_MENU_MAIN_JOIN];
			joynet_set_client_chat_callback(vgolf_client, vgolf_chat_callback);
			joynet_set_client_global_callback(vgolf_client, vgolf_client_callback);
			vgolf_open_chat_log();
			#ifndef T3F_ANDROID
				ncdmsg_add(&vgolf_messages, "Press 't' to chat.", al_map_rgb(255, 255, 255), 0.0, 0);
			#endif
		}
		else
		{
			joynet_destroy_client(vgolf_client);
			vgolf_client = NULL;
			ncdmsg_add(&vgolf_messages, "Could not connect to host.", al_map_rgb(255, 255, 255), 0.0, 300);
		}
	}
	al_start_timer(t3f_timer);
}

static bool vgolf_create_server_list_menu(void)
{
	int i;
	char text[1024] = {0};
	bool fail = false;

	if(nmenu[VGOLF_MENU_HOST_LIST])
	{
		destroy_newmenu(nmenu[VGOLF_MENU_HOST_LIST]);
		nmenu[VGOLF_MENU_HOST_LIST] = NULL;
	}
	if(!vgolf_server_list)
	{
		vgolf_server_list = t3net_get_server_list("www.t3-i.com/t3net2/master/query.php", VGOLF_VERSION_NAME, VGOLF_SERVER_VERSION);
		if(!vgolf_server_list)
		{
			printf("failed to get server list\n");
			fail = true;
		}
	}
	else
	{
		if(!t3net_update_server_list_2(vgolf_server_list))
		{
			fail = true;
		}
	}
	if(!fail)
	{
		if(vgolf_server_list->entries <= 0)
		{
			ncdmsg_add(&vgolf_messages, "No public servers available.", al_map_rgb(255, 255, 255), 0.0, 300);
			fail = true;
		}
	}
	else
	{
		ncdmsg_add(&vgolf_messages, "Unable to retrieve server list.", al_map_rgb(255, 255, 255), 0.0, 300);
	}
	if(vgolf_server_list && !fail)
	{
		nmenu[VGOLF_MENU_HOST_LIST] = create_newmenu("Select Host", NULL, 0, 320 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_64]) * vgolf_server_list->entries) / 2 - 15 - 64);
		if(nmenu[VGOLF_MENU_HOST_LIST])
		{
			add_newmenu_item(nmenu[VGOLF_MENU_HOST_LIST], "Select Host", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
			for(i = 0; i < vgolf_server_list->entries; i++)
			{
				sprintf(text, "%s (%s)", vgolf_server_list->entry[i]->name, vgolf_server_list->entry[i]->capacity);
				add_newmenu_item(nmenu[VGOLF_MENU_HOST_LIST], text, NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_64]) * (i + 1), &vgolf_font[VGOLF_FONT_64], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_select_host, NULL, NULL);
			}
		}
		else
		{
			printf("failed to create host list menu\n");
		}
	}
	return !fail;
}

void vgolf_menu_proc_join_game(void)
{
	al_stop_timer(t3f_timer);
	if(vgolf_create_server_list_menu())
	{
		if(nmenu[VGOLF_MENU_HOST_LIST])
		{
			current_newmenu = nmenu[VGOLF_MENU_HOST_LIST];
		}
	}
	al_start_timer(t3f_timer);
}

void vgolf_menu_proc_disconnect(void)
{
	al_stop_timer(t3f_timer);
	joynet_disconnect_from_game_server(vgolf_client_game, vgolf_client);
	joynet_destroy_client(vgolf_client);
	vgolf_client = NULL;
	current_newmenu = nmenu[VGOLF_MENU_MAIN];
	al_start_timer(t3f_timer);
	ncdmsg_clear(&vgolf_messages);
	vgolf_close_chat_log();
	vgolf_reset_content_lists();
}

void vgolf_menu_proc_begin_hosting(void)
{
	entering_text = 0;
	if(strlen(entered_text) > 0 && strlen(entered_text) < 127)
	{
		int c;

		strcpy(vgolf_host_name, entered_text);
		al_stop_timer(t3f_timer);
		if(vgolf_open_server(vgolf_host_name, 5566, false))
		{
			vgolf_hosting = true;
			vgolf_client = joynet_create_client();
			if(vgolf_client)
			{
				al_rest(0.25); // give server thread time to start up
				if(joynet_connect_to_game_server(vgolf_client_game, vgolf_client, "127.0.0.1", 5566))
				{
					for(c = 0; c < vgolf_client_game->players; c++)
					{
						vgolf_player_list[c] = 0;
					}
					joynet_set_client_screen_name(vgolf_client, vgolf_nick);
					joynet_watch_game(vgolf_client_game);
					current_newmenu = nmenu[VGOLF_MENU_MAIN_HOST];
					joynet_set_client_chat_callback(vgolf_client, vgolf_chat_callback);
					vgolf_open_chat_log();
				}
				else
				{
					joynet_destroy_client(vgolf_client);
					vgolf_client = NULL;
					vgolf_close_server();
					vgolf_hosting = false;
				}
			}
			else
			{
				vgolf_close_server();
				vgolf_hosting = false;
			}
		}
		al_start_timer(t3f_timer);
		entering_text = 0;
	}
}

void vgolf_menu_proc_buffer_left(void)
{
	if(vgolf_config[CONFIG_INPUT_BUFFER] > 1)
	{
		vgolf_config[CONFIG_INPUT_BUFFER]--;
	}
}

void vgolf_menu_proc_buffer_right(void)
{
	if(vgolf_config[CONFIG_INPUT_BUFFER] < 16)
	{
		vgolf_config[CONFIG_INPUT_BUFFER]++;
	}
}

void vgolf_menu_proc_set_name(void)
{
	#ifndef T3F_ANDROID
		t3f_clear_keys();
		entering_text = 5;
		if(!strcasecmp(vgolf_nick, "Unnamed"))
		{
			strcpy(entered_text, "");
		}
		else
		{
			strcpy(entered_text, vgolf_nick);
		}
		entered_text_pos = strlen(entered_text);
		current_newmenu = nmenu[VGOLF_MENU_NICK];
	#else
		t3f_open_edit_box("Enter Network ID", vgolf_nick, 64, "CapWords", NULL, NULL);
	#endif
	play_select_sample();
}

void vgolf_menu_proc_accept_name(void)
{
	if(strlen(entered_text) > 0 && strlen(entered_text) < 127)
	{
		strcpy(vgolf_nick, entered_text);
	}
	current_newmenu = nmenu[VGOLF_MENU_OPTIONS_ONLINE];
	play_select_sample();
}

void vgolf_menu_proc_cancel_name(void)
{
	current_newmenu = nmenu[VGOLF_MENU_OPTIONS_ONLINE];
	play_esc_sample();
}

void vgolf_menu_proc_cancel_hosting(void)
{
	entering_text = 0;
	hl_sub_menu_back(0, NULL);
}

static void vgolf_dlc_preview_destroy_callback(void * data)
{
//	al_destroy_bitmap((ALLEGRO_BITMAP *)data);
}

void vgolf_menu_proc_dlc_avatars(void)
{
	al_stop_timer(t3f_timer);
	if(vgolf_dlc_avatars)
	{
		t3net_destroy_dlc_list(vgolf_dlc_avatars, vgolf_dlc_preview_destroy_callback);
	}

	vgolf_dlc_avatars = t3net_get_dlc_list("http://www.t3-i.com/t3net2/dlc/query.php", "vgolf_deluxe", 0);
	if(vgolf_dlc_avatars && vgolf_dlc_avatars->items > 0)
	{
		vgolf_selected_dlc_type = VGOLF_CONTENT_AVATARS;
		vgolf_selected_dlc_item = 0;
		if(vgolf_create_dlc_page())
		{
			vgolf_state = VGOLF_STATE_DLC;
		}
	}
	else
	{
		ncdmsg_add(&vgolf_messages, "Failed to retrieve DLC list!", al_map_rgb(255, 255, 255), 0.0, 300);
	}
	al_start_timer(t3f_timer);
}

void vgolf_menu_proc_dlc_courses(void)
{
	al_stop_timer(t3f_timer);
	if(vgolf_dlc_courses)
	{
		t3net_destroy_dlc_list(vgolf_dlc_courses, vgolf_dlc_preview_destroy_callback);
	}
	vgolf_dlc_courses = t3net_get_dlc_list("http://www.t3-i.com/t3net2/dlc/query.php", "vgolf_deluxe", 1);
	if(vgolf_dlc_courses && vgolf_dlc_courses->items > 0)
	{
		vgolf_selected_dlc_type = VGOLF_CONTENT_COURSES;
		vgolf_selected_dlc_item = 0;
		if(vgolf_create_dlc_page())
		{
			vgolf_state = VGOLF_STATE_DLC;
		}
	}
	else
	{
		ncdmsg_add(&vgolf_messages, "Failed to retrieve DLC list!", al_map_rgb(255, 255, 255), 0.0, 300);
	}
	al_start_timer(t3f_timer);
}

void vgolf_menu_proc_dlc_commentaries(void)
{
	al_stop_timer(t3f_timer);
	if(vgolf_dlc_commentaries)
	{
		t3net_destroy_dlc_list(vgolf_dlc_commentaries, vgolf_dlc_preview_destroy_callback);
	}
	vgolf_dlc_commentaries = t3net_get_dlc_list("http://www.t3-i.com/t3net2/dlc/query.php", "vgolf_deluxe", 2);
	if(vgolf_dlc_commentaries && vgolf_dlc_commentaries > 0)
	{
		vgolf_selected_dlc_type = VGOLF_CONTENT_COMMENTARIES;
		vgolf_selected_dlc_item = 0;
		if(vgolf_create_dlc_page())
		{
			vgolf_state = VGOLF_STATE_DLC;
		}
	}
	else
	{
		ncdmsg_add(&vgolf_messages, "Failed to retrieve DLC list!", al_map_rgb(255, 255, 255), 0.0, 300);
	}
	al_start_timer(t3f_timer);
}

int vgolf_menu_proc_toggle_chat(int i, void * p)
{
	/* toggle chat */
	if(vgolf_chat_state == 0 || vgolf_chat_state == 3)
	{
		vgolf_chat_state = 1;
	}
	else if(vgolf_chat_state == 2 || vgolf_chat_state == 1)
	{
		vgolf_chat_state = 3;
	}
	locally_block_click = true;
	return 1;
}

#ifdef T3F_ANDROID
	static void vgolf_edit_box_chat_callback(void * data)
	{
		if(strlen(entered_text) > 0)
		{
			joynet_send_client_chat(vgolf_client, entered_text, 0);
		}
	}

	int vgolf_menu_proc_chat_message(int i, void * p)
	{
		/* toggle chat */
		if(vgolf_chat_state == 2 || vgolf_chat_state == 1)
		{
			vgolf_chat_state = 3;
		}
		strcpy(entered_text, "");
		t3f_open_edit_box("Enter Message", entered_text, 256, "CapSentences", vgolf_edit_box_chat_callback, NULL);
		locally_block_click = true;
		return 1;
	}
#endif

int vgolf_menu_proc_hide_fake_page(int i, void * p)
{
	fake_page_hidden = true;
	return 1;
}

void vgolf_set_up_menus(void)
{
	int px[VGOLF_MAX_PLAYERS] = {0, 213, 426, 0, 213, 426, 0, 426};
	int py[VGOLF_MAX_PLAYERS] = {0, 0, 0, 160, 160, 160, 320, 320};
	int i, lh, pos;

	/* create hyperlink pages */
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		player_page[i] = hyperlink_page_create(px[i], py[i]);
		hyperlink_page_add_element_text(player_page[i], hl_player_name, player_page_text_buffer[i], &vgolf_font[VGOLF_FONT_32], 213 / 2, 160 / 2 - 40 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]), VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_animation(player_page[i], hl_player_avatar, NULL, 213 / 2, 160 / 2 + 2, HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_text(player_page[i], hl_remove_player, "Remove Player", &vgolf_font[VGOLF_FONT_32], 213 / 2, 160 / 2 + 40, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

		player_network_page[i] = hyperlink_page_create(px[i], py[i]);
		hyperlink_page_add_element_text(player_network_page[i], NULL, player_page_text_buffer[i], &vgolf_font[VGOLF_FONT_32], 213 / 2, 160 / 2 - 40 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]), VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
		hyperlink_page_add_element_animation(player_network_page[i], NULL, NULL, 213 / 2, 160 / 2 + 2, HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

		player_start_page[i] = hyperlink_page_create(px[i], py[i]);
//		hyperlink_page_add_element_text(player_start_page[i], hl_add_player, "Add Player", &vgolf_font[VGOLF_FONT_32], 213 / 2, 160 / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) / 2, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
//		hyperlink_page_add_element_text(player_start_page[i], NULL, "No Player", &vgolf_font[VGOLF_FONT_32], 213 / 2, 160 / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) / 2, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	}
	player_add_page = hyperlink_page_create(px[1], py[7]);
	hyperlink_page_add_element_text(player_add_page, hl_add_player, "Add Player", &vgolf_font[VGOLF_FONT_48], 213 / 2, 160 / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_32]) / 2, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

	player_global_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(player_global_page, hl_player_next, "Next >", &vgolf_font[VGOLF_FONT_96], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "Next >") - 4, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(player_global_page, hl_player_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);

	player_global_slave_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(player_global_slave_page, hl_player_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);

	course_global_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(course_global_page, hl_summary_play, "Play >", &vgolf_font[VGOLF_FONT_96], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "Play >") - 4, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(course_global_page, hl_summary_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);

	course_global_slave_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(course_global_slave_page, hl_summary_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);

	sub_menu_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(sub_menu_page, hl_sub_menu_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);

	create_scoreboard_page();

	profile_global_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(profile_global_page, hl_profile_back, "< Back", &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(profile_global_page, hl_profile_delete, "Delete Profile", &vgolf_font[VGOLF_FONT_96], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "Delete Profile") - 4, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	create_profile_page(0);

	leaderboard_global_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(leaderboard_global_page, hl_profile_back, leaderboard_menu_text, &vgolf_font[VGOLF_FONT_96], 2, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	create_profile_page(0);

	create_account_page(0);

	game_complete_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(game_complete_page, hl_game_complete_finished, "Finished >", &vgolf_font[VGOLF_FONT_96], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "Finished >") - 4, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);

	fake_game_complete_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(fake_game_complete_page, vgolf_menu_proc_hide_fake_page, "Finished >", &vgolf_font[VGOLF_FONT_96], 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "Finished >") - 4, 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);

	lh = t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]);
	player_new_account_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(player_new_account_page, NULL, "Enter Name", &vgolf_font[VGOLF_FONT_96], 320 + 2, 109, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(player_new_account_page, hl_player_account_name_proc, entered_text, &vgolf_font[VGOLF_FONT_96], 320 + 2, 109 + lh * 1 + lh / 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(player_new_account_page, hl_player_add_account, "Okay", &vgolf_font[VGOLF_FONT_96], 320 + 2, 109 + lh * 2 + lh, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(player_new_account_page, hl_player_cancel_account, "Cancel", &vgolf_font[VGOLF_FONT_96], 320 + 2, 109 + lh * 3 + lh, VGOLF_COLOR_MENU_OPTION_DARK, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

	ball_drop_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(ball_drop_page, NULL, "Ball Lost", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(ball_drop_page, vgolf_ball_lost_drop, "Drop", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(ball_drop_page, vgolf_ball_lost_rehit, "Rehit", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

	fake_ball_drop_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(fake_ball_drop_page, NULL, "Ball Lost", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(fake_ball_drop_page, vgolf_menu_proc_hide_fake_page, "Drop", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(fake_ball_drop_page, vgolf_menu_proc_hide_fake_page, "Rehit", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

	take_gimme_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(take_gimme_page, NULL, "Having Trouble?", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(take_gimme_page, vgolf_gimme_take, "Take Gimme", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(take_gimme_page, vgolf_gimme_keep_playing, "Keep Playing", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

	fake_take_gimme_page = hyperlink_page_create(0, 0);
	hyperlink_page_add_element_text(fake_take_gimme_page, NULL, "Having Trouble?", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2, VGOLF_COLOR_MENU_OPTION, VGOLF_COLOR_MENU_OPTION, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(fake_take_gimme_page, vgolf_menu_proc_hide_fake_page, "Take Gimme", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(fake_take_gimme_page, vgolf_menu_proc_hide_fake_page, "Keep Playing", &vgolf_font[VGOLF_FONT_96], 320 + 2, 240 - (t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2) / 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_CENTER | HYPERLINK_ELEMENT_FLAG_SHADOW);

	create_course_page();

	chat_page = hyperlink_page_create(0, 1);
//	hyperlink_page_add_element_text(chat_page, vgolf_menu_proc_toggle_chat, vgolf_chat_page_text, &vgolf_font[VGOLF_FONT_32], vgolf_chat_width + t3f_get_text_width(vgolf_font[VGOLF_FONT_32], " "), 0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	hyperlink_page_add_element_text(chat_page, vgolf_menu_proc_toggle_chat, vgolf_chat_page_text, &vgolf_font[VGOLF_FONT_CHAT_OVERLAY], 640.0 - t3f_get_text_width(vgolf_font[VGOLF_FONT_CHAT_OVERLAY], "<"), 0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	#ifdef T3F_ANDROID
		hyperlink_page_add_element_text(chat_page, vgolf_menu_proc_chat_message, "_", &vgolf_font[VGOLF_FONT_CHAT_OVERLAY], 640.0 - 	t3f_get_text_width(vgolf_font[VGOLF_FONT_CHAT_OVERLAY], "<") - t3f_get_text_width(vgolf_font[VGOLF_FONT_CHAT_OVERLAY], "_ "), 0, VGOLF_COLOR_MENU_MAIN_DARK, VGOLF_COLOR_MENU_MAIN, al_map_rgb(0, 0, 0), HYPERLINK_ELEMENT_FLAG_CLICKABLE | HYPERLINK_ELEMENT_FLAG_SHADOW);
	#endif

	/* set up menus */
	nmenu[VGOLF_MENU_PROFILE_DELETE] = create_newmenu("Delete Profile", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_PROFILE_DELETE], "Delete Profile?", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_PROFILE_DELETE], "Yes", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, delete_profile_yes, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_PROFILE_DELETE], "No", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, delete_profile_no, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_PROFILE_DELETE], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_NAME_ENTRY] = create_newmenu("Enter Name", profile_name_update, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_NAME_ENTRY], "Enter Name", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_HEADER, NULL, VGOLF_COLOR_MENU_HEADER, NULL, NULL, NULL, NULL);
	add_newmenu_child_variable(nmenu[VGOLF_MENU_NAME_ENTRY], nmenu[VGOLF_MENU_NAME_ENTRY]->vtext[0], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], al_map_rgb(192, 192, 192), NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	newmenu_centre(nmenu[VGOLF_MENU_NAME_ENTRY], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_OPTIONS_AUDIO] = create_newmenu("Audio Options", audio_options_update, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "Audio Options", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "Music Volume", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 0, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_AUDIO], nmenu[VGOLF_MENU_OPTIONS_AUDIO]->vtext[0], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 1, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "100%") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 1, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_music_volume_left, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "100%") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 1, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_music_volume_right, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "Sound Volume", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 2, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_AUDIO], nmenu[VGOLF_MENU_OPTIONS_AUDIO]->vtext[1], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "100%") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_sfx_volume_left, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "100%") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 3, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_sfx_volume_right, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "Commentary Volume", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 4, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_AUDIO], nmenu[VGOLF_MENU_OPTIONS_AUDIO]->vtext[2], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 5, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "100%") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 5, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_commentary_volume_left, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "100%") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 5, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_commentary_volume_right, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "Announcer", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 6, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_AUDIO], nmenu[VGOLF_MENU_OPTIONS_AUDIO]->vtext[3], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 7, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Default Voice") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 7, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_commentary_announcer_left, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_AUDIO], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Default Voice") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 7, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_set_commentary_announcer_right, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_OPTIONS_AUDIO], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	pos = 0;
	nmenu[VGOLF_MENU_OPTIONS_VIDEO] = create_newmenu("Video Options", video_options_update, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_VIDEO], "Video Options", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	#ifndef T3F_ANDROID
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_VIDEO], "Screen", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
		pos++;
		add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_VIDEO], nmenu[VGOLF_MENU_OPTIONS_VIDEO]->vtext[0], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_VIDEO], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Fullscreen") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_gfx_mode_left, NULL, NULL);
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_VIDEO], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Fullscreen") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_gfx_mode_right, NULL, NULL);
		pos++;
	#endif
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_VIDEO], "Texture Quality", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	pos++;
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_VIDEO], nmenu[VGOLF_MENU_OPTIONS_VIDEO]->vtext[1], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_VIDEO], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Standard") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_gfx_quality_left, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_VIDEO], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Standard") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_gfx_quality_right, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_OPTIONS_VIDEO], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_NICK] = create_newmenu("Screen Name", profile_name_update, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_NICK], "Enter Screen Name", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_child_variable(nmenu[VGOLF_MENU_NICK], nmenu[VGOLF_MENU_NICK]->vtext[0], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_NICK], "Okay", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_accept_name, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_NICK], "Cancel", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_cancel_name, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_NICK], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	pos = 0;
	nmenu[VGOLF_MENU_OPTIONS_ONLINE] = create_newmenu("Online Options", online_options_update, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "Online Options", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "Upload Scores", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	pos++;
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_ONLINE], nmenu[VGOLF_MENU_OPTIONS_ONLINE]->vtext[0], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Yes") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_toggle_upload, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Yes") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_toggle_upload, NULL, NULL);
	pos++;
	if(!vgolf_demo)
	{
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "Download DLC", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
		pos++;
		add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_ONLINE], nmenu[VGOLF_MENU_OPTIONS_ONLINE]->vtext[1], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Manual") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_toggle_dlc, NULL, NULL);
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Manual") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_toggle_dlc, NULL, NULL);
		pos++;
	}
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "Low Bandwidth Mode", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	pos++;
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_ONLINE], nmenu[VGOLF_MENU_OPTIONS_ONLINE]->vtext[2], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Off") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_toggle_low_bandwidth, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "Off") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_toggle_low_bandwidth, NULL, NULL);
	pos++;
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "Buffer Input", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	pos++;
	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_ONLINE], nmenu[VGOLF_MENU_OPTIONS_ONLINE]->vtext[3], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "16 Frames") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_buffer_left, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "16 Frames") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_buffer_right, NULL, NULL);
	pos++;
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "Screen Name", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	pos++;
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_ONLINE], "", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_set_name, NULL, NULL);
	nmenu[VGOLF_MENU_OPTIONS_ONLINE]->item[nmenu[VGOLF_MENU_OPTIONS_ONLINE]->items - 1].vtext = vgolf_nick;
//	add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_ONLINE], nmenu[VGOLF_MENU_OPTIONS_ONLINE]->vtext[2], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 6, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	newmenu_centre(nmenu[VGOLF_MENU_OPTIONS_ONLINE], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	pos = 0;
	nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY] = create_newmenu("Gameplay Options", gameplay_options_update, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY], "Gameplay Options", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	#ifdef T3F_ANDROID
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY], "Dead Spot", NEWMENU_ITEM_TYPE_HEADER, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
		pos++;
		add_newmenu_child_variable(nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY], nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY]->vtext[0], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY], "<", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, -t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "00") / 2 - 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_dead_spot_left, NULL, NULL);
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY], ">", NEWMENU_ITEM_TYPE_VOPTION_PLATFORM, t3f_get_text_width(vgolf_font[VGOLF_FONT_48], "00") / 2 + 32, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * pos, &vgolf_font[VGOLF_FONT_48], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, config_dead_spot_right, NULL, NULL);
		pos++;
	#endif
	newmenu_centre(nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_OPTIONS] = create_newmenu("Options", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS], "Options", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS], "Video", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_OPTIONS_VIDEO], NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS], "Audio", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_OPTIONS_AUDIO], NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS], "Online", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_OPTIONS_ONLINE], NULL, NULL, NULL);
	#ifdef T3F_ANDROID
		add_newmenu_item(nmenu[VGOLF_MENU_OPTIONS], "Gameplay", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 3, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_OPTIONS_GAMEPLAY], NULL, NULL, NULL);
	#endif
	newmenu_centre(nmenu[VGOLF_MENU_OPTIONS], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_HOST_NAME] = create_newmenu("Host Name", host_name_update, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_HOST_NAME], "Host Name", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]), &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_HOST_NAME], "Begin Hosting", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_host_game_name, NULL, NULL);
	nmenu[VGOLF_MENU_HOST_NAME]->item[nmenu[VGOLF_MENU_HOST_NAME]->items - 1].vtext = vgolf_host_name;
//	add_newmenu_child_variable(nmenu[VGOLF_MENU_HOST_NAME], nmenu[VGOLF_MENU_HOST_NAME]->vtext[0], 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_MAIN_DARK);
	add_newmenu_item(nmenu[VGOLF_MENU_HOST_NAME], "Begin Hosting", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_begin_hosting, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_HOST_NAME], "Cancel", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2 + t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, NULL, VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_cancel_hosting, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_HOST_NAME], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_PLAY_ONLINE] = create_newmenu("Play Online", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_PLAY_ONLINE], "Play Online", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_PLAY_ONLINE], "Host Game", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_PLAY_ONLINE_HOST], vgolf_menu_proc_host_game, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_PLAY_ONLINE], "Join Game", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_join_game, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_PLAY_ONLINE], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_MAIN_HOST] = create_newmenu("Main Menu", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN_HOST], "Play Game", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, player_setup_proc, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN_HOST], "Close Server", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_close_host, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_MAIN_HOST], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_MAIN_JOIN] = create_newmenu("Main Menu", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN_JOIN], "Play Game", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, player_setup_proc, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN_JOIN], "Disconnect", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_disconnect, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_MAIN_JOIN], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_PLAY] = create_newmenu("Play", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_PLAY], "Game Type", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_PLAY], "Local", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, player_setup_proc, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_PLAY], "Online", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_PLAY_ONLINE], NULL, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_PLAY], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	nmenu[VGOLF_MENU_DLC] = create_newmenu("DLC", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_DLC], "DLC Type", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_DLC], "Avatars", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_dlc_avatars, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_DLC], "Courses", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_dlc_courses, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_DLC], "Commentaries", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, vgolf_menu_proc_dlc_commentaries, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_DLC], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	pos = 0;
	nmenu[VGOLF_MENU_MAIN] = create_newmenu("Main Menu", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN], "Play Game", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_PLAY], NULL, NULL, NULL);
	pos++;
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN], "Profiles", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, profile_view_proc, NULL, NULL);
	pos++;
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN], "Leaderboards", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, leaderboards_view_proc, NULL, NULL);
	pos++;
	if(!vgolf_demo)
	{
		add_newmenu_item(nmenu[VGOLF_MENU_MAIN], "Browse DLC", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_DLC], NULL, NULL, NULL);
		pos++;
	}
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN], "Options", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_OPTIONS], NULL, NULL, NULL);
	pos++;
	add_newmenu_item(nmenu[VGOLF_MENU_MAIN], "Credits", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, credits_proc, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_MAIN], VGOLF_MENU_MIN_Y, VGOLF_MENU_MAX_Y);

	pos = 0;
	nmenu[VGOLF_MENU_TITLE] = create_newmenu("Title Menu", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_TITLE], "Start", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_MAIN], NULL, NULL, NULL);
	pos++;
	if(vgolf_demo)
	{
		add_newmenu_item(nmenu[VGOLF_MENU_TITLE], "Purchase", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, menu_purchase, NULL, NULL);
		pos++;
	}
	add_newmenu_item(nmenu[VGOLF_MENU_TITLE], "Privacy Info", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, menu_privacy, NULL, NULL);
	pos++;
	add_newmenu_item(nmenu[VGOLF_MENU_TITLE], "Quit", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * pos, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, menu_quit, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_TITLE], 240, 462);

	/* in-game menus */
	nmenu[VGOLF_MENU_UPLOAD_FAIL] = create_newmenu("Retry Upload", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_UPLOAD_FAIL], "Failed to Upload Scores", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_UPLOAD_FAIL], "Retry", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, menu_proc_upload_retry, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_UPLOAD_FAIL], "Cancel", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, menu_proc_upload_cancel, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_UPLOAD_FAIL], 0, 480);

	nmenu[VGOLF_MENU_EXIT_GAME] = create_newmenu("Exit Game", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_EXIT_GAME], "Exit Game?", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_EXIT_GAME], "Yes", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, exit_game_yes, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_EXIT_GAME], "No", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, exit_game_no, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_EXIT_GAME], 0, 480);

	/* create in-game options menus */
	nmenu[VGOLF_MENU_GOPTIONS_AUDIO] = duplicate_newmenu(nmenu[VGOLF_MENU_OPTIONS_AUDIO]);
	newmenu_centre(nmenu[VGOLF_MENU_GOPTIONS_AUDIO], 0, 480);
	nmenu[VGOLF_MENU_GOPTIONS_VIDEO] = duplicate_newmenu(nmenu[VGOLF_MENU_OPTIONS_VIDEO]);
	newmenu_centre(nmenu[VGOLF_MENU_GOPTIONS_VIDEO], 0, 480);
	nmenu[VGOLF_MENU_GOPTIONS_ONLINE] = duplicate_newmenu(nmenu[VGOLF_MENU_OPTIONS_ONLINE]);
	newmenu_centre(nmenu[VGOLF_MENU_GOPTIONS_ONLINE], 0, 480);
	nmenu[VGOLF_MENU_GOPTIONS] = duplicate_newmenu(nmenu[VGOLF_MENU_OPTIONS]);
	newmenu_centre(nmenu[VGOLF_MENU_GOPTIONS], 0, 480);

	nmenu[VGOLF_MENU_IN_GAME] = create_newmenu("In-Game Options", NULL, 0, 0);
	add_newmenu_item(nmenu[VGOLF_MENU_IN_GAME], "In-Game Menu", NEWMENU_ITEM_TYPE_HEADER, 0, -t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_OPTION, NULL, VGOLF_COLOR_MENU_OPTION, NULL, NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_IN_GAME], "Resume Game", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 0, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, NULL, in_game_resume, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_IN_GAME], "Options", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 1, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_GOPTIONS], NULL, NULL, NULL);
	add_newmenu_item(nmenu[VGOLF_MENU_IN_GAME], "Quit", NEWMENU_ITEM_TYPE_OPTION, 0, t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) * 2, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN_DARK, &vgolf_font[VGOLF_FONT_96], VGOLF_COLOR_MENU_MAIN, nmenu[VGOLF_MENU_EXIT_GAME], NULL, NULL, NULL);
	newmenu_centre(nmenu[VGOLF_MENU_IN_GAME], 0, 480);

}

void vgolf_destroy_menus(void)
{
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		hyperlink_page_destroy(player_page[i]);
		hyperlink_page_destroy(player_network_page[i]);
		hyperlink_page_destroy(player_start_page[i]);
	}
	hyperlink_page_destroy(player_add_page);
	hyperlink_page_destroy(player_global_page);
	hyperlink_page_destroy(player_global_slave_page);
	hyperlink_page_destroy(course_global_page);
	hyperlink_page_destroy(course_global_slave_page);
	hyperlink_page_destroy(sub_menu_page);
	hyperlink_page_destroy(scoreboard_page);
	hyperlink_page_destroy(profile_global_page);
	hyperlink_page_destroy(leaderboard_global_page);
	hyperlink_page_destroy(profile_page);
	hyperlink_page_destroy(player_account_page);
	hyperlink_page_destroy(game_complete_page);
	hyperlink_page_destroy(fake_game_complete_page);
	hyperlink_page_destroy(player_new_account_page);
	hyperlink_page_destroy(ball_drop_page);
	hyperlink_page_destroy(fake_ball_drop_page);
	hyperlink_page_destroy(take_gimme_page);
	hyperlink_page_destroy(fake_take_gimme_page);
	hyperlink_page_destroy(course_page);
	hyperlink_page_destroy(course_select_page);
	hyperlink_page_destroy(chat_page);

	for(i = 0; i < 256; i++)
	{
		if(nmenu[i])
		{
			destroy_newmenu(nmenu[i]);
		}
	}
}
