#include "../t3f/t3f.h"
#include "../t3f/sound.h"
#include "../t3f/music.h"
#include "../joynet/joynet.h"
#include "../joynet/serialize.h"
#include "../version.h"
#include "../globals.h"
#include "../newmenu.h"
#include "../menu_proc.h"
#include "../init.h"
#include "client.h"
#include "../gameplay.h"
#include "../replay.h"

void vgolf_message_callback(char * message, int option)
{
	ncdmsg_add(&vgolf_messages, message, al_map_rgb(255, 255, 255), 0.0, option);
}

int vgolf_game_channel_callback(JOYNET_MESSAGE * mp)
{
	switch(mp->type)
	{
		case JOYNET_GAME_MESSAGE_CHECK_ID:
		{
			joynet_watch_game(vgolf_client_game);
			break;
		}
		case JOYNET_GAME_MESSAGE_CHECK_ID_FAIL:
		{
			short len;
			char message[256] = {0};
			char buffer[1024] = {0};

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getw(vgolf_client_game->serial_data, &len);
			joynet_read(vgolf_client_game->serial_data, message, len);
			sprintf(buffer, "Server and client are not compatible.\nServer version: %s\nClient version: %s", message, VGOLF_SERVER_VERSION_STRING);
			ncdmsg_add(&vgolf_messages, buffer, al_map_rgb(255, 255, 255), 16.0, 300);
			enet_peer_disconnect(vgolf_client->peer, JOYNET_DISCONNECT_CLIENT_CLOSED);
			vgolf_client_disconnected = true;
			current_newmenu = nmenu[VGOLF_MENU_MAIN];
			break;
		}
		case JOYNET_GAME_MESSAGE_CONNECT:
		{
			short controller;
			short port;

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getw(vgolf_client_game->serial_data, &controller);
			joynet_getw(vgolf_client_game->serial_data, &port);

/*			strcpy(vgolf_client_game->player[port]->name, "Guest");
			if(vgolf_client_game->player[port]->local)
			{
				joynet_update_player_options(vgolf_client_game, port);
			} */
/*			if(!vgolf_player_list[port])
			{
				vgolf_player_list[port] = 1;
				game_player[port].profile = 0;
				game_player[port].avatar = 0;
				player_page_pointer[port] = player_page[port];
				player_page_pointer[port]->element[1].data = vgolf_avatars.avatar[0]->image[0];
				sprintf(player_page_text_buffer[port], "%s", vgolf_profiles.item[game_player[port].profile].name);
			} */

//			vgolf_player[port].controller = port;
//			vgolf_player[port].playing = true;
//			vgolf_player[port].profile_read = false;
			break;
		}
		case JOYNET_GAME_MESSAGE_ADD_PLAYER:
		{
			short player;

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getw(vgolf_client_game->serial_data, &player);

			if(!vgolf_player_list[player])
			{
				vgolf_player_list[player] = 1;
				game_player[player].savatar = vgolf_avatars.sum[0];
				if(vgolf_client_game->player[player]->local)
				{
					player_page_pointer[player] = player_page[player];
					if(vgolf_client)
					{
						strcpy(vgolf_client_game->player[player]->name, vgolf_nick);
					}
					else
					{
						strcpy(vgolf_client_game->player[player]->name, "Guest");
					}
					strcpy(player_page_pointer[player]->element[0].data, vgolf_client_game->player[player]->name);
					game_player[player].setup_step = VGOLF_PLAYER_SETUP_START;
					joynet_update_player_options(vgolf_client_game, player);
				}
				else
				{
					player_page_pointer[player] = player_network_page[player];
					game_player[player].profile = 0; // non-local players should have profile set to 0
				}
				if(game_player[player].avatar)
				{
					vgolf_destroy_avatar(game_player[player].avatar);
				}
				game_player[player].avatar = vgolf_load_avatar(vgolf_avatars.fn[0], NULL, vgolf_avatars.fi[0]);
				player_page[player]->element[1].data2 = (void **)(&game_player[player].avatar->image[0]);
				player_network_page[player]->element[1].data2 = (void **)(&game_player[player].avatar->image[0]);
//				player_page_pointer[player]->element[1].data2 = (void **)(&game_player[player].avatar->image[0]);
				hl_player_name(player, player_page_pointer[player]);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_REMOVE_PLAYER:
		{
			short player;

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getw(vgolf_client_game->serial_data, &player);

			/* if we removed a local player, get out of the game and inform the
			 * player */
			if(vgolf_state == VGOLF_STATE_PLAY && vgolf_client_game->player[player]->local)
			{
				end_game_helper();
				ncdmsg_add(&vgolf_messages, "You have been removed from the game.", al_map_rgb(255, 255, 255), 0.0, 300);
			}

			/* mark player as being removed here, will actually remove player elsewhere */
			else
			{
				if(vgolf_state == VGOLF_STATE_PLAY)
				{
					if(vgolf_replay_mode != VGOLF_REPLAY_MODE_PLAY)
					{
						vgolf_remove_player[player] = 1;
						vgolf_remove_player_frame[player] = vgolf_client_game->input_buffer->filled_frames; // remove player when we get to this frame
					}
				}
				else
				{
					vgolf_player_list[player] = 0;
					player_page_pointer[player] = player_start_page[player];
				}
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_UPDATE_OPTION:
		{
			break;
		}
		case JOYNET_GAME_MESSAGE_UPDATE_PLAYER_OPTIONS:
		{
			short player;

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getw(vgolf_client_game->serial_data, &player);

			strcpy(player_page_pointer[player]->element[0].data, vgolf_client_game->player[player]->name);
			break;
		}
		case JOYNET_GAME_MESSAGE_SELECT_PLAYER_CONTENT:
		{
			short player;
			short list;
			int a;

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getw(vgolf_client_game->serial_data, &player);
			joynet_getw(vgolf_client_game->serial_data, &list);
			if(list == VGOLF_CONTENT_AVATARS)
			{
				game_player[player].savatar = vgolf_client_game->player[player]->selected_content[VGOLF_CONTENT_AVATARS];
				a = vgolf_find_avatar(game_player[player].savatar);
				if(a >= 0)
				{
					if(game_player[player].avatar)
					{
						vgolf_destroy_avatar(game_player[player].avatar);
					}
					game_player[player].avatar = vgolf_load_avatar(vgolf_avatars.fn[a], NULL, vgolf_avatars.fi[a]);
					player_page[player]->element[1].data2 = (void **)(&game_player[player].avatar->image[0]);
					player_network_page[player]->element[1].data2 = (void **)(&game_player[player].avatar->image[0]);
				}
				if(vgolf_client_game->player[player]->local)
				{
					vgolf_state = VGOLF_STATE_PLAYER_SETUP;
					if(game_player[player].profile > 0)
					{
						vgolf_profiles.item[game_player[player].profile].avatar = game_player[player].savatar;
					}
				}
			}
			else if(list == VGOLF_CONTENT_COURSES)
			{
				game_player[player].scourse = vgolf_client_game->player[player]->selected_content[VGOLF_CONTENT_COURSES];
				a = vgolf_find_course(game_player[player].scourse);
				if(a >= 0)
				{
					if(game_player[player].course_preview)
					{
						t3f_destroy_animation(game_player[player].course_preview);
					}
					game_player[player].course_preview = vgolf_load_course_preview(vgolf_courses.item[a].filename, vgolf_courses.item[a].fi);
				}
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_START:
		{
			int i;

			vgolf_spawn_client_keep_alive_thread();
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				if(game_player[i].course_preview)
				{
					t3f_destroy_animation(game_player[i].course_preview);
					game_player[i].course_preview = NULL;
				}
			}
			reset_letters();
			destroy_main_image_bank();
			if(!vgolf_start_game())
			{
				printf("Failed to start game!");
				return 1;
			}
			vgolf_kill_client_keep_alive_thread();
			break;
		}
		case JOYNET_GAME_MESSAGE_PAUSE:
		{
			vgolf_pause_game();
			break;
		}
		case JOYNET_GAME_MESSAGE_RESUME:
		{
			vgolf_resume_game();
			break;
		}
		case JOYNET_GAME_MESSAGE_END:
		{
			end_game_helper(); // joynet is crashing when we try to process input after the game has ended
			break;
		}
		case JOYNET_GAME_MESSAGE_SELECT_PLAYER:
		{
			if(vgolf_replay_mode != VGOLF_REPLAY_MODE_PLAY)
			{
//				printf("select player %d\n", vgolf_client_game->input_buffer->filled_frames);
				vgolf_passed_turn_frame = vgolf_client_game->input_buffer->filled_frames; // tell game that turn has passed when it has reached this frame
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_INPUT:
		{
//			printf("input received\n");
			break;
		}
		case VGOLF_GAME_MESSAGE_TYPING:
		{
/*			char player = 0;
			char players = 0;
			int i;

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getc(vgolf_client_game->serial_data, &players);
			for(i = 0; i < players; i++)
			{
				joynet_getc(vgolf_client_game->serial_data, &player);
				vgolf_player[(int)player].flags |= PP2_PLAYER_FLAG_TYPING;
			} */
			break;
		}
		case VGOLF_GAME_MESSAGE_TYPING_DONE:
		{
/*			char player = 0;
			char players = 0;
			int i;

			joynet_serialize(vgolf_client_game->serial_data, mp->data);
			joynet_getc(vgolf_client_game->serial_data, &players);
			for(i = 0; i < players; i++)
			{
				joynet_getc(vgolf_client_game->serial_data, &player);
				vgolf_player[(int)player].flags = vgolf_player[i].flags & ~PP2_PLAYER_FLAG_TYPING;
			} */
			break;
		}

		/* this message is sent when the master wants to start the game and some
		 * players haven't finished selecting */
		case VGOLF_GAME_MESSAGE_WANT_START:
		{
			vgolf_start_game_timer = 60 * 10; // give players 10 seconds
			break;
		}

		/* if master cancels or some other situation comes up where the countdown
		 * needs to be stopped */
		case VGOLF_GAME_MESSAGE_CANCEL_START:
		{
			vgolf_start_game_timer = 0;
			break;
		}

		/* the master has started the game, should show "today's matchup" screen */
		case VGOLF_GAME_MESSAGE_STARTING:
		{
			vgolf_state = VGOLF_STATE_STARTING_GAME;
			break;
		}
	}
	return 0;
}

int vgolf_chat_callback(char * user, char * message)
{
	char bmessage[1024] = {0};
	char buser[256] = {0};
	float tab = 0.0;

	t3f_play_sample(main_game_vars.sound_bank.message_sound, 1.0, 0.0, 1.0);
	sprintf(buser, "%s: ", user);
	tab = t3f_get_text_width(vgolf_font[VGOLF_FONT_32], buser);
	sprintf(bmessage, "%s%s", buser, message);
	ncdmsg_add(&vgolf_messages, bmessage, al_map_rgb(255, 255, 0), tab, 0);
	if(vgolf_chat_state == 1 || vgolf_chat_state == 2)
	{
		vgolf_chat_new_message = true;
	}
	vgolf_log_chat(buser, message);

	return 0;
}
