#include "t3f/t3f.h"
#include "globals.h"
#include "profile.h"

int profile_mode = 0;
int profile_selected = 0;
int profile_mode_selected = -1;

int load_profiles(PROFILE_LIST * lp, char * fn)
{
	const ALLEGRO_FILE_INTERFACE * old_interface;
	ALLEGRO_FILE * fp;
	char header[4] = {'V', 'P', 'D', 1};
	char rheader[4];
	unsigned long cid = 0;
	char cused[VGOLF_MAX_COURSES] = {0};
	int i, j, k;
	int ci = -1;
	int tc = 0;
	int ret = 0;
	
	old_interface = al_get_new_file_interface();
	al_set_standard_file_interface();

	fp = t3f_open_file(t3f_data_path, fn, "rb");
	if(fp)
	{
		al_fread(fp, rheader, 4);
		if(!memcmp(header, rheader, 3) && rheader[3] != 0)
		{
			memset(lp, 0, sizeof(PROFILE_LIST)); // zero everything out so we don't get uninitialized values
			lp->items = al_fread32le(fp);
			for(i = 0; i < lp->items; i++)
			{
				al_fread(fp, lp->item[i].name, 128);
				lp->item[i].avatar = al_fread32le(fp);
				tc = al_fread32le(fp);
				memset(cused, 0, VGOLF_MAX_COURSES);
				for(j = 0; j < tc; j++)
				{
					
					/* read course id and find which course this entry points to */
					cid = al_fread32le(fp);
					ci = -1;
					for(k = 0; k < vgolf_courses.items; k++)
					{
						if(cid == vgolf_courses.item[k].sum)
						{
							ci = k;
							break;
						}
					}
					
					/* if the course id is found, load this data into correct spot */
					if(ci >= 0 && !cused[ci])
					{
						lp->item[i].course_info[ci].id = cid;
						for(k = 0; k < 18; k++)
						{
							lp->item[i].course_info[ci].score[k] = al_fread32le(fp);
						}
						lp->item[i].course_info[ci].total_score = al_fread32le(fp);
						lp->item[i].course_info[ci].won = al_fread32le(fp);
						lp->item[i].course_info[ci].lost = al_fread32le(fp);
						lp->item[i].course_info[ci].completed = al_fread32le(fp);
						cused[ci] = 1;
					}
					
					/* otherwise skip data */
					else
					{
						for(k = 0; k < 18; k++)
						{
							al_fread32le(fp);
						}
						al_fread32le(fp);
						al_fread32le(fp);
						al_fread32le(fp);
						al_fread32le(fp);
					}
				}
				for(j = 0; j < VGOLF_MAX_PROFILE_DATA; j++)
				{
					lp->item[i].total.data[j] = al_fread32le(fp);
				}
				lp->item[i].total.time = al_fread32le(fp);
			}
			al_fclose(fp);
			ret = 1;
		}
	}

	al_set_new_file_interface(old_interface);

	return ret;
}

int save_profiles(PROFILE_LIST * lp, char * fn)
{
	const ALLEGRO_FILE_INTERFACE * old_interface;
	ALLEGRO_FILE * fp;
	char header[4] = {'V', 'P', 'D', 1};
	int i, j, k;
	int ret = 0;
	
	old_interface = al_get_new_file_interface();
	al_set_standard_file_interface();

	fp = t3f_open_file(t3f_data_path, fn, "wb");
	if(fp)
	{
		al_fwrite(fp, header, 4);
		
		al_fwrite32le(fp, lp->items);
		for(i = 0; i < lp->items; i++)
		{
			al_fwrite(fp, lp->item[i].name, 128);
			al_fwrite32le(fp, lp->item[i].avatar);
			al_fwrite32le(fp, vgolf_courses.items);
			for(j = 0; j < vgolf_courses.items; j++)
			{
				al_fwrite32le(fp, vgolf_courses.item[j].sum);
				for(k = 0; k < 18; k++)
				{
					al_fwrite32le(fp, lp->item[i].course_info[j].score[k]);
				}
				al_fwrite32le(fp, lp->item[i].course_info[j].total_score);
				al_fwrite32le(fp, lp->item[i].course_info[j].won);
				al_fwrite32le(fp, lp->item[i].course_info[j].lost);
				al_fwrite32le(fp, lp->item[i].course_info[j].completed);
			}
			for(j = 0; j < VGOLF_MAX_PROFILE_DATA; j++)
			{
				al_fwrite32le(fp, lp->item[i].total.data[j]);
			}
			al_fwrite32le(fp, lp->item[i].total.time);
		}
		al_fclose(fp);
		ret = 1;
	}
	al_set_new_file_interface(old_interface);
	return ret;
}

void clear_profiles(PROFILE_LIST * lp)
{
	lp->items = 0;
}
